#!/usr/bin/env python

import os
import collections
import argparse
import numpy
import spglib
import math

import structure.crystal
import structure.spg

def csstd():
    #structure.crystal.csa()
    #structure.crystal.csa2()

    arg = argparse.ArgumentParser(description="ac-csa")
    arg.add_argument("-c","--cif",action="store",dest="cif")
    arg.add_argument("-o","--output",action="store",dest="fout")
    #arg.add_argument("--ocif",action="store",dest="foutcif")
    arg.add_argument("--tol",action="store",dest="tol",default="1.0e-5")
    args = arg.parse_args()
    cry = structure.crystal.Crystal()
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    print("ver.",cry.version)
    print()
    print("[crystal structure analysis]")

    matid = ""
    spgnum_cif = 0

    #tol = 1.0e-5
    tol = float(args.tol)

    if args.cif != None:
        print("cif:",args.cif)
        cry.read_cif(args.cif)
        cry.check_cif()

        cry.construct_structure()

        print("[cif]")

        print(cry.space_group_crystal_system)
        print(cry.symmetry_Int_Tables_number)
        print(cry.symmetry_space_group_name)

        print(cry.cell.length_a,cry.cell.length_b,cry.cell.length_c)
        print(cry.cell.angle_alpha,cry.cell.angle_beta,cry.cell.angle_gamma)

        matid = cry.matid
        spgnum_cif = cry.symmetry_Int_Tables_number

        atoms = []
        for e in range(len(cry.velement_name)):
            atoms.append(cry.velement_name[e])

        print("[csa]")
        #b1,hallnum_ccell,hallnum_pcell = structure.spg.csa_check(cry,tol)
        #print("csa:",b1)

        convcell = numpy.array(cry.unit_conv,float)
        convatom = []  # pos
        for a in cry.vatom_conv:
            convatom.append([a.position[0],a.position[1],a.position[2]])
        convnum = []
        for i in range(len(cry.velement_size_conv)):
            for j in range(cry.velement_size_conv[i]):
                convnum.append(i)

        ccell = (convcell,convatom,convnum)

        print( "[conventional cell (csa)]")
        print_cell(ccell)

        dataset = spglib.get_symmetry_dataset(ccell,tol)
        spgnum_ccell = dataset["number"]
        hallnum_ccell = dataset["hall_number"]
        print_dataset(dataset)

        #symmetry = spglib.get_symmetry(ccell)
        #print(symmetry)

        # temporary
        cry.symmetry_space_group_name_hall = dataset["hall"]

        print("*** standardization ***")
        scell = spglib.standardize_cell(ccell)

        print( "[conventional cell (std)]")
        print_cell(scell)

        dataset = spglib.get_symmetry_dataset(scell,tol)
        spgnum_scell = dataset["number"]
        hallnum_scell = dataset["hall_number"]
        print_dataset(dataset)

        # temporary
        cry.symmetry_space_group_name_hall = dataset["hall"]

        cifdata = make_cifdata(scell,atoms,tol)

        if args.fout != None:
            print("output cif (std):",args.fout)
            #cry.write_cif(args.fout)
            write_cif_file(args.fout, cifdata)

def make_cifdata(cell, atoms, tol=1.0e-5):
    cifdata = []

    scell = spglib.standardize_cell(cell)
    #print("*** scell ***")
    #print_cell(scell)
    dataset = spglib.get_symmetry_dataset(scell, tol)
    symmetry = spglib.get_symmetry(scell)
    
    wyckoffs = dataset["wyckoffs"]
    equiv_atoms = dataset["equivalent_atoms"]
    trans_matrix = dataset["transformation_matrix"]
    origin_shift = dataset["origin_shift"]
    hall_number = dataset["hall_number"]
    hall_symbol = dataset["hall"]
    sg_type = spglib.get_spacegroup_type(hall_number)
    #print("trans_matrix")
    #print(trans_matrix)
    #print("origin_shift")
    #print(origin_shift)

    lattice = scell[0]
    positions = scell[1]
    numbers = scell[2]
    # for n, pos in zip(numbers, positions):
    #     print(n, pos)
    
    #f.write("data_{}\n".format(os.path.splitext(os.path.basename(fname))[0]))
    #f.write("data_%s\n" % os.path.splitext(os.path.basename(fname))[0])
    cifdata.append(("data_",""))
    cifdata.append(("", None))

    chemform = ""
    num = [0 for i in range(len(atoms))]
    for i,a in enumerate(atoms):
        for n in numbers:
            if i==n:
                num[i] += 1
    if "C" in atoms:
        chemform += "C"+str(num[atoms.index("C")])+" "
    if "H" in atoms:
        chemform += "H"+str(num[atoms.index("H")])+" "
    for i,a in enumerate(atoms):
        if a != "C" and a != "H":
            chemform += a+str(num[i])+" "
    chemform = chemform.strip()
    print("chemical_formula",chemform)

    cifdata.append(("_chemical_formula_sum",chemform))
    crystal_system = get_crystal_system(dataset["number"])
    cifdata.append(("_space_group_crystal_system", crystal_system))

    # cifdata.append(("_symmetry_space_group_name_H-M", sg_type["international_full"]))
    name_HM = sanitize_name_HM(sg_type["international_short"])
    cifdata.append(("_symmetry_space_group_name_H-M", name_HM))
    cifdata.append(("_symmetry_Int_Tables_number", dataset["number"]))
    cifdata.append(("_symmetry_space_group_name_Hall", hall_symbol))
    cifdata.append(("", None))

    cifdata_symmetry_list = ("loop_symmetry", [])
    cifdata_symmetry_list[1].append(["_symmetry_equiv_pos_site_id", "_symmetry_equiv_pos_as_xyz"])

    ind = 0
    for rot,trans in zip(dataset['rotations'], dataset['translations']):
        symbols = make_symop(rot, trans)
        ind += 1
        cifdata_symmetry_list[1].append([ind, symbols])
        # print("rotation")
        # print(rot)
        # print("translation")
        # print(trans)
        # print(symbols)

    cifdata.append(cifdata_symmetry_list)
    cifdata.append(("", None))

    a,b,c = celllength(lattice)
    alpha,beta,gamma = cellangle(lattice)
    cifdata.append(("_cell_length_a", a))
    cifdata.append(("_cell_length_b", b))
    cifdata.append(("_cell_length_c", c))
    cifdata.append(("_cell_angle_alpha", alpha))
    cifdata.append(("_cell_angle_beta", beta))
    cifdata.append(("_cell_angle_gamma", gamma))
    cifdata.append(("", None))

    # find inequivalent atoms and calculate their multiplicitis
    inequiv_atoms = collections.OrderedDict()
    for num in equiv_atoms:
        if num not in inequiv_atoms:
            inequiv_atoms[num] = 1
        else:
            inequiv_atoms[num] += 1

    sg = dataset["international"].strip()

    #print("space group number {}".format(dataset["number"]))
    #print("space group {}".format(dataset["international"]))
    #print("hall symbol {}".format(dataset["hall"]))
    #print("space group number %s" % dataset["number"])
    #print("space group %s" % dataset["international"])
    #print("hall symbol %s" % dataset["hall"])
          
    print("atom_")
    print("label name wyckoff multiplicity x y z")

    cifdata_atomlist = ("loop_atom", [])
    cifdata_atomlist[1].append(["_atom_site_label",  "_atom_site_fract_x", "_atom_site_fract_y", "_atom_site_fract_z",
                                "_atom_site_occupancy", "_atom_site_symmetry_multiplicity", "_atom_site_Wyckoff_symbol",
                                "_atom_site_type_symbol"])
    suffix_list = {}
    for atom in atoms:
        suffix_list[atom] = 1
    for num in inequiv_atoms:
        atom_type_number = numbers[num]
        atom_type = atoms[atom_type_number]
        atom_label = atom_type + str(suffix_list[atom_type])
        suffix_list[atom_type] += 1
        cifdata_atomlist[1].append([atom_label, positions[num][0], positions[num][1], positions[num][2],
                                    1.0, inequiv_atoms[num], wyckoffs[num], atom_type])
        print(atom_label, atom_type, wyckoffs[num], inequiv_atoms[num], positions[num][0], positions[num][1], positions[num][2])

    cifdata.append(cifdata_atomlist)
    cifdata.append(("", None))

    return cifdata

def write_cif_file(fname, cifdata):
    ''' write cif data to file
        fname: output filename
        cifdata:  cif data as a list of tuple (key, val)
                  key corresponds to a cif key.
                  if key is not loop_, print "key val"
                  if key is loop_, val[0] is a list of identifier in loop_ and val[1:] contains data.
    '''
    with open(fname, "w") as f:

        for key,val in cifdata:
            if key == "data_":
                f.write("%s%s\n" % (key,val));
            elif key == "_chemical_formula_sum":
                f.write("%s '%s'\n" % (key,val));
            elif key == "_space_group_crystal_system":
                f.write("%s %s\n" % (key,val));
            elif key == "_symmetry_space_group_name_H-M":
                f.write("%s '%s'\n" % (key,val));
            elif key == "_symmetry_Int_Tables_number":
                f.write("%s %s\n" % (key,val));
            elif key == "_symmetry_space_group_name_Hall":
                f.write("%s '%s'\n" % (key,val));

            elif key == "loop_symmetry":
                f.write("loop_\n")
                loop_keys = val[0]
                loop_vals = val[1:]
                for lk in loop_keys:
                    #f.write("    %s\n" % lk)
                    f.write("%s\n" % lk)
                for lv in loop_vals:
                    #f.write(" %d '%s'\n" % (lv[0],lv[1]))
                    f.write("%d '%s'\n" % (lv[0],lv[1]))

            elif key.find("_cell_length")>=0:
                f.write("%s %15.9f\n" % (key,val));
            elif key.find("_cell_angle")>=0:
                f.write("%s %15.9f\n" % (key,val));

            elif key == "loop_atom":
                f.write("loop_\n")
                loop_keys = val[0]
                loop_vals = val[1:]
                for lk in loop_keys:
                    #f.write("    %s\n" % lk)
                    f.write("%s\n" % lk)
                for lv in loop_vals:
                    #f.write(" %s %15.9f %15.9f %15.9f %f %s %s %s\n" % \
                    f.write("%s %15.9f %15.9f %15.9f %f %s %s %s\n" % \
                                (lv[0],lv[1],lv[2],lv[3],lv[4],lv[5],lv[6],lv[7]))

            elif key == "":
                f.write("\n")

            else:
                f.write("%s %s" % (key, val))


def print_cell(cell):
    lattice, positions, numbers = cell
    print("lattice vectors")
    for vec in lattice:
        #print("{:15.9f} {:15.9f} {:15.9f}".format(vec[0], vec[1], vec[2]))
        print("%15.9f %15.9f %15.9f" % (vec[0], vec[1], vec[2]))
    print("atomic postions and type")
    for coor, atype in zip(positions, numbers):
        #print("{:4d} {:15.9f} {:15.9f} {:15.9f}".format(atype, coor[0], coor[1], coor[2]))
        print("%4d %15.9f %15.9f %15.9f" % (atype, coor[0], coor[1], coor[2]))

def print_dataset(dataset):
    print("number",dataset["number"])
    print("international",dataset["international"])
    print("hall_symbol",dataset["hall"])
    print("hall_number",dataset["hall_number"])
    print("main_axis_setting",dataset["choice"])
    print("pointgroup",dataset["pointgroup"])
    hall_number = dataset["hall_number"]
    sg_type = spglib.get_spacegroup_type(hall_number)
    print_spgtype(sg_type)
    #print(dataset["wyckoffs"])
    #print(dataset["equivalent_atoms"])
    print("trans_matrix\n",dataset["transformation_matrix"])
    print("original_shift\n",dataset["origin_shift"])
    #print(dataset["rotations"])
    #print(dataset["translations"])
    #print(dataset["std_lattice"])
    #print(dataset["std_positions"])
    #print(dataset["std_types"])

def print_spgtype(sgtype):
    print("number",sgtype["number"])
    print("international_short",sgtype["international_short"])
    print("international_full",sgtype["international_full"])
    print("international",sgtype["international"])
    print("hall_sysmbol",sgtype["hall_symbol"])
    print("main_axis_setting",sgtype["choice"])
    #print(sgtype["schoenflies"])
    print("pointgroup",sgtype["pointgroup_schoenflies"])
    #print(sgtype["pointgroup_international"])
    #print(sgtype["arithmetic_crystal_class_number"])
    #print(sgtype["arithmetic_crystal_class_symbol"])

def sanitize_name_HM(name):
    work = []
    cur = ""
    length = len(name)
    i = 0
    while i < length:
        if name[i] not in "_/":
            if cur:
                work.append(cur)
            if name[i] == "-":
                cur = name[i:i+2]
                i += 2
            else:
                cur = name[i]
                i += 1
        elif name[i] == "_":
            cur += name[i+1]
            i += 2
        elif name[i] == "/":
            cur += name[i:i+2]
            i += 2
    if cur:
        work.append(cur)
    return " ".join(work)

def get_crystal_system(it_number):
    '''retrun crystal system from IT number'''
    if it_number <= 0:
        return "error"
    elif it_number < 3:
        return "triclinic"
    elif it_number < 16:
        return "monoclinic"
    elif it_number < 75:
        return "orthorhombic"
    elif it_number < 143:
        return "tetragonal"
    elif it_number < 168:
        return "trigonal"
    elif it_number < 195:
        return "hexagonal"
    elif it_number < 231:
        return "cubic"
    else:
        return "error"

def make_symop(rot, trans):
    '''make symop symbols from rotation matrix and translation vectors
       rot: rotational matrix in numpy 3x3 array
       trans: translational vector
       return string 
    '''
    symbols = []
    for i in range(3):
        symop = ""
        if rot[i,0] == 1:
            symop += "x"
        elif rot[i,0] == -1:
            symop += "-x"
        elif rot[i,0] == 0:
            pass
        else:
            print("error in rotation matrix")
        if rot[i,1] == 1:
            if symop:
                symop += "+y"
            else:
                symop += "y"
        elif rot[i,1] == -1:
            symop += "-y"
        elif rot[i,1] == 0:
            pass
        else:
            print("error in rotation matrix")
        if rot[i,2] == 1:
            if symop:
                symop += "+z"
            else:
                symop += "z"
        elif rot[i,2] == -1:
            symop += "-z"
        elif rot[i,2] == 0:
            pass
        else:
            print("error in rotation matrix")
        (shift, dummy) = math.modf(trans[i])
        if shift < 0:
            shift += 1.0
        if abs(shift) > 1.0e-4 and abs(shift-1.0) > 1.0e-4:
            if shift > 0:
                symop += "+"
            # symop += str(trans[i])
            symop += float2rational(trans[i])
        symbols.append(symop)
    return ",".join(symbols)

def roundint(x):
    return int(math.floor(x+0.5))

def float2rational(x):
    '''make rational representation of x
       assume 0 < x < 1
       Denominator is assumed to be 2,3, or 4, 6.
    '''
    tol = 1.0e-3
    denoms = [2, 3, 4, 6]
    for d in denoms:
        w = x * d
        n = roundint(w)
        # print(x, d, w, n, w-n)
        if abs(w-n) < tol:
            #s = "{:d}/{:d}".format(n, d)
            s = "%d/%d" % (n, d)
            # print("float2rational:", x, s)
            return s
    sys.stderr.write("can't convert to rational {}\n".format(x))
    sys.exit(1)

def celllength(cell):
    ''' calculate lattice constants from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return a,b,c
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la = math.sqrt(numpy.dot(a,a))
    lb = math.sqrt(numpy.dot(b,b))
    lc = math.sqrt(numpy.dot(c,c))
    #print("cell_length:",la,lb,lc)
    return la,lb,lc

def cellangle(cell):
    ''' calculate lattice angles (in degree) from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return alpha,beta,gamma
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la,lb,lc = celllength(cell)
    #aa = math.acos(numpy.dot(b,c)/(lb*lc)) * 180.0 / pi
    #ab = math.acos(numpy.dot(c,a)/(lc*la)) * 180.0 / pi
    #ac = math.acos(numpy.dot(a,b)/(la*lb)) * 180.0 / pi
    aa = math.degrees(math.acos(numpy.dot(b,c)/(lb*lc)))
    ab = math.degrees(math.acos(numpy.dot(c,a)/(lc*la)))
    ac = math.degrees(math.acos(numpy.dot(a,b)/(la*lb)))
    #print("cell_angle:",aa,ab,ac)
    return aa,ab,ac

if __name__ == "__main__":
    csstd()
