#!/usr/bin/env python

import sys
import os
import argparse
import time

import jobmanage.job

'''update joblist.txt and submit jobs
'''

#def update_job(joblist_file,caldir_name = "caldir", verbose = False, dryrun = False):
def update_job(joblist_file,caldir_name = "caldir", verbose = False, dryrun = False, qstat = False):
    d = os.path.dirname(os.path.abspath(joblist_file))
    #print("dir: ", d)
    caldir = os.path.join(d, caldir_name)
    print("joblist:",os.path.abspath(joblist_file))
    print("caldir:",caldir)

    job = jobmanage.job.Job()
    #maxjob = float(job.cfgdata.get("maxjob", 0))
    maxjob = job.maxjob

    fjob = os.path.join(d, "joblist.txt")
    fjob_old = os.path.join(d, "joblist.txt.bak")
    fjob_new = os.path.join(d, "joblist.txt.new")

    f0 = open(fjob,"r")
    f = open(fjob_new, "w")

    if qstat:
        jobs0 = jobmanage.job.jobstat()

    print("[jobs]")
    numjob = 0
    for l in f0:
        l = l.strip()
        words = l.split()
        if len(words) < 3:
            continue
        matid = words[0]
        status = words[1]
        jobid = words[2]
        #print(matid)

        if qstat:
            qstatus = jobs0[jobid][1]
            if status == "Q" and qstatus in "QR":
                if verbose:
                    print(matid, "in queue",jobid,qstatus,job0[jobid][3])
            elif status == "Q" and qstatus == "-":
                fstat = os.path.join(caldir, matid, "STATUS_RUN")
                #print("statusfile= ", fstat)
                if os.path.exists(fstat):
                    current = open(fstat, "r").read()
                else:
                    current = " "
                if not "START" in current:
                    status = "W"
                    if verbose:
                        print(matid, "not in queue Q->W")

        if status in "FE":     # finished or error
            f.write(l)
            f.write("\n")
            # output.write("{} {} {}\n".format(matid, status, jobid))
            if verbose:
                if status=="F":
                    print(matid, "finish")
                elif status=="E":
                    print(matid, "error")
        elif status in "QR":   # queued or running
            fstat = os.path.join(caldir, matid, "STATUS_RUN")
            # print("statusfile= ", fstat)
            if os.path.exists(fstat):
                current = open(fstat, "r").read()
            else:
                current = " "
            if "FAILED" in current:
                status = "E"
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                print(matid, "*** error ***")
            elif "FINISH" in current:
                status = "F"
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                if verbose:
                    print(matid, "finish")
            elif "START" in current:
                status = "R"
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                if verbose:
                    print(matid, "running")
                numjob += 1
            else:
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                numjob += 1
                if verbose:
                    print(matid, "in queue")
        elif status == "W":
            if maxjob == 0 or numjob < maxjob:
                cwd = os.getcwd()
                os.chdir(os.path.join(caldir, matid))
                if not dryrun:
                    jobid = job.jobsubmit(matid)
                    status = "Q"
                    fstat_job = open("STATUS_JOB", "w")
                    fstat_job.write("submitted: "+time.ctime()+"\n")
                    fstat_job.close()
                    print(matid, "submitted", jobid)
                else:
                    print(matid, "able to submit")
                numjob += 1
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                os.chdir(cwd)
            else:
                #f.write("{} {} {}\n".format(matid, status, jobid))
                f.write("{0} {1} {2}\n".format(matid, status, jobid))
                print(matid, "wait")
    f0.close()
    f.close()

    if os.path.exists(fjob_old):
        os.remove(fjob_old)
    os.rename(fjob, fjob_old)
    os.rename(fjob_new, fjob)

    # update joblist
    if qstat:
        print("[update joblist]")
        jobs0 = jobmanage.job.jobstat()
        print("[jobs]")
        f0 = open(fjob,"r")
        f = open(fjob_new, "w")
        for l in f0:
            l = l.strip()
            words = l.split()
            if len(words) < 3:
                continue
            matid = words[0]
            status = words[1]
            jobid = words[2]

            qstatus = jobs0[jobid][1]
            if status == "Q" and qstatus in "QR":
                if verbose:
                    print(matid, "in queue",jobid,qstatus,job0[jobid][3])
            elif status == "Q" and qstatus == "-":
                fstat = os.path.join(caldir, matid, "STATUS_RUN")
                #print("statusfile= ", fstat)
                if os.path.exists(fstat):
                    current = open(fstat, "r").read()
                else:
                    current = " "
                if not "START" in current:
                    status = "W"
                    if verbose:
                        print(matid, "not in queue Q->W")

            f.write("{0} {1} {2}\n".format(matid, status, jobid))
            print("{0} {1} {2}".format(matid, status, jobid))
        f0.close()
        f.close()
        os.rename(fjob_new, fjob)

def main():
    arg = argparse.ArgumentParser(description="ac-update")
    arg.add_argument("joblist",nargs=1, type=str, default="")
    arg.add_argument('-v', "--verbose", action="store_true")
    arg.add_argument('-c', "--caldir", action="store",default="caldir")
    arg.add_argument('-q', "--qstat", dest="qstat",action="store_true")
    arg.add_argument('-n', "--dry-run", dest="dryrun", action="store_true", help="don't submit new jobs")
    args = arg.parse_args()

    verbose = args.verbose
    dryrun = args.dryrun
    caldir = args.caldir
    joblist = args.joblist[0]
    qstat = args.qstat
    #update_job(joblist,caldir,verbose,dryrun)
    update_job(joblist,caldir,verbose,dryrun,qstat)

if __name__ == "__main__":
    main()
