import os
import shutil
import math
import numpy
import collections

#from calculator import *
from calculation.calculator import *

class Abinit(Calculator):
    acell = None
    acell0 = None
    acell1 = None
    rprim = None

    def __init__(self):
        super(Abinit,self).__init__()
        #print("abinit")
        self.scenario = "abinit.para.cfg"
        #self.nkp_max = None

    def calc(self):
        print("abinit...",self.matid)

    def inputs(self, task):

        task.input["nsppol"]=self.spin
        #task.input["nspden"]=self.spin
        task.input["nband"]=self.nbands
        [kx,ky,kz] = self.nkp
        task.input["ngkpt"]= "%d %d %d" % (kx,ky,kz)
        
        if task.name == "spin":
            task.input["nsppol"]=2
        elif task.name == "opt1":
            pass
        elif task.name == "opt2":
            pass
        elif task.name == "scf":
            pass
        elif task.name == "dos":
            task.input["nstep"] = ""
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                task.input["ngkpt"]= "%d %d %d" % (kx*2,ky*2,kz*2)
            task.input["nshiftk"] = 1
            task.input["shiftk"] = "0.0 0.0 0.0"
        elif task.name == "pdos":
            task.input["nstep"] = ""
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                task.input["ngkpt"]= "%d %d %d" % (kx*2,ky*2,kz*2)
            task.input["nshiftk"] = 1
            task.input["shiftk"] = "0.0 0.0 0.0"
        elif task.name == "band":
            #task.input["nband"]=""
            task.input["nstep"] = ""
            task.input["nkpt"] = len(self.kpath)
            task.input["kpt"] = ""

        self.input_calc(task.input)
        self.jobfiles()

    def potentials(self,ele):
        for e in ele:
            pp = self.pp[e]
            print("pp:",pp)
            shutil.copy2(self.ppdir+"/"+pp,".")

    def jobfiles(self):
        f = open("job.files","w")
        f.write("job.in\n")
        f.write("job.out\n")
        f.write("jobxi\n")
        f.write("jobxo\n")
        f.write("tmp\n")
        for e in self.ele:
            f.write("../%s\n" % self.pp[e])
        f.close()

    def input_calc(self,input):
        # cell
        cell = numpy.array(self.cell,float)
        #print("cell:",cell)
        acell = self.acell
        #print("acell:",acell)
        rprim = numpy.array(self.cell,float)
        rprim[0] = rprim[0]/acell[0]
        rprim[1] = rprim[1]/acell[1]
        rprim[2] = rprim[2]/acell[2]
        #print("rprim:",rprim)

        # shiftk
        shiftk = []
        if self.cry_system == "cubic" and self.bravais_center == "body":
            #shiftk.append([0.5,0.5,0.5])
            shiftk.append([0.25,0.25,0.25])
            shiftk.append([-0.25,-0.25,-0.25])
        elif self.cry_system == "cubic" and self.bravais_center == "face":
            shiftk.append([0.5,0.5,0.5])
            shiftk.append([0.5,0.0,0.0])
            shiftk.append([0.0,0.5,0.0])
            shiftk.append([0.0,0.0,0.5])
        elif self.cry_system == "hexagonal":
            shiftk.append([0.0,0.0,0.5])
        else:
            shiftk.append([0.5,0.5,0.5])
        
        f = open("job.in","w")
        f.write("#\n")
        #for k,v in input.iteritems():
        for k,v in input.items():
            if k=="acell":
                f.write("%s %s %s %s angstrom\n" % (k,acell[0],acell[1],acell[2]))
            elif k=="rprim":
                f.write("%s\n" % (k))
                #f.write("%10.5f %10.5f %10.5f\n" % (rprim[0][0],rprim[0][1],rprim[0][2]))
                #f.write("%10.5f %10.5f %10.5f\n" % (rprim[1][0],rprim[1][1],rprim[1][2]))
                #f.write("%10.5f %10.5f %10.5f\n" % (rprim[2][0],rprim[2][1],rprim[2][2]))
                f.write("%15.10f %15.10f %15.10f\n" % (rprim[0][0],rprim[0][1],rprim[0][2]))
                f.write("%15.10f %15.10f %15.10f\n" % (rprim[1][0],rprim[1][1],rprim[1][2]))
                f.write("%15.10f %15.10f %15.10f\n" % (rprim[2][0],rprim[2][1],rprim[2][2]))
            elif k=="ntypat":
                f.write("%s %d\n" % (k,self.nele))
            elif k=="znucl":
                f.write("%s " % (k))
                for e in self.ele:
                    f.write("%s " % self.elements[e][0])
                f.write("\n")                
            elif k=="natom":
                f.write("%s %d\n" % (k,self.natom))
            elif k=="typat":
                f.write("%s " % (k))
                for i in range(len(self.atom)):
                    #print(self.atom[i][0])
                    for j in range(len(self.ele)):
                        if self.atom[i][0] == self.ele[j]:
                            f.write("%d " % (j+1))
                f.write("\n")                
            elif k=="xred":
                f.write("%s\n" % (k))
                for a in self.atom:
                    #f.write("%10.5f %10.5f %10.5f\n" % (a[1],a[2],a[3]))                    
                    f.write("%15.10f %15.10f %15.10f\n" % (a[1],a[2],a[3]))                    
            #elif k=="ngkpt":
            #    #f.write("%s %d %d %d\n" % (k,self.nkp[0],self.nkp[1],self.nkp[2]))
            #    f.write("%s %s\n" % (k,v))
            elif k=="nshiftk":
                if len(str(v))>0:
                    f.write("%s %s\n" % (k,v))
                else:
                    f.write("%s %d\n" % (k,len(shiftk)))
            elif k=="shiftk":
                if len(str(v))>0:
                    f.write("%s %s\n" % (k,v))
                else:
                    f.write("%s\n" % (k))
                    for k in shiftk:
                        f.write("%10.5f %10.5f %10.5f\n" % (k[0],k[1],k[2]))
            elif k=="kpt":
                f.write("%s\n" % (k))
                for k in self.kpath:
                    if(len(k)==4):
                        #f.write("%10.5f %10.5f %10.5f\n" % (k[0],k[1],k[2]))
                        f.write("%15.10f %15.10f %15.10f\n" % (k[0],k[1],k[2]))
                    elif(len(k)>4):
                        #f.write("%10.5f %10.5f %10.5f # %s\n" % (k[0],k[1],k[2],k[4]))
                        f.write("%15.10f %15.10f %15.10f # %s\n" % (k[0],k[1],k[2],k[4]))
            elif k=="nband":
                f.write("%s %s\n" % (k,v))
            elif k=="nsppol":
                f.write("%s %s\n" % (k,v))
                # temporary
                if v == 2:
                    #f.write("nspden 2\n")
                    #f.write("occopt 4\n")
                    f.write("spinat\n")
                    if len(self.atom)==1:
                        f.write("0.0 0.0 2.0\n")
                    else:
                        for i in range(len(self.atom)):
                            f.write("0.0 0.0 1.0\n")
            elif len(str(v))>0:
                f.write("%s %s\n" % (k,v))
                
        f.close()
            
    def nbands(self,ele,a):
        natom = 0
        za = 0
        nbands = 0
        for i in range(len(a)):
            natom += int(a[i])
        zv = []
        for e in ele:
            p = self.pp[e]
            doc = ElementTree.parse(p)
            root = doc.getroot()
            for e0 in root.findall("atom"):
                #print("%s"  % (e0.attrib["valence"]))
                zval = e0.attrib["valence"]
                zv.append(zval)
        for i in range(len(a)):
            za += float(zv[i]) * float(a[i])
        nelect = za
        print("nelect:",nelect)
        # temporary
        self.nelect = nelect
        nbands = int(nelect/2)+natom*2
        nbands = int((nelect)*0.6+0.5)+max(4,natom*2)
        if(nbands%2 == 1): nbands+=1
        print("nbands:",nbands)
        return nbands

    def structure(self,name, d):
        if name == "cif":
            # temporary
            self.acell = [self.cell_a,self.cell_b,self.cell_c]
            self.acell0 = self.acell
            rprim = numpy.array(self.cell,float)
            rprim[0] = rprim[0]/self.acell[0]
            rprim[1] = rprim[1]/self.acell[1]
            rprim[2] = rprim[2]/self.acell[2]
            self.rprim = rprim
            pass
        else:
            # d = self.calc[name]["_calc_dir"]
            acell = []
            rprim = []
            xred = []
            axred = []
            f0 = open("../"+d+"/job.out")
            for l in f0:
                if l.find("  acell")>=0:
                    l0 = l.split()
                    acell.append([l0[1],l0[2],l0[3]])
                if l.find("  rprim")>=0:
                    rprim.append([])
                    l0 = l.split()
                    rprim[-1].append([l0[1],l0[2],l0[3]])
                    for i in range(2):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        rprim[-1].append([l0[0],l0[1],l0[2]])
                if l.find("  xred")>=0:
                    xred.append([])
                    l0 = l.split()
                    xred[-1].append([l0[1],l0[2],l0[3]])
                    for i in range(self.natom-1):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        xred[-1].append([l0[0],l0[1],l0[2]])
                if l.find("reduced coordinates (array xred)")>=0:
                    axred.append([])
                    for i in range(self.natom):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        axred[-1].append([l0[0],l0[1],l0[2]])

            if len(rprim)==0: # rprim default value
                rprim.append([[1,0,0],[0,1,0],[0,0,1]])

            acell1 = numpy.array(acell[-1],numpy.float)
            acell1 = acell1 * unit_bohr_angstrom
            #print("acell1",acell1)
            rprim1 = numpy.array(rprim[-1],numpy.float)
            #print("rprim1",rprim1)
            cell1 = numpy.zeros((3,3))
            for i in range(3):
                cell1[i] = numpy.dot(rprim1[i],acell1[i])
            #print("cell",cell1)
            
            self.acell = acell1
            self.cell0 = self.cell
            self.cell = cell1
            self.cell1 = self.cell

            if len(xred)>0 or len(axred)>0:
                if len(xred) > 0:
                    atom1 = numpy.array(xred[-1],numpy.float)
                else:
                    atom1 = numpy.array(axred[-1],numpy.float)
            #print("atom",atom1)
            self.atom0 = self.atom
            for i in range(len(atom1)):
                self.atom[i][1] = atom1[i][0]
                self.atom[i][2] = float(atom1[i][1])
                self.atom[i][3] = float(atom1[i][2])
            self.atom1 = self.atom

    def mag(self, d):
        mag = 0.0
        # d = self.calc["check_spin"]["_calc_dir"]
        f = open("../"+d+"/job.out")
        for l in f:
            if l.find("Total magnetization (exact up - dn):")>=0:
                l0 = l.split(":")
                mag = l0[1].strip()
            #if l.find("Magnetization (Bohr magneton)=")>=0:
            #    l0 = l.split("=")
            #    mag = l0[1].strip()
        mag = float(mag)
        #print("mag:",mag)
        return mag

    def efermi(self, d):
        efermi = 0.0
        # d = self.calc["scf"]["_calc_dir"]
        f = open("../"+d+"/job.out")
        for l in f:
            if l.find("Fermi")>=0:
                l0 = l.split()
                efermi = l0[6]
        efermi = float(efermi)
        print("efermi:",efermi)
        return efermi

    def results(self, task, f):
        name = task.name
        d = task.option["_calc_dir"]

        etotal = None
        efermi = None
        mag = None
        acell = []
        rprim = []
        xred = []
        axred = []

        ispin = self.spin
        nbands = self.nbands
        nelect = self.nelect
        nkpoints = None
        step = []

        f0 = open("job.out")
        for l in f0:
            if l.find("nspden =")>=0:
                l0 = l.split()
                ispin = l0[8]
            if l.find("nkpt =")>=0:
                l0 = l.split()
                nkpoints = l0[8]
            if l.find(" nband ")>=0:
                l0 = l.split()
                nbands = l0[1]

            if l.find("Etotal=")>=0:
                l0 = l.split()
                etotal = l0[2]
            if l.find("Fermi")>=0:
                l0 = l.split()
                efermi = l0[6]
            if l.find("Total magnetization (exact up - dn):")>=0:
                l0 = l.split(":")
                mag = l0[1].strip()

            if l.find("  acell")>=0:
                l0 = l.split()
                acell.append([l0[1],l0[2],l0[3]])
            if l.find("  rprim")>=0:
                rprim.append([])
                l0 = l.split()
                rprim[-1].append([l0[1],l0[2],l0[3]])
                for i in range(2):
                    #c = f0.next()
                    c = next(f0)
                    l0 = c.split()
                    rprim[-1].append([l0[0],l0[1],l0[2]])
            if l.find("  xred")>=0:
                xred.append([])
                l0 = l.split()
                xred[-1].append([l0[1],l0[2],l0[3]])
                for i in range(self.natom-1):
                    #c = f0.next()
                    c = next(f0)
                    l0 = c.split()
                    xred[-1].append([l0[0],l0[1],l0[2]])
            if l.find("reduced coordinates (array xred)")>=0:
                axred.append([])
                for i in range(self.natom):
                    #c = f0.next()
                    c = next(f0)
                    l0 = c.split()
                    axred[-1].append([l0[0],l0[1],l0[2]])

            #if l.find("--- Iteration:")>=0:
            if l.find("     iter   Etot(hartree)")>=0:
                step.append([])
            if l.find(" ETOT")>=0:
                l0 = l.split()
                step[-1].append(l0[2])

        f0.close()

        print("ispin:",ispin)
        print("nbands:",nbands)
        print("nelect:",nelect)
        print("nkpoints:",nkpoints)
        print("nstep:",len(step))

        if etotal is not None:
            etotal = float(etotal) * unit_hartree_ev
        #if efermi is not None:
        #    efermi = float(efermi) * unit_hartree_ev
        if len(step)>0:
            for i in range(len(step)):
                if len(step[i])>0:
                    e = numpy.array(step[i],float)
                    step[i] = e * unit_hartree_ev

        print("etotal:",etotal)
        print("efermi:",efermi)
        print("mag:",mag)

        if len(rprim)==0: # rprim default value
            rprim.append([[1,0,0],[0,1,0],[0,0,1]])

        #print("acell",acell)
        #print("rprim",rprim)
        #print("xred",xred)
        #print("axred",axred)

        cell0 = None
        atom0 = None
        if len(acell)>0 and len(rprim)>0:
            acell0 = numpy.array(acell[0],numpy.float)
            acell0 = acell0 * unit_bohr_angstrom
            #print("acell0",acell0)
            rprim0 = numpy.array(rprim[0],numpy.float)
            #print("rprim0",rprim0)
            cell0 = numpy.zeros((3,3))
            for i in range(3):
                cell0[i] = numpy.dot(rprim0[i],acell0[i])
            #print("cell0",cell0)
        if len(xred)>0 or len(axred)>0:
            if len(xred) > 0:
                atom0 = numpy.array(xred[0],numpy.float)
            elif len(xred) > 0:
                atom0 = numpy.array(axred[0],numpy.float)
            #print("atom0",atom0)

        cell1 = None
        atom1 = None
        if len(acell)>0 and len(rprim)>0:
            acell1 = numpy.array(acell[-1],numpy.float)
            acell1 = acell1 * unit_bohr_angstrom
            #print("acell1",acell1)
            rprim1 = numpy.array(rprim[-1],numpy.float)
            #print("rprim1",rprim1)
            cell1 = numpy.zeros((3,3))
            for i in range(3):
                cell1[i] = numpy.dot(rprim1[i],acell1[i])
            #print("cell1",cell1)
        if len(xred)>0 or len(axred)>0:
            if len(xred) > 0:
                atom1 = numpy.array(xred[-1],numpy.float)
            else:
                atom1 = numpy.array(axred[-1],numpy.float)
            #print("atom1",atom1)

        nkp = None
        f0 = open("job.in")
        for l in f0:
            if l.find("ngkpt")>=0:
                l0 = l.split()
                nkp = [l0[1],l0[2],l0[3]]
        f0.close()

        # dos
        dos = None
        if name == "dos":
            dos = []
            f0 = open("jobxo_DOS")
            for l in f0:
                if l.find("# at")>=0 and l.find("energies")>=0:
                    l0 = l.split()
                    ndos = int(l0[2])
                if self.spin==1:
                    if l.find("# energy(Ha)     DOS  integrated DOS")>=0:
                        dos.append([])
                        for i in range(ndos):
                            #l = f0.next()
                            l = next(f0)
                            l0 = l.split()
                            #print(l0)
                            l1 = numpy.array(l0,float)
                            l1[0] = l1[0]*unit_hartree_ev
                            l1[1:2] = l1[1:2]/unit_hartree_ev
                            dos[0].append([l1[0],l1[1],l1[2]])
                elif self.spin==2:
                    if l.find("Spin-up DOS")>=0:
                        #f0.next()
                        next(f0)
                        dos.append([])
                        for i in range(ndos):
                            #l = f0.next()
                            l = next(f0)
                            l0 = l.split()
                            #print(l0)
                            l1 = numpy.array(l0,float)
                            l1[0] = l1[0]*unit_hartree_ev
                            l1[1:2] = l1[1:2]/unit_hartree_ev
                            dos[0].append([l1[0],l1[1],l1[2]])
                    if l.find("Spin-dn DOS")>=0:
                        #f0.next()
                        next(f0)
                        dos.append([])
                        for i in range(ndos):
                            #l = f0.next()
                            l = next(f0)
                            l0 = l.split()
                            #print(l0)
                            l1 = numpy.array(l0,float)
                            l1[0] = l1[0]*unit_hartree_ev
                            l1[1:2] = l1[1:2]/unit_hartree_ev
                            dos[1].append([l1[0],l1[1],l1[2]])
            f0.close()
            ndos = len(dos[0])
            print("ndos",ndos)

        # pdos
        pdos = None
        if name == "pdos":
            pdos = []
            for i in range(len(self.atom)):
                f0 = "jobxo_DOS_AT%04d" % (i+1)
                print(f0,i)
                pdos.append([])
                f0 = open(f0)
                for l in f0:
                    if l.find("# at")>=0 and l.find("energies")>=0:
                        l0 = l.split()
                        ndos = int(l0[2])
                    if self.spin==1:
                        if l.find("# energy(Ha)")>=0:
                            pdos[-1].append([])
                            for i in range(ndos):
                                #l = f0.next()
                                l = next(f0)
                                l0 = l.split()
                                #print(l0)
                                l1 = numpy.array(l0,float)
                                l1[0] = l1[0]*unit_hartree_ev
                                l1[1:5] = l1[1:5]/unit_hartree_ev
                                pdos[-1][0].append([l1[0],l1[1],l1[2],l1[3],l1[4],l1[5]])
                    elif self.spin==2:
                        if l.find("Spin-up DOS")>=0:
                            pdos[-1].append([])
                        if l.find("Spin-dn DOS")>=0:
                            pdos[-1].append([])
                        if l.find("# energy(Ha)")>=0:
                            for i in range(ndos):
                                #l = f0.next()
                                l = next(f0)
                                l0 = l.split()
                                #print(l0)
                                l1 = numpy.array(l0,float)
                                l1[0] = l1[0]*unit_hartree_ev
                                l1[1:5] = l1[1:5]/unit_hartree_ev
                                pdos[-1][-1].append([l1[0],l1[1],l1[2],l1[3],l1[4],l1[5]])
                f0.close()
            ndos = len(pdos[0][0])
            print("ndos",ndos)

            pdoscol = ["energy","l=0","l=1","l=2","l=3","l=4"]

        # band
        nkpoints = None
        kp = None
        ke = None
        if name == "dos" or name == "band":
            f0 = open("jobxo_EIG")
            kp = []
            ke = []  # spin.kp,band,[ene]
            for l in f0:
                if l.find("Eigenvalues")>=0 and l.find("hartree")>=0:
                    l0 = l.split()
                    nkpoints = l0[4]
                    kp = []
                    #ke = []
                    ke.append([]) # spin
                elif l.find("Eigenvalues")>=0 and l.find("eV")>=0:
                    l0 = l.split()
                    nkpoints = l0[6]
                    kp = []
                    #ke = []
                    if len(ke)>0 and l.find("SPIN UP")>=0:
                        del ke[0]
                    elif len(ke)>1 and l.find("SPIN DOWN")>=0:
                        del ke[1]
                    ke.append([]) # spin

                elif l.find("kpt#")>=0 and l.find("kpt=")>=0:
                    l0 = l.split(",")
                    l1 = l0[1].split() # nband
                    nb = int(l1[1])
                    l1 = l0[3].split() # kpt
                    kp.append([float(l1[1]),float(l1[2]),float(l1[3])])
                    ke[-1].append([])
                    ke0 = []
                    for i in range(int(nb/8)+1):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        for j in l0:
                            ke0.append(j)
                    for e in ke0:
                        ke[-1][-1].append([float(e),0.0])

            print("kp:",len(kp))
            print("ke:",len(ke),len(ke[0]))
            for i in range(len(ke)):
                for j in range(len(ke[i])):
                    for k in range(len(ke[i][j])):
                        #print(i,j,ke[i][j][k])
                        pass

        # update parameter
        doc2 = ElementTree.parse(f)
        root2 = doc2.getroot()
        #for e0 in root2.findall("calculation[@name="+name+"]/properties")):
        for e0 in root2.findall("calculation"):
            if e0.attrib.get("name") == name:
                # structure
                for e1 in e0.findall("structure"):
                    if e1.attrib.get("name") == "initial":
                        if cell0 is not None:
                            e2 = ElementTree.SubElement(e1,"lattice")
                            e2.set("type","primitive")
                            e3 = ElementTree.SubElement(e2,"a1")
                            e3.text = str(cell0[0][0])+" "+str(cell0[0][1])+" "+str(cell0[0][2])
                            e3 = ElementTree.SubElement(e2,"a2")
                            e3.text = str(cell0[1][0])+" "+str(cell0[1][1])+" "+str(cell0[1][2])
                            e3 = ElementTree.SubElement(e2,"a3")
                            e3.text = str(cell0[2][0])+" "+str(cell0[2][1])+" "+str(cell0[2][2])
                            #e2 = ElementTree.SubElement(e1,"atom")
                        if atom0 is not None:
                            e2 = ElementTree.SubElement(e1,"atom")
                            for i in range(len(atom0)):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name",self.atom[i][0])
                                e3.text = str(atom0[i][0])+" "+str(atom0[i][1])+" "+str(atom0[i][2])
                        
                    if e1.attrib.get("name") == "final":
                        if cell1 is not None:
                            e2 = ElementTree.SubElement(e1,"lattice")
                            e2.set("type","primitive")
                            e3 = ElementTree.SubElement(e2,"a1")
                            e3.text = str(cell1[0][0])+" "+str(cell1[0][1])+" "+str(cell1[0][2])
                            e3 = ElementTree.SubElement(e2,"a2")
                            e3.text = str(cell1[1][0])+" "+str(cell1[1][1])+" "+str(cell1[1][2])
                            e3 = ElementTree.SubElement(e2,"a3")
                            e3.text = str(cell1[2][0])+" "+str(cell1[2][1])+" "+str(cell1[2][2])
                            #e2 = ElementTree.SubElement(e1,"atom")
                        if atom1 is not None:
                            e2 = ElementTree.SubElement(e1,"atom")
                            for i in range(len(atom1)):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name",self.atom[i][0])
                                e3.text = str(atom1[i][0])+" "+str(atom1[i][1])+" "+str(atom1[i][2])

                # kpoints
                for e1 in e0.findall("kpoints"):
                    if e1.attrib.get("type") == "kmesh":
                        if nkp is not None:
                            e1.text = str(nkp[0])+" "+str(nkp[1])+" "+str(nkp[2])
                    elif e1.attrib.get("type") == "kpath":
                        e1.clear()
                        e1.set("type","kpath")
                        for k in self.kpath:
                            e2 = ElementTree.SubElement(e1,"i")
                            if len(k)>4:
                                e2.set("name",str(k[4]))
                            #e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                            #s = "%10.5f %10.5f %10.5f" % (k[0],k[1],k[2])
                            s = "%15.10f %15.10f %15.10f" % (k[0],k[1],k[2])
                            e2.text = s.strip()

                # parameter
                for e1 in e0.findall("parameters"):
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","ispin")
                    e2.text = str(ispin)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nbands")
                    e2.text = str(nbands)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nelect")
                    e2.text = str(nelect)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nkpoints")
                    e2.text = str(nkpoints)
                    if kp is not None:
                        e2 = ElementTree.SubElement(e1,"kpoints")
                        for i in range(len(kp)):
                            e3 = ElementTree.SubElement(e2,"i")
                            e3.text = str(kp[i][0])+" "+str(kp[i][1])+" "+str(kp[i][2])

                # property
                for e1 in e0.findall("properties"):
                    if etotal is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal")
                        e2.set("unit","eV")
                        e2.text = str(etotal)
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal_per_atom")
                        e2.set("unit","eV/atom")
                        e2.text = str(float(etotal)/self.natom)
                    if efermi is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","efermi")
                        e2.set("unit","eV")
                        e2.text = str(efermi)
                    if mag is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","magnetization")
                        e2.set("unit","uB")
                        e2.text = str(mag)
                    if dos is not None:
                        e2 = ElementTree.SubElement(e1,"dos")
                        for i in range(len(dos)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(dos[i])):
                                e4 = ElementTree.SubElement(e3,"i")
                                e4.text = str(dos[i][j][0])+" "+str(dos[i][j][1])+" "+str(dos[i][j][2])
                    if pdos is not None:
                        e2 = ElementTree.SubElement(e1,"pdos")
                        #e3 = ElementTree.SubElement(e2,"i")
                        #e3.set("name","value")
                        #s = ""
                        #for i in range(len(pdoscol)):
                        #    s += str(pdoscol[i]) + " "
                        #e3.text = s.strip()
                        for i in range(len(pdos)):
                            e3 = ElementTree.SubElement(e2,"atom")
                            e3.set("name",self.atom[i][0])
                            e3.set("atom",str(i+1))

                            e4 = ElementTree.SubElement(e3,"i")
                            e4.set("name","value")
                            s = ""
                            for j in range(len(pdoscol)):
                                s += str(pdoscol[j]) + " "
                            e4.text = s.strip()

                            for j in range(len(pdos[i])):
                                e4 = ElementTree.SubElement(e3,"spin")
                                e4.set("spin",str(j+1))
                                for k in range(len(pdos[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    s = ""
                                    for l in range(len(pdos[i][j][k])):
                                        s += str(pdos[i][j][k][l]) + " "
                                    e5.text = s.strip()
                    if ke is not None:
                        e2 = ElementTree.SubElement(e1,"band")
                        for i in range(len(ke)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(ke[i])):
                                e4 = ElementTree.SubElement(e3,"kpoint")
                                kp = j+1
                                e4.set("kpoint",str(kp))
                                for k in range(len(ke[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    #s = "%10.5f %10.5f" % (ke[i][j][k][0],ke[i][j][k][1])
                                    #e5.text = s
                                    e5.text = str(ke[i][j][k][0])+" "+str(ke[i][j][k][1])

                # step
                for e1 in e0.findall("step"):
                    if step is not None:
                        for i in range(len(step)):
                            e2 = ElementTree.SubElement(e1,"sc")
                            e2.set("step",str(i+1))
                            for j in range(len(step[i])):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name","etotal")
                                e3.set("unit","eV")
                                e3.text = str(step[i][j])

        tree = ElementTree.tostring(root2,'utf-8')        
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

        if name == "scf":
            self.charge()

    def charge(self):
        f = open("charge.in","w")
        f.write("jobxo_DEN\n")
        if self.spin==2:
            f.write("0\n")
        f.write("14\n")
        f.write("charge.cube\n")
        f.write("0\n")
        f.close()
        
        cmd = self.cmd
        cmd = cmd.rstrip("abinit")
        cmd = cmd+"cut3d"
        os.system(cmd+" < charge.in > log2")
        
    def calstat(self):
        stat = []

        f0 = "calculations.xml"

        if not os.path.exists(f0):
            return stat

        doc = ElementTree.parse(f0)
        root = doc.getroot()

        matid = None
        chemform = None
        crysys = None
        spg = None
        spgno = None
        cif = None
        calc = []
        for e in root.findall("material"):
            for e1 in e.getchildren():
                if e1.tag == "material_id":
                    matid = e1.text
                if e1.tag == "chemical_formula":
                    chemform = e1.text
                if e1.tag == "crystal_system":
                    crysys = e1.text
                if e1.tag == "spacegroup":
                    spg = e1.text
                if e1.tag == "spacegroup_no":
                    spgno = e1.text
                if e1.tag == "cif":
                    cif = e1.text
        for e in root.findall("calculation"):
            calc.append(e.attrib.get("name"))

        stat.append([matid,chemform,crysys,spg,spgno,cif])
        stat.append(calc)
        stat.append([])
        #stat.append([0]*len(calc))

        for name in calc:
            for e in root.findall("calculation"):
                if e.attrib.get("name")==name:
                    s=0
                    for e1 in e.findall("inputs/i"):
                        if e1.attrib.get("name")=="_calc_dir":
                            d = e1.text
                            f1 = d+"/job.out"
                            if os.path.exists(f1):
                                f = open(f1)
                                for l in f:
                                    if l.find("Calculation completed.")>=0:
                                        s = 0
                                        break
                                    else:
                                        s = 1
                            else:
                                s = -1

            stat[2].append(s)
            
        return stat
