import os
import string
import shutil
import argparse
import glob
import math
import numpy
import time

import structure.crystal
import jobmanage.job
import calc
import calprop
import toast_config

class Autocalc():
    version = "0.6.0.037"

    def __init__(self, program, mat, caldir, nproc, scenario):

        self.program = program
        self.mat=mat
        self.caldir = caldir
        self.nproc = nproc
        self.scenario = scenario

        self.csdir = "/home/mits/csdata/data"

    def calculate(self):
        print("auto calculation...")

        program_options, job_options = toast_config.read_toast_config(self.program)

        # scenario
        if os.path.exists(self.scenario):
            self.scenario = os.path.abspath(self.scenario)
        else:
            if os.path.exists(toast_config.cfgdir+"/"+self.scenario):
                self.scenario = os.path.abspath(toast_config.cfgdir+"/"+self.scenario)
            else:
                # self.scenario = "default"
                self.scenario = os.path.join(toast_config.cfgdir, program_options["scenario"])
                if not os.path.exists(self.scenario):
                    print("Error scenaro {} does not exist".format(self.scenario))

        print("[scenario]\n",self.scenario)

        # calculate
        self.calc_prep()
        self.calc()

    def calc_prep(self):
        #files = os.listdir(self.matdir)
        #files = glob.glob(self.matdir+"/*.cif")
        #files.sort()

        mat = self.mat

        d = self.caldir
        if not os.path.exists(d):
            os.mkdir(d)

        d = os.path.join(self.caldir, mat)
        if not os.path.exists(d):
            os.mkdir(d)

        # copy scenario into self.caldir
        #if not os.path.exists(self.scenario):
        #    shutil.copy2(self.scenario, d)
        self.scenario2 = os.path.abspath(os.path.join(d, os.path.basename(self.scenario)))
        if not os.path.exists(self.scenario2):
            shutil.copy2(self.scenario, d)

        print("[materials]")
        cif = mat+".cif"

        # atomwork
        if not os.path.exists(self.csdir+"/"+mat+"/"+cif):
            cif = mat+"-1-2.cif"
        shutil.copy2(self.csdir+"/"+mat+"/"+cif,d)

        cry = structure.crystal.Crystal()
        cry.read_cif(d+"/"+cif)
        cry.check_cif()
        cry.construct_structure()

        ele = []
        for e in range(len(cry.velement_name)):
            ele.append(cry.velement_name[e])
        natom_ele = []
        for e in range(len(cry.velement_size)):
            natom_ele.append(cry.velement_size[e])
        chemform = cry.chemical_formula_sum
        if not chemform:
            s = [e+str(na) for e,na in zip(ele,natom_ele)]
            #chemform = string.join(s)
            chemform = " ".join(s)
        chemform = chemform.replace(" ","")
        crysys = cry.space_group_crystal_system
        spgname = cry.symmetry_space_group_name
        spgnum = str(cry.symmetry_Int_Tables_number)
        nele = len(ele)
        natom = len(cry.vatom_prim)

        print(mat,chemform,crysys,spgname,spgnum,cif,nele,natom)

    def calc(self):
        job = jobmanage.job.Job()
        cmd = str(os.path.dirname(__file__))+"/../ac-calc.py"
        cmd = os.path.abspath(cmd)

        cwd = os.getcwd()

        print("[jobs]")
        #maxjob = float(job.cfgdata.get("maxjob", 0))
        #maxjob = job.maxjob
        #numjob = 0

        mat = self.mat

        os.chdir(self.caldir)
        # now in caldir

        os.chdir(self.caldir+"/"+mat)

        print("calc...",mat)
        #cmdline = "python "+cmd+" -m "+mat+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" > log 2> err"
        cmdline = "python "+cmd+" -m "+mat+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" > log"

        job.jobscript(cmdline, mat, self.nproc)
        jobid = job.jobsubmit(mat)

        print("jobid:",jobid)

        os.chdir("..")

        os.chdir(cwd)

def autocalc():
    arg = argparse.ArgumentParser(description="ac-setup")
    arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-m","--mat",action="store",dest="mat",default="mat")
    arg.add_argument("-c","--caldir",action="store",dest="caldir",default="caldir")
    arg.add_argument("-s","--scenario",action="store",dest="scenario",default="default")
    arg.add_argument("-n","--nproc",action="store",dest="nproc",default=1)
    args = arg.parse_args()
    # ac = Autocalc()
    ac = Autocalc(args.program, args.mat, args.caldir, args.nproc, args.scenario)
    # ac.program = args.program
    # ac.mat = args.mat
    # ac.caldir = args.caldir
    # ac.nproc = args.nproc
    # ac.scenario = args.scenario
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    print("ver.",ac.version)
    print()
    print("[auto calculation]")
    print("program:",ac.program)
    print("mat:",ac.mat)
    print("cal:",ac.caldir)
    print("scenario:",ac.scenario)
    print("nproc:",ac.nproc)
    ac.calculate()

if __name__ == "__main__":
    autocalc()
