import os
import string
import shutil
import argparse
import glob
import math
import numpy
import time
from xml.etree import ElementTree
import zipfile

import structure.crystal
import jobmanage.job
#import calc
#import calprop
#import calvis
#import toast_config
import calculation.calc
import calculation.calprop
import calculation.calvis
import calculation.toast_config

class Autocalc():
    version = "0.6.0.037"

    def __init__(self, program, matdir, caldir, matlist, nproc, scenario):

        self.program = program
        self.matdir = matdir
        self.caldir = caldir
        self.matlist = matlist
        self.nproc = nproc
        self.scenario = scenario

        self.mat=None

    def calculate(self):
        print("auto calculation...")

        #program_options, job_options = toast_config.read_toast_config(self.program)
        program_options, job_options = calculation.toast_config.read_toast_config(self.program)

        # scenario
        if os.path.exists(self.scenario):
            self.scenario = os.path.abspath(self.scenario)
        else:
            #if os.path.exists(toast_config.cfgdir+"/"+self.scenario):
            if os.path.exists(calculation.toast_config.cfgdir+"/"+self.scenario):
                #self.scenario = os.path.abspath(toast_config.cfgdir+"/"+self.scenario)
                self.scenario = os.path.abspath(calculation.toast_config.cfgdir+"/"+self.scenario)
            else:
                # self.scenario = "default"
                #self.scenario = os.path.join(toast_config.cfgdir, program_options["scenario"])
                self.scenario = os.path.join(calculation.toast_config.cfgdir, program_options["scenario"])
                if not os.path.exists(self.scenario):
                    print("Error scenaro {} does not exist".format(self.scenario))

        print("[scenario]\n",self.scenario)

        # calculate
        self.calc_prep()
        self.calc()

    def calc_prep(self):
        #files = os.listdir(self.matdir)
        files = glob.glob(self.matdir+"/*.cif")
        files.sort()

        d = self.caldir
        if not os.path.exists(d):
            os.mkdir(d)

        # copy scenario into self.caldir
        shutil.copy2(self.scenario, d)
        self.scenario2 = os.path.abspath(os.path.join(d, os.path.basename(self.scenario)))

        self.mat = []
        if self.matlist != None and os.path.exists(self.matlist):
            fmat = open(self.matlist)
            for l in fmat:
                if "#" in l:
                    continue
                else:
                    l0 = l.split()
                    self.mat.append(l0[0])
        else:
            for f in files:
                f0 = f.split("/")
                m0 = f0[-1].split(".")
                mat = m0[0]
                cif = f0[-1]
                # atomwork
                if "-1-2" in mat:
                    mat = mat.replace("-1-2","")
                self.mat.append(mat)

        print("[materials]")
        # skip mat if directory self.caldir/mat exists
        matlist = []
        for mat in self.mat:
            d = os.path.join(self.caldir, mat)
            if os.path.exists(d):
                print("{} already exists".format(mat))
            else:
                matlist.append(mat)
        self.mat = matlist
        # self.mat = [mat for mat in self.mat if not os.path.exists(os.path.join(self.caldir, mat))]

        # fmat = open("matlist.txt","w")
        fmat = open("matlist.txt","a")
        numcalc = 0
        self.mat_ok = []
        for i,mat in enumerate(self.mat):

            cif = mat+".cif"
            cif_file = os.path.join(self.matdir, cif)
            # atomwork
            if not os.path.exists(cif_file):
                cif = mat+"-1-2.cif"
                cif_file = os.path.join(self.matdir, cif)
            if not os.path.exists(cif_file):
                print("CIF file for {} not found".format(mat))
                continue

            try:
                d = os.path.join(self.caldir, mat)
                os.mkdir(d)
                shutil.copy2(cif_file, d)

                cry = structure.crystal.Crystal()
                cry.read_cif(self.matdir+"/"+cif)
                cry.check_cif()
                cry.construct_structure()

                ele = []
                for e in range(len(cry.velement_name)):
                    ele.append(cry.velement_name[e])
                natom_ele = []
                for e in range(len(cry.velement_size)):
                    natom_ele.append(cry.velement_size[e])
                chemform = cry.chemical_formula_sum
                if not chemform:
                    s = [e+str(na) for e,na in zip(ele,natom_ele)]
                    #chemform = string.join(s)
                    chemform = " ".join(s)
                chemform = chemform.replace(" ","")
                crysys = cry.space_group_crystal_system
                spgname = cry.symmetry_space_group_name
                spgnum = str(cry.symmetry_Int_Tables_number)
                nele = len(ele)
                natom = len(cry.vatom_prim)
            #except:
            #    print("{} error in processing cif file: {}".format(i+1, cif_file))
            except Exception as e:
                print(e)
            else:
                self.mat_ok.append(mat)
                numcalc += 1
                print(i+1, numcalc,mat,chemform,crysys,spgname,spgnum,cif,nele,natom)
                fmat.write(mat+" "+chemform+" "+crysys+" "+spgname+" "+spgnum+" "+cif+"\n")
        fmat.close()

    def calc(self):
        dirs = os.listdir(self.caldir)
        dirs.sort()
        job = jobmanage.job.Job()
        cmd = str(os.path.dirname(__file__))+"/../ac-calc.py"
        cmd = os.path.abspath(cmd)

        cwd = os.getcwd()

        print("[jobs]")
        #maxjob = float(job.cfgdata.get("maxjob", 0))
        maxjob = job.maxjob
        numjob = 0
        # fjob = open("joblist.txt","w")
        fjob = open("joblist.txt","a")

        os.chdir(self.caldir)
        # now in caldir

        for matid in self.mat_ok:
            os.chdir(matid)

            print("calc...",matid)
            #cmdline = "python "+cmd+" -m "+matid+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" > log 2> err"
            #cmdline = "python "+cmd+" -m "+matid+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" > log"
            # csh redirect stdout and stder
            #cmdline = "python "+cmd+" -m "+matid+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" >& log"
            cmdline = "python3 "+cmd+" -m "+matid+" -p "+self.program+" -s "+self.scenario2+" -n "+str(self.nproc)+" >& log"
            job.jobscript(cmdline, matid, self.nproc)

            #print(maxjob,numjob)
            if maxjob == 0 or numjob < maxjob:
                jobid = job.jobsubmit(matid)
                numjob += 1
                #fjob.write("{} Q {}\n".format(matid, jobid))
                fjob.write("{0} Q {1}\n".format(matid, jobid))
                fstat_job = open("STATUS_JOB", "w")
                fstat_job.write("submitted: "+time.ctime()+"\n")
                fstat_job.close()
                print(matid,"submitted",jobid)
            else:
                #fjob.write("{} W {}\n".format(matid, 0))
                fjob.write("{0} W {1}\n".format(matid, 0))
                print(matid,"wait")

            os.chdir("..")

        os.chdir(cwd)

        # for k,v in job.jobid.iteritems():
        #     print(k,v)
        #     fjob.write(k+" "+v+"\n")
        #     #print(job.jobid(k))
        fjob.close()

def caldata():
    arg = argparse.ArgumentParser(description="ac-summary")
    #arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-c","--caldir",action="store",dest="caldir")
    arg.add_argument("-o","--output",action="store",dest="fout",default="caldata.txt")
    args = arg.parse_args()
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    #print("ver.",ac.version)
    print()
    print("[caldata]")
    #print("program:",args.program)
    print("caldir:",args.caldir)

    f0 = args.fout
    print("file:",f0)
    f = open(f0,"w")
    f.write("material ")
    f.write("chemica_formula ")
    f.write("spacegroup_name ")
    f.write("spacegroup_no ")
    f.write("num_type ")
    f.write("num_atom(prim) ")
    f.write("lattice_parameter(original) ")
    f.write("lattice_parameter(relaxed) ")
    f.write("total_energy(eV/atom) ")
    f.write("fermi_energy(eV) ")
    f.write("bandgaptype ")
    f.write("bandgap(eV) ")
    f.write("\n")

    caldir = args.caldir
    dirs = os.listdir(args.caldir)        
    dirs.sort()
    for d in dirs:
        f0 = caldir+"/"+d+"/calculations.xml"
        print(d)
        if os.path.exists(f0):
            #v = calprop.caldata2(f0)
            v = calculation.calprop.caldata2(f0)
            for l in v:
                if l:
                    f.write(str(l)+" ")
                else:
                    f.write("- ")
            f.write("\n")
    f.close()    

def caldata2():
    arg = argparse.ArgumentParser(description="ac-summary")
    #arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-c","--caldir",action="store",dest="caldir")
    arg.add_argument("-o","--output",action="store",dest="fout",default="caldata.html")
    arg.add_argument("-j","--jsdir",action="store",dest="jsdir",default=None)
    arg.add_argument("-a","--archive",action="store",dest="zout",default=None)
    args = arg.parse_args()
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    #print("ver.",ac.version)
    print()
    print("[calvis]")
    #print "program:",args.program
    print("caldir:",args.caldir)

    f0 = args.fout
    print("file:",f0)
    f = open(f0,"w")

    jsdir = args.jsdir
    if jsdir==None or jsdir=="cdn":
        #jsdir = str(os.path.dirname(__file__))+"/../../js"        
        jsdir0 = "https://code.jquery.com"
        jsdir1 = "https://cdn.datatables.net/1.10.19/js"
        cssdir1 = "https://cdn.datatables.net/1.10.19/css"
    elif jsdir=="local":
        jsdir = str(os.path.dirname(__file__))+"/../../js"
        jsdir0 = jsdir
        jsdir1 = jsdir
        cssdir1 = jsdir

    print("jsdir:",jsdir)

    f.write("<html>\n")
    f.write("<head>\n")
    f.write("<title>TOAST: Template Oriented Atomic Simulation Toolkit</title>\n")
    f.write("<style type=\"text/css\">\n")
    f.write("@import \""+cssdir1+"/jquery.dataTables.min.css\";\n")
    #f.write("@import \""+cssdir1+"/dataTables.tableTools.min.css\";\n")
    f.write("table {font-size: 10px;}\n")
    f.write("</style>\n")
    f.write("<script type=\"text/javascript\" src=\""+jsdir0+"/jquery.min.js\"></script>\n")
    f.write("<script type=\"text/javascript\" src=\""+jsdir1+"/jquery.dataTables.min.js\"></script>\n")
    #f.write("<script type=\"text/javascript\" src=\""+jsdir+"/dataTables.tableTools.min.js\"></script>\n")
    f.write("</head>\n")
    f.write("<body bgcolor=white>\n")
    f.write("<b>Template Oriented Atomic Simulation Toolkit (TOAST)</b><p>\n")
    f.write("<small><b>Ab-initio Calculation</b></small><p>\n")

    f.write("<small><b>Calculation Result</b></small><br>\n")
    f.write("<script>\n")
    f.write("$(document).ready(function(){\n")
    f.write("$('#table').dataTable({\n")
    f.write("\"aLengthMenu\": [[100,1000,5000,10000], [100,1000,5000,10000]],\n")
    f.write("\"iDisplayLength\": 100,\n")
    #f.write("\"sDom\":'T<\"clear\">lfrtip',\n")
    #f.write("\"oTableTools\": { \n")
    #f.write("\"sSwfPath\": \""+jsdir+"/copy_csv_xls_pdf.swf\",\n")
    #f.write("\"aButtons\": [\"csv\",\"pdf\",\"print\"]}\n")
    f.write("});\n")
    f.write("});\n")
    f.write("</script>\n")
    f.write("<table id=\"table\" class=\"display compact\" cellspacing=\"0\">\n")
    f.write("<thead>\n")
    f.write("<tr>")
    f.write("<th rowspan=2></th>")
    f.write("<th rowspan=2></th>")
    f.write("<th rowspan=2>Status</th>")
    f.write("<th rowspan=2>material</th>")
    f.write("<th rowspan=2>chemica_formula</th>")
    f.write("<th rowspan=2>crystal system</th>")
    f.write("<th rowspan=2>spacegroup_name</th>")
    f.write("<th rowspan=2>spacegroup_no</th>")
    f.write("<th rowspan=2>num_type</th>")
    f.write("<th rowspan=2>num_atom(prim)</th>")
    f.write("<th colspan=6>lattice_parameter(original)</th>")
    f.write("<th colspan=6>lattice_parameter(relaxed)</th>")
    f.write("<th rowspan=2>total_energy(eV/atom)</th>")
    f.write("<th rowspan=2>fermi_energy(eV)</th>")
    f.write("<th rowspan=2>bandgaptype</th>")
    f.write("<th rowspan=2>bandgap(eV)</th>")
    f.write("</tr>\n")
    f.write("<tr>")
    f.write("<th/><th/><th/>")
    f.write("<th/><th/><th/>")
    f.write("<th/><th/><th/>")
    f.write("<th/><th/><th/>")
    f.write("</tr>\n")
    f.write("</thead>\n")

    f.write("<tbody>\n")

    caldir = args.caldir
    dirs = os.listdir(args.caldir)        
    dirs.sort()
    dirs0 = []
    i=0
    for d in dirs:
        if os.path.isdir(caldir+"/"+d)==False:
            continue
        f1 = caldir+"/"+d+"/calculations.xml"
        if os.path.exists(f1)==False:
            continue

        b = True
        try:
            doc = ElementTree.parse(f1)
            #dirs0.append(d)
            root = doc.getroot()

            e0 = root.findall("structure[@name='relaxed']/")
            if len(e0)==0:
                b = False

            e0 = root.findall("calculation[@name='scf']/properties/")
            if len(e0)==0:
                b = False
            e0 = root.findall("calculation[@name='dos']/properties/") + root.findall("calculation[@name='nscf']/properties/")
            if len(e0)==0:
                b = False
            else:  # temporary
                for e1 in root.findall("calculation[@name='dos']/properties/dos/spin/i") + root.findall("calculation[@name='tdos']/properties/dos/spin/i"):
                    if e1.text.find("******")>=0:
                        b = False
                        break
            e0 = root.findall("calculation[@name='band']/properties/")
            if len(e0)==0:
                b = False
            else:  # temporary
                for e1 in root.findall("calculation[@name='band']/properties/band/spin/kpoint/i"):
                    if e1.text.find("******")>=0:
                        b = False
                        break
            if b:
                dirs0.append(d)
        except ElementTree.ParseError:
            b = False

        i += 1
        print(i,d,b)

        f.write("<tr>")

        if b:
            f2 = caldir+"/"+d+"/calculations.html"
            f.write("<th><a href=\""+f2+"\">"+str(i)+"</th>")
            f.write("<td>"+d+"</td>")
            f.write("<td>O</td>")
        else:
            f.write("<th>"+str(i)+"</th>")
            f.write("<td>"+d+"</td>")
            f.write("<td>X</td>")

        #v = calprop.caldata2(f1)
        v = calculation.calprop.caldata2(f1)
        for l in v:
            if l:
                f.write("<td>"+str(l)+"</td>")
            else:
                f.write("<td/>")

        f.write("</tr>\n")

        if b:
            f2 = caldir+"/"+d+"/calculations.html"
            #calvis.visdata(f1,f2,jsdir)
            calculation.calvis.visdata(f1,f2,jsdir)
        
    f.write("</tbody>\n")
    f.write("</table>\n")
    f.write("</body>\n")
    f.write("</html>\n")

    f.close()    

    if args.zout!=None:
        print("[archive]")
        f3 = args.zout
        print("zout:",f3)
        #z = zipfile.ZipFile(f3,'w',zipfile.ZIP_DEFLATED)
        z = zipfile.ZipFile(f3,'w',zipfile.ZIP_DEFLATED,allowZip64=True)
        f = open(f0)
        s = ""
        lf = f.readlines()
        for l in lf:
            l = l.strip().replace("href=\""+caldir+"/","href=\"")
            #print(l)
            s += l+"\n"
        z.writestr("caldata.html",s)
        for d in dirs0:
            print(d)
            f1 = caldir+"/"+d+"/calculations.xml"
            doc = ElementTree.parse(f1)
            root = doc.getroot()
            e = root.find("material/material_id")
            mat = e.text
            e = root.find("material/cif")
            cif = e.text
            print(mat,cif)
            z.write(caldir+"/"+d+"/calculations.html",d+"/calculations.html")
            z.write(caldir+"/"+d+"/calculations.xml",d+"/calculations.xml")
            z.write(caldir+"/"+d+"/"+cif,d+"/"+cif)
            z.write(caldir+"/"+d+"/"+mat+".relaxed.cif",d+"/"+mat+".relaxed.cif")
            z.write(caldir+"/"+d+"/"+mat+".bz.spt",d+"/"+mat+".bz.spt")
        f.close()

def calstat():
    arg = argparse.ArgumentParser(description="ac-stat")
    arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-c","--caldir",action="store",dest="caldir")
    arg.add_argument("-o","--output",action="store",dest="fout",default="calstat.txt")
    args = arg.parse_args()
    caldir = args.caldir
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    #print "ver.",ac.version
    print()
    print("[calstat]")
    print("program:",args.program)
    print("caldir:",args.caldir)

    #cal = calc.CalcStat(args.program)
    cal = calculation.calc.CalcStat(args.program)
    # cal.program = args.program

    f0 = os.path.abspath(args.fout)
    print("file:",f0)
    f = open(f0,"w")
    f.write("material ")
    f.write("chemica_formula ")
    f.write("spacegroup_name ")
    f.write("spacegroup_no ")
    f.write("calculation_status ")
    f.write("\n")

    caldir = args.caldir
    dirs = os.listdir(args.caldir)        
    dirs.sort()

    cwd = os.getcwd()
    os.chdir(caldir)
    for d in dirs:
        if os.path.isdir(d)==False:
            continue
        f1 = d+"/calculations.xml"
        if os.path.exists(f1)==False:
            continue

        os.chdir(d)
        matid = d
        #print(matid,)
        print(matid+" ",end="")
        v = cal.calstat()
        #print(v)
        if len(v)>0:
            for l in v[0]:
                f.write(l+" ")
            for l in v[2]:
                if l==-1:
                    s = "- "
                if l==0:
                    s = "o "
                if l==1:
                    s = "E "
                f.write(s)
                #print(s,)
                print(s,end="")

        f.write("\n")
        print()
        os.chdir("..")

    os.chdir(cwd)

    f.close()    

def autocalc():
    arg = argparse.ArgumentParser(description="ac-setup")
    arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-m","--matdir",action="store",dest="matdir",default="matdir")
    arg.add_argument("-c","--caldir",action="store",dest="caldir",default="caldir")
    arg.add_argument("-l","--matlist",action="store",dest="matlist")
    arg.add_argument("-s","--scenario",action="store",dest="scenario",default="default")
    arg.add_argument("-n","--nproc",action="store",dest="nproc",default=1)
    arg.add_argument('-q', "--qstat", dest="qstat",action="store_true")
    args = arg.parse_args()
    # ac = Autocalc()
    ac = Autocalc(args.program, args.matdir, args.caldir, args.matlist, args.nproc, args.scenario)
    # ac.program = args.program
    # ac.matdir = args.matdir
    # ac.caldir = args.caldir
    # ac.matlist = args.matlist
    # ac.nproc = args.nproc
    # ac.scenario = args.scenario
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    print("ver.",ac.version)
    print()
    print("[auto calculation]")
    print("program:",ac.program)
    print("mat:",ac.matdir)
    print("cal:",ac.caldir)
    print("scenario:",ac.scenario)
    print("nproc:",ac.nproc)
    ac.calculate()

    # update joblist
    if args.qstat:
        print("[update joblist]")
        jobs0 = jobmanage.job.jobstat()
        print("[jobs]")
        f0 = open("joblist.txt","r")
        f = open("joblist.txt.new", "w")
        for l in f0:
            l = l.strip()
            words = l.split()
            if len(words) < 3:
                continue
            matid = words[0]
            status = words[1]
            jobid = words[2]

            qstatus = jobs0[jobid][1]
            if status == "Q" and qstatus in "QR":
                print(matid, "in queue",jobid,qstatus,job0[jobid][3])
            elif status == "Q" and qstatus == "-":
                fstat = os.path.join(ac.caldir, matid, "STATUS_RUN")
                #print("statusfile= ", fstat)
                if os.path.exists(fstat):
                    current = open(fstat, "r").read()
                else:
                    current = " "
                if not "START" in current:
                    status = "W"
                    print(matid, "not in queue Q->W")

            f.write("{0} {1} {2}\n".format(matid, status, jobid))
            print("{0} {1} {2}".format(matid, status, jobid))
        f0.close()
        f.close()
        os.rename("joblist.txt.new", "joblist.txt")

if __name__ == "__main__":
    autocalc()
