import os
import numpy

from xml.etree import ElementTree
from xml.dom import minidom

def visdata(f0,f1,jsdir=None):
    print("[visdata]")
    #doc = ElementTree.parse("calculations.xml")
    doc = ElementTree.parse(f0)
    root = doc.getroot()

    if jsdir==None or jsdir=="cdn":
        jsdir0 = "https://code.jquery.com"
        jsdir1 = "https://chemapps.stolaf.edu/jmol/jsmol"
        jsdir2 = "https://cdn.plot.ly"
    elif jsdir=="local":
        jsdir = str(os.path.dirname(__file__))+"/../../js"
        jsdir0 = jsdir
        jsdir1 = jsdir+"/jmol"
        jsdir2 = jsdir

    #print(jsdir)

    #f = open("calculations.html","w")
    f = open(f1,"w")

    f.write("<html>\n");
    f.write("<head>\n");
    f.write("<meta charset=\"utf-8\">\n");
    f.write("<title>TOAST: Template Oriented Atomic Simulation Toolkit</title>\n")
    #f.write("<meta charset=\"UTF-8\">\n");
    f.write("<style type=\"text/css\">\n");
    f.write("table { empty-cells: show; font-size: small; }\n");
    #f.write("table { empty-cells: show; font-size: 12px; }\n");
    #f.write("table th{ background: #1B3AB4; color: #FFF; }\n");
    f.write("table th{ background: #1B3AB4; color: #FFF; width: 150px; }\n");
    #f.write("table td{ background: #E2E4FF; }\n");
    f.write("table td{ background: #E2E4FF; width: 150px; }\n");
    f.write("</style>\n");
    f.write("<script type=\"text/javascript\" src=\""+jsdir1+"/JSmol.min.js\"></script>\n");
    f.write("<script type=\"text/javascript\" src=\""+jsdir0+"/jquery.min.js\"></script>\n");
    f.write("<script type=\"text/javascript\" src=\""+jsdir2+"/plotly-1.44.3.min.js\"></script>\n");
    f.write("</head>\n");
    f.write("<body bgcolor=white>\n");

    f.write("<small><b>Material Information</b></small><p>");
    f.write("<p>");

    matid = None
    chemform = None
    crysys = None
    spg = None
    spgno = None
    cif = None

    e = root.find("material/material_id")
    if e!=None:
        matid = e.text
    e = root.find("material/chemical_formula")
    if e!=None:        
        chemform = e.text
    e = root.find("material/crystal_system")
    if e!=None:        
        crysys = e.text
    e = root.find("material/spacegroup")
    if e!=None:        
        spg = e.text
    e = root.find("material/spacegroup_no")
    if e!=None:        
        spgno = e.text
    e = root.find("material/cif")
    if e!=None:        
        cif = e.text

    f.write("<table>");
    f.write("<tr><th>Material ID</th><td>"+matid+"</td>");
    #f.write("<th>Substance ID</th><td>"+sub+"</td></tr>");
    f.write("<tr><th>Chemical Formula</th><td>"+chemform+"</td></tr>");
    f.write("<tr><th>CIF file</th><td><a href="+cif+">"+cif+"</a></td></tr>");
    f.write("</table>");
    f.write("<p>");

    f.write("<small><b>Crystal Structure</b></small><p>");

    f.write("<table>");
    f.write("<tr><th>Crystal System</th><td>"+crysys+"</td></tr>");
    #f.write("<tr><th>Lattice System</th><td>"+latsys+"</td></tr>");
    f.write("<tr><th>Space Group</th><td>"+spg+"</td></tr>");
    f.write("<tr><th>Space Group Number</th><td>"+spgno+"</td></tr>");
    #f.write("<tr><th>Pearson Symbol</th><td>"+"</td></tr>");
    f.write("</table>");
    f.write("<p>");

    # structure original
    ccell = []
    pcell = []
    catom = []
    patom = []
    for e0 in root.findall("structure[@name='original']"):
        for e1 in e0.findall("lattice[@type='conventional']"):
            for e2 in e1.findall("a"):
                la = e2.text
            for e2 in e1.findall("b"):
                lb = e2.text
            for e2 in e1.findall("c"):
                lc = e2.text
            for e2 in e1.findall("alpha"):
                aa = e2.text
            for e2 in e1.findall("beta"):
                ab = e2.text
            for e2 in e1.findall("gamma"):
                ac = e2.text
            for e2 in e1.findall("a1"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a2"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a3"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
        for e1 in e0.findall("lattice[@type='primitive']"):
            for e2 in e1.findall("a1"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a2"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a3"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
        for e1 in e0.findall("atom[@type='conventional']/i"):
            l0 = e1.attrib.get("name")
            l1 = e1.text.split()
            catom.append([l0,l1[0],l1[1],l1[2]])
        for e1 in e0.findall("atom[@type='primitive']/i"):
            l0 = e1.attrib.get("name")
            l1 = e1.text.split()
            patom.append([l0,l1[0],l1[1],l1[2]])

    f.write("<small><b>Cell Parameter</b></small>");
    f.write("<table>");
    f.write("<tr>");
    f.write("<th>a (A)</th><th>b (A)</th><th>c (A)</th>");
    f.write("</tr>");
    f.write("<tr>");
    f.write("<td>"+la+"</td>");
    f.write("<td>"+lb+"</td>");	    
    f.write("<td>"+lc+"</td>");	    
    f.write("</tr>");	    
    f.write("<tr>");
    f.write("<th>alpha</th><th>beta</th><th>gamma</th>");
    f.write("</tr>");
    f.write("<td>"+aa+"</td>");	    
    f.write("<td>"+ab+"</td>");	    
    f.write("<td>"+ac+"</td>");	    
    f.write("</tr>");
    f.write("</table>");

    f.write("<small><b>Atom Sites</b></small>");
    f.write("<table>");
    f.write("<tr>");	    
    f.write("<th>No</th><th>Element</th><th>Label</th><th>x</th><th>y</th><th>z</th>");
    f.write("<th>Occupancy</th><th>Symmetry Multiplicity</th><th>Wyckoff Symbol</th><th>U iso or equiv</th>");
    f.write("</tr>");
    f.write("</table>");
    f.write("<p>\n");

    cif1 = cif
    cif2 = matid+".relaxed.cif"

    f.write("<script type=\"text/javascript\">\n");
    f.write("var script = \"load "+ cif +" {1 1 1}\";\n");
    #f.write("var script = \"load XmlVasp::"+ cif +"\";\n");
    f.write("var info0 = {\n");
    f.write("width: 400,\n");
    f.write("height: 250,\n");
    f.write("antialiasDisplay: true,\n");
    f.write("disableJ2SLoadMonitor: true,\n");
    f.write("use: \"HTML5\",\n");
    #f.write("j2sPath: \"../jmol/j2s\",\n");
    f.write("j2sPath: \""+jsdir1+"/j2s\",\n");
    f.write("script: script,\n");
    if jsdir==None or jsdir=="cdn":
        f.write("serverURL: \"https://chemapps.stolaf.edu/jmol/jsmol/php/jsmol.php\",\n")
    #f.write("bondWidth: 4,\n");
    f.write("color: \"white\",\n");
    if jsdir==None or jsdir=="cdn":
        f.write("language: \"en\",\n");
        #f.write("languageTranslation: false,\n");
    f.write("showfrank: false,\n");
    f.write("disableInitialConsole: true,\n");
    f.write("}\n");
    f.write("Jmol.getApplet(\"structure1\",info0)\n");
    f.write("Jmol.jmolHtml(\"<small>\");\n");
    f.write("Jmol.jmolRadioGroup( structure1, [\n");
    f.write("[\"load "+cif+" {1 1 1}\",\"CIF file (Initial)\",\"checked\"],\n");
    f.write("[\"load "+cif+" {2 2 2}\",\"CIF file (Initial)(2x2x2)\"],\n");
    #f.write("[\"load "+cif2+" {1 1 1}\",\"CIF file (relaxed)\"],\n");
    #f.write("[\"load "+pcell1+"\",\"Primitive Cell (Initial)\"],\n");
    #f.write("[\"load "+pcell2+"\",\"Primitive Cell (Relaxed)\"],\n");
    f.write("])\n");
    f.write("Jmol.jmolHtml(\"</small>\");\n");
    f.write("delete Jmol._tracker;\n");
    f.write("</script>\n");
    f.write("<p>\n");

    # structure
    f.write("<small><b>Structure</b></small><p>");

    f.write("<small><b>Conventional Cell (Initial)</b></small>");

    f.write("<table>");
    f.write("<tr>");
    f.write("<th>a (A)</th><td>"+ccell[0][0]+"</td><td>"+ccell[0][1]+"</td><td>"+ccell[0][2]+"</td>");
    f.write("</tr>");
    f.write("<tr>");
    f.write("<th>b (A)</th><td>"+ccell[1][0]+"</td><td>"+ccell[1][1]+"</td><td>"+ccell[1][2]+"</td>");
    f.write("</tr>");	    
    f.write("<tr>");
    f.write("<th>c (A)</th><td>"+ccell[2][0]+"</td><td>"+ccell[2][1]+"</td><td>"+ccell[2][2]+"</td>");
    f.write("</tr>");
    f.write("</table>");

    f.write("<small><b>Atom Coordinates</b></small>");
    f.write("<table>");
    f.write("<tr>");	    
    f.write("<th>No</th><th>Element</th><th>x</th><th>y</th><th>z</th>");
    f.write("<th>Site ID</th><th>Operation ID</th>");
    f.write("</tr>");

    for a in catom:
        f.write("<tr>");	    
        f.write("<td>"+"</td>");
        f.write("<td>"+a[0]+"</td>");
        f.write("<td>"+a[1]+"</td>");
        f.write("<td>"+a[2]+"</td>");
        f.write("<td>"+a[3]+"</td>");
        f.write("<td>"+"</td>");
        f.write("<td>"+"</td>");
        f.write("</tr>");

    f.write("</table>");
    f.write("<p>");

    f.write("<small><b>Primitive Cell (Initial)</b></small>");

    f.write("<table>");
    f.write("<table>");
    f.write("<tr>");
    f.write("<th>a (A)</th><td>"+pcell[0][0]+"</td><td>"+pcell[0][1]+"</td><td>"+pcell[0][2]+"</td>");
    f.write("</tr>");
    f.write("<tr>");
    f.write("<th>b (A)</th><td>"+pcell[1][0]+"</td><td>"+pcell[1][1]+"</td><td>"+pcell[1][2]+"</td>");
    f.write("</tr>");	    
    f.write("<tr>");
    f.write("<th>c (A)</th><td>"+pcell[2][0]+"</td><td>"+pcell[2][1]+"</td><td>"+pcell[2][2]+"</td>");
    f.write("</tr>");
    f.write("</table>");

    f.write("<small><b>Atom Coordinates</b></small>");
    f.write("<table>");
    f.write("<tr>");	    
    f.write("<th>No</th><th>Element</th><th>x</th><th>y</th><th>z</th>");
    f.write("<th>Site ID</th><th>Operation ID</th>");
    f.write("</tr>");

    for a in patom:
        f.write("<tr>");	    
        f.write("<td>"+"</td>");
        f.write("<td>"+a[0]+"</td>");
        f.write("<td>"+a[1]+"</td>");
        f.write("<td>"+a[2]+"</td>");
        f.write("<td>"+a[3]+"</td>");
        f.write("<td>"+"</td>");
        f.write("<td>"+"</td>");
        f.write("</tr>");

    f.write("</table>");
    f.write("<p>");
    
    # structure relaxed
    ccell = []
    pcell = []
    catom = []
    patom = []
    for e0 in root.findall("structure[@name='relaxed']"):
        for e1 in e0.findall("lattice[@type='conventional']"):
            for e2 in e1.findall("a"):
                la = e2.text
            for e2 in e1.findall("b"):
                lb = e2.text
            for e2 in e1.findall("c"):
                lc = e2.text
            for e2 in e1.findall("alpha"):
                aa = e2.text
            for e2 in e1.findall("beta"):
                ab = e2.text
            for e2 in e1.findall("gamma"):
                ac = e2.text
            for e2 in e1.findall("a1"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a2"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a3"):
                l0 = e2.text.split()
                ccell.append([l0[0],l0[1],l0[2]])
        for e1 in e0.findall("lattice[@type='primitive']"):
            for e2 in e1.findall("a1"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a2"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
            for e2 in e1.findall("a3"):
                l0 = e2.text.split()
                pcell.append([l0[0],l0[1],l0[2]])
        for e1 in e0.findall("atom[@type='conventional']/i"):
            l0 = e1.attrib.get("name")
            l1 = e1.text.split()
            catom.append([l0,l1[0],l1[1],l1[2]])
        for e1 in e0.findall("atom[@type='primitive']/i"):
            l0 = e1.attrib.get("name")
            l1 = e1.text.split()
            patom.append([l0,l1[0],l1[1],l1[2]])

    f.write("<small><b>Primitive Cell (Relaxed)</b></small>");

    f.write("<table>");
    f.write("<tr>");
    f.write("<th>a (A)</th><td>"+pcell[0][0]+"</td><td>"+pcell[0][1]+"</td><td>"+pcell[0][2]+"</td>");
    f.write("</tr>");
    f.write("<tr>");
    f.write("<th>b (A)</th><td>"+pcell[1][0]+"</td><td>"+pcell[1][1]+"</td><td>"+pcell[1][2]+"</td>");
    f.write("</tr>");	    
    f.write("<tr>");
    f.write("<th>c (A)</th><td>"+pcell[2][0]+"</td><td>"+pcell[2][1]+"</td><td>"+pcell[2][2]+"</td>");
    f.write("</tr>");
    f.write("</table>");

    f.write("<small><b>Atom Coordinates</b></small>");
    f.write("<table>");
    f.write("<tr>");	    
    f.write("<th>No</th><th>Element</th><th>x</th><th>y</th><th>z</th>");
    f.write("<th>Site ID</th><th>Operation ID</th>");
    f.write("</tr>");

    for a in patom:
        f.write("<tr>");	    
        f.write("<td>"+"</td>");
        f.write("<td>"+a[0]+"</td>");
        f.write("<td>"+a[1]+"</td>");
        f.write("<td>"+a[2]+"</td>");
        f.write("<td>"+a[3]+"</td>");
        f.write("<td>"+"</td>");
        f.write("<td>"+"</td>");
        f.write("</tr>");

    f.write("</table>");
    f.write("<p>");

    f.write("<small><b>Brillouin Zone</b></small><p>");

    f.write("<small>");
    #f.write("Brillouin Zone Image File (SVG File): <a href=/"+data+"/"+matid+"/"+matid+".bz.svg>"+matid+".bz.svg</a><br>");
    #f.write("Gnuplot Script File: <a href=/"+data+"/"+matid+"/"+matid+".bz.plt>"+matid+".bz.plt</a><br>");
    # temporary
    #f.write("Brillouin Zone Image File (SVG File): <a href="+matid+".bz.svg>"+matid+".bz.svg</a><br>");
    f.write("Gnuplot Script File: <a href="+matid+".bz.plt>"+matid+".bz.plt</a><br>");
    f.write("Jmol Script File: <a href="+matid+".bz.spt>"+matid+".bz.spt</a><br>");
    f.write("</small>\n");

    #f.write("<object data=/datas/"+matid+"/"+matid+".bz.svg width=400 height=300></object>");

    spt = matid+".bz.spt";
    f.write("<script type=\"text/javascript\">\n");
    f.write("var script = \"load "+ spt +"\";\n");
    f.write("var info1 = {\n");
    f.write("width: 400,\n");
    f.write("height: 250,\n");
    f.write("antialiasDisplay: true,\n");
    f.write("disableJ2SLoadMonitor: true,\n");
    f.write("use: \"HTML5\",\n");
    #f.write("j2sPath: \"../jmol/j2s\",\n");
    f.write("j2sPath: \""+jsdir1+"/j2s\",\n");
    f.write("script: script,\n");
    if jsdir==None or jsdir=="cdn":
        f.write("serverURL: \"https://chemapps.stolaf.edu/jmol/jsmol/php/jsmol.php\",\n")
    #f.write("bondWidth: 4,\n");
    f.write("color: \"white\",\n");
    if jsdir==None or jsdir=="cdn":
        f.write("language: \"en\",\n");
        #f.write("languageTranslation: false,\n");
    f.write("showfrank: false,\n");
    f.write("disableInitialConsole: true,\n");
    f.write("}\n");
    f.write("Jmol.getApplet(\"bz1\",info1)\n");
    f.write("delete Jmol._tracker;\n");
    f.write("</script>\n");

    f.write("<p>\n");

    etotal = None
    etotal_atom = None
    efermi = None
    efermi2 = None
    bandgap_type = None
    bandgap = None

    for e0 in root.findall("calculation[@name='scf']/properties/i"):
        if e0.attrib.get("name") == "etotal":
            etotal = e0.text
        if e0.attrib.get("name") == "etotal_per_atom":
            etotal_atom = e0.text
        if e0.attrib.get("name") == "efermi":
            efermi = e0.text
    #for e0 in root.findall("calculation[@name='dos' or @name='nscf']/properties/i"):
    for e0 in root.findall("calculation[@name='dos']/properties/i") \
            + root.findall("calculation[@name='nscf']/properties/i"):
        if e0.attrib.get("name") == "efermi":
            efermi2 = e0.text
        if e0.attrib.get("name") == "bandgap_type":
            bandgap_type = e0.text
        if e0.attrib.get("name") == "bandgap":
            bandgap = e0.text

    print("etotal",etotal,etotal_atom)
    print("efermi",efermi,efermi2)
    print("bandgap",bandgap_type,bandgap)

    f.write("<small><b>Energy</b> (scf)</small><br>");

    f.write("<table>");
    f.write("<tr><th>Total Energy (eV)</th><td>"+etotal+"</td></tr>");
    f.write("<tr><th>Total Energy (eV/atom)</th><td>"+etotal_atom+"</td></tr>");
    f.write("<tr><th>Fermi Energy (eV/atom)</th><td>"+efermi+"</td></tr>");
    #f.write("<tr><th>Fermi Energy (eV/atom)</th><td>"+efermi2+"</td></tr>");
    #f.write("<tr><th>Magnetization (uB)</th><td>"+mag1+"</td></tr>");
    f.write("<tr/>");
    #de = float(efermi2)-float(efermi)
    if efermi2!=None:
        de = float(efermi2)-float(efermi)
    else:
        efermi2=""
        de=""
    f.write("<tr><th>Fermi Energy (DOS) (eV/atom)</th><td>"+efermi2+"</td><td>"+str(de)+"</td></tr>");
    #if bandgap_type == "metal":
    #    f.write("<tr><th>Fermi Energy (FS) (eV/atom)</th><td>"+efermi3+"</td><td>"+df.format(efermi3-efermi)+"</td></tr>");

    f.write("</table>");
    f.write("<p>");

    # band
    f.write("<small><b>Band Structure</b></small><br>\n");
    f.write("<p>\n");

    ispin = None
    nbands = 0
    nkpoints = 0;

    primcell = []
    recipcell = []
    ispin = None
    nbands = None
    nkpoints = None
    nelect = None
    efermi = None
    kp = []  # kx,ky,kz
    ke = []  # spin,kp,band,[ene,occ]
    for e0 in root.findall("structure[@name='relaxed']/lattice[@type='primitiive']"):
        for e1 in e0.findall("a1"):
            l = e1.text.split()
            primcell.append([float(l[0]),float(l[1]),float(l[2])])
        for e1 in e0.findall("a2"):
            l = e1.text.split()
            primcell.append([float(l[0]),float(l[1]),float(l[2])])
        for e1 in e0.findall("a3"):
            l = e1.text.split()
            primcell.append([float(l[0]),float(l[1]),float(l[2])])
    for e0 in root.findall("structure[@name='relaxed']/lattice[@type='reciprocal']"):
        for e1 in e0.findall("a1"):
            l = e1.text.split()
            recipcell.append([float(l[0]),float(l[1]),float(l[2])])
        for e1 in e0.findall("a2"):
            l = e1.text.split()
            recipcell.append([float(l[0]),float(l[1]),float(l[2])])
        for e1 in e0.findall("a3"):
            l = e1.text.split()
            recipcell.append([float(l[0]),float(l[1]),float(l[2])])
    for e0 in root.findall("calculation[@name='band']/parameters/i"):
        if e0.attrib.get("name") == "ispin":
            ispin = int(e0.text)
        if e0.attrib.get("name") == "nbands":
            nbands = int(e0.text)
        if e0.attrib.get("name") == "nkpoints":
            nkpoints = int(e0.text)
        if e0.attrib.get("name") == "nelect":
            nelect = float(e0.text)
    for e0 in root.findall("calculation[@name='band']/kpoints/i"):
        l = e0.text.split()
        if e0.attrib.get("name"):
            kp0 = e0.attrib.get("name")
            kp.append([float(l[0]),float(l[1]),float(l[2]),kp0])
        else:
            kp.append([float(l[0]),float(l[1]),float(l[2])])
    for e0 in root.findall("calculation[@name='band']/properties/band/spin"):
        if e0.attrib.get("spin") == "1":
            ke.append([])
            for i,e1 in enumerate(e0.findall("kpoint")):
                ke[0].append([])
                for e2 in e1.findall("i"):
                    l = e2.text.split()
                    ke[0][i].append([float(l[0]),float(l[1])])
        if e0.attrib.get("spin") == "2":
            ke.append([])
            for i,e1 in enumerate(e0.findall("kpoint")):
                ke[1].append([])
                for e2 in e1.findall("i"):
                    l = e2.text.split()
                    ke[1][i].append([float(l[0]),float(l[1])])
    for e0 in root.findall("calculation[@name='band']/properties/i[@name='efermi']"):
        efermi = float(e0.text)
    for e0 in root.findall("calculation[@name='scf']/properties/i[@name='efermi']"):
        efermi2 = float(e0.text)

    print("ispin",ispin)
    print("nbands",nbands)
    print("nkpoints",nkpoints)
    print("nelect",nelect)
    print("efermi",efermi,efermi2)

    primcell = numpy.array(primcell,float)
    recipcell = numpy.array(recipcell,float)
    print("primcell",primcell)
    print("recipcell",recipcell)

    kp0 = []
    kp1 = []
    dk = 0.0
    for i,k in enumerate(kp):
        if i==0:
            k0 = numpy.array(kp[i][0:3],float)
            #k0 = numpy.dot(recipcell,k0)
            k0 = numpy.dot(recipcell.T,k0)
        else:
            k0 = numpy.array(kp[i-1][0:3],float)
            #k0 = numpy.dot(recipcell,k0)
            k0 = numpy.dot(recipcell.T,k0)
        k1 = numpy.array(kp[i][0:3],float)
        #k1 = numpy.dot(recipcell,k1)
        k1 = numpy.dot(recipcell.T,k1)
        dk0 = numpy.sqrt(numpy.dot(k1-k0,k1-k0))
        if i>0 and len(kp[i-1])==4 and len(kp[i])==4:
            dk0 = 0.0
        dk = dk + dk0 
        kp0.append(dk)
        if len(k)==4:
            kp1.append([k[3],dk])

    xmin = min(kp0)
    xmax = max(kp0)
    #print(kp1)
	    
    # plotly
    for i in range(ispin):
        f.write("<div id=\"band"+str(i)+"\" style=\"width: 600px; height: 500px; margin: 0 align: left\" auto></div>\n");
        f.write("<script type=\"text/javascript\">\n")

        for j in range(nbands):
            f.write("var band"+str(j)+" = {\n")
            f.write("x: [")
            for k in range(nkpoints):
                f.write(str(kp0[k]))
                if(k<nkpoints-1):
                    f.write(",")
            f.write("],\n")
            f.write("y: [")
            for k in range(nkpoints):
                d = ke[i][k][j][0]
                #d -= efermi
                d -= efermi2
                f.write(str(d))
                if(k<nkpoints-1):
                    f.write(",")
            f.write("],\n")
            f.write("name: 'band "+str(j+1)+"',")
            f.write("mode: 'lines'};\n")

        f.write("var data = [")
        for j in range(nbands):
            f.write("band"+str(j))
            if(j<nbands-1):
                f.write(",")
        f.write("];\n")
        f.write("var layout = {\n")
        f.write("title: 'Band Structure (spin"+str(i+1)+")',")
        f.write("xaxis: {")
        f.write("title: {text: 'K Point'},")

        f.write("tickvals: [")
        if len(kp1)>0:
            for j,k in enumerate(kp1):
                f.write(str(k[1]))
                if(j<len(kp1)-1):
                    f.write(",");
        f.write("],\n")
        f.write("ticktext: [")
        if len(kp1)>0:
            for j,k in enumerate(kp1):
                f.write("'"+k[0]+"'")
                if(j<len(kp1)-1):
                    f.write(",");
        f.write("],\n")

        f.write("},")
        f.write("yaxis: {")
        f.write("title: {text: 'Energy (E-Ef) (eV)'},")
        f.write("},")
        f.write("showlegend: true,")
        #f.write("legend: {\"orientation\": \"h\",}")
        f.write("hovermode: 'closest',")
        f.write("};\n")
        f.write("Plotly.newPlot(\"band"+str(i)+"\",data,layout,{showSendToCloud: false});\n");
        f.write("</script>\n")

    # dos
    f.write("<small><b>Density of State</b></small><br>\n")
    f.write("<p>\n")

    ispin = None
    natoms = None
    ndos = 0
    npdos = 0
    tdos = [] # spin,[ene,dos]
    pdos = [] # atom,spin,[ene,pdos]
    pdoscol = []

    e0 = root.find(".//calculation[@name='scf']/parameters/i[@name='ispin']")
    if e0!=None:
        ispin = int(e0.text)
    e0 = root.findall(".//structure[@name='original']/atom[@type='primitive']/i")
    if e0!=None:
        natoms = len(e0)

    #for e0 in root.findall(".//calculation[@name='dos' or @name='tdos']/properties/dos/spin"):
    for e0 in root.findall(".//calculation[@name='dos']/properties/dos/spin") \
            + root.findall(".//calculation[@name='tdos']/properties/dos/spin"):
        if e0.attrib.get("spin") == "1":
            tdos.append([])
            for i,e1 in enumerate(e0.findall("i")):
                l = e1.text.split()
                tdos[0].append([float(l[0]),float(l[1])])
        if e0.attrib.get("spin") == "2":
            tdos.append([])
            for i,e1 in enumerate(e0.findall("i")):
                l = e1.text.split()
                tdos[1].append([float(l[0]),float(l[1])])

    #for e0 in root.findall(".//calculation[@name='dos' or @name='tdos']/properties/i[@name='efermi']"):
    for e0 in root.findall(".//calculation[@name='dos']/properties/i[@name='efermi']") \
            + root.findall(".//calculation[@name='tdos']/properties/i[@name='efermi']"):
        efermi = float(e0.text)
    for e0 in root.findall(".//calculation[@name='scf']/properties/i[@name='efermi']"):
        efermi2 = float(e0.text)

    #for e0 in root.findall(".//calculation[@name='dos' or @name='tdos']/properties/pdos/atom"):
    for e0 in root.findall(".//calculation[@name='dos']/properties/pdos/atom") \
            + root.findall(".//calculation[@name='tdos']/properties/pdos/atom") \
            + root.findall(".//calculation[@name='pdos']/properties/pdos/atom"):
        for e1 in e0.findall("i[@name='value']"):
            l = e1.text.split()
            pdoscol.append(l)
        pdos.append([])
        for e1 in e0.findall("spin"):
            if e1.attrib.get("spin") == "1":
                pdos[-1].append([])
                for e2 in e1.findall("i"):
                    l = e2.text.split()
                    #pdos[-1][0].append(map(float,l))
                    pdos[-1][0].append(list(map(float,l)))
            if e1.attrib.get("spin") == "2":
                pdos[-1].append([])
                for e2 in e1.findall("i"):
                    l = e2.text.split()
                    #pdos[-1][1].append(map(float,l))
                    pdos[-1][1].append(list(map(float,l)))

    print("ispin",ispin)
    print("natoms",natoms)
    print("efermi",efermi,efermi2)
    #print("pdoscol",pdoscol)
    #print("pdos",len(pdos))

    # tdos
    for i in range(ispin):
        f.write("<div id=\"dos"+str(i)+"\" style=\"width: 600px; height: 400px; margin: 0 align: left\" auto></div>\n");
        f.write("<script type=\"text/javascript\">\n")
        f.write("var tdos = {\n")
        f.write("x: [")
        for j in range(len(tdos[0])):
            e = tdos[i][j][0]-efermi2
            #e = tdos[i][j][0]
            f.write(str(e));
            if j<len(tdos[i])-1:
                f.write(",");
        f.write("],\n")
        f.write("y: [")
        for j in range(len(tdos[0])):
            f.write(str(tdos[i][j][1]));
            if j<len(tdos[i])-1:
                f.write(",");
        f.write("],\n")
        f.write("name: 'Total DOS',")
        f.write("mode: 'lines'};\n")
        f.write("var data = [tdos];\n")
        f.write("var layout = {\n")
        f.write("title: 'Density of State (spin"+str(i+1)+")',")
        f.write("xaxis: {")
        f.write("title: {text: 'Energy (E-Ef) (eV)'},")
        f.write("},")
        f.write("yaxis: {")
        f.write("title: {text: 'Total DOS'},")
        f.write("},")
        f.write("showlegend: true,")
        #f.write("legend: {\"orientation\": \"h\",}")
        f.write("hovermode: 'closest',")
        f.write("};\n")
        f.write("Plotly.newPlot(\"dos"+str(i)+"\",data,layout,{showSendToCloud: false});\n");
        f.write("</script>\n")

    f.write("<details>\n")
    f.write("<summary><small><b>Partial Density of State</b></small></summary>\n")

    # pdos
    for i in range(natoms):
        for j in range(ispin):
            f.write("<div id=\"pdos"+str(i)+str(j)+"\" style=\"width: 600px; height: 400px; margin: 0 align: left\" auto></div>\n");
            f.write("<script type=\"text/javascript\">\n")
            for k in range(1,len(pdoscol[i])):
                f.write("var pdos"+str(k)+" = {\n")
                f.write("x: [")
                for l in range(len(pdos[i][j])):
                    e = pdos[i][j][l][0]-efermi2
                    #e = pdos[i][j][l][0]
                    f.write(str(e));
                    if l<len(pdos[i][j])-1:
                        f.write(",");
                f.write("],\n")
                f.write("y: [")
                for l in range(len(pdos[i][j])):
                    f.write(str(pdos[i][j][l][k]));
                    if l<len(pdos[i][j])-1:
                        f.write(",");
                f.write("],\n")
                f.write("name: '"+pdoscol[i][k]+"',")
                f.write("mode: 'lines'};\n")
            f.write("var data = [")
            for k in range(1,len(pdoscol[i])):
                f.write("pdos"+str(k))
                if k<len(pdoscol[i])-1:
                   f.write(",")
            f.write("];\n")
            f.write("var layout = {\n")
            f.write("title: 'Partial Density of State ("+"Atom "+str(i+1)+", Spin "+str(j+1)+")',");
            f.write("xaxis: {")
            f.write("title: {text: 'Energy (E-Ef) (eV)'},")
            f.write("},")
            f.write("yaxis: {")
            f.write("title: {text: 'Partial DOS'},")
            f.write("},")
            f.write("showlegend: true,")
            #f.write("legend: {\"orientation\": \"h\",}")
            f.write("hovermode: 'closest',")
            f.write("};\n")
            f.write("Plotly.newPlot(\"pdos"+str(i)+str(j)+"\",data,layout,{showSendToCloud: false});\n");
            f.write("</script>\n")

    f.write("</details>\n")
    f.write("<p>\n");

    f.write("<small><b>Charge Density</b></small><p>\n");

    f.write("<small>");
    f.write("Charge Density Data File (Gaussian Cube File): <a href="+matid+".charge.cube>"+matid+".charge.cube</a><br>");
    f.write("</small>");
    f.write("<p>\n");

    # property
    f.write("<small><b>Electronic Property</b></small><p>\n");

    f.write("<table>");    
    #f.write("<tr><th>Total Energy (eV)</th><td>"+etotal+"</td></tr>");
    #f.write("<tr><th>Total Energy (eV/atom)</th><td>"+etotal_atom+"</td></tr>");
    #f.write("<tr><th>Fermi Energy (eV/atom)</th><td>"+efermi+"</td></tr>");
    ##f.write("<tr><th>Fermi Energy (eV/atom)</th><td>"+efermi2+"</td></tr>");
    #f.write("<tr><th>Magnetization (uB)</th><td>"+mag1+"</td></tr>");
    ##f.write("<tr><th>Formation Energy (eV/atom)</th><td>"+eform+"</td></tr>");
    f.write("<tr><th>Band Gap (eV)</th><td>");
    if bandgap and float(bandgap)>0.0:
        f.write(bandgap)
    f.write("</td></tr>");
    #f.write("<tr><th>Band Gap Type</th><td>"+bandgap_type+"</td></tr>");
    f.write("<tr><th>Band Gap Type</th><td>")
    if bandgap_type:
        f.write(bandgap_type)
    f.write("</td></tr>");
    f.write("</table>");
    f.write("<p>\n");

    f.write("</body>\n");
    f.write("</html>\n");

    f.close()

if __name__ == "__main__":
    f0 = "calculations.xml"
    f1 = "calculations.html"
    jsdir = None
    visdata(f0,f1,jsdir)
