import os
import shutil
import math
import numpy
import collections

#from calculator import *
#import calculation.calculator
from calculation.calculator import *

def _get_structure_from_input(fname, natom):
    '''get structure from espresso input file.
       This routine assumes that ntyp = 0 and atomis positions are in crystal coordinates
    '''
    cell = []
    atoms = []
    f = open(fname, "r")
    for line in f:
        if "CELL_PARAMETERS" in line:
            for i in range(3):
                #cell.append(f.next().split()[0:3])
                cell.append(next(f).split()[0:3])
        if "ATOMIC_POSITIONS" in line:
            for i in range(natom):
                #atoms.append(f.next().split()[0:4])
                atoms.append(next(f).split()[0:4])
    return cell, atoms


class Espresso(Calculator):

    def __init__(self):
        super(Espresso,self).__init__()
        #print("espresso")
        self.scenario = "espresso.para.cfg"
        #self.nkp_max = None

    def calc(self):
        print("espresso...",self.matid)

    def inputs(self, task):
        name = task.name
        task.input["prefix"]=self.matid
        [kx,ky,kz] = self.nkp
        #task.input["K_POINTS"]="automatic"
        task.input["K_POINTS"]="automatic\n %d %d %d 0 0 0" % (kx,ky,kz)

        #for k,v in task.input.iteritems():
        for k,v in task.input.items():
            if k.find("_namelist")>=0:
                if v == "system":
                    task.input["nat"]=self.natom
                    task.input["ntyp"]=self.nele
                    task.input["nspin"]=self.spin
                    task.input["nbnd"]=self.nbands

        if name == "spin":
            task.input["nspin"]=2
        elif name == "opt1":
            pass
        elif name == "opt2":
            pass
        elif name == "scf":
            pass
        elif name == "nscf":
            task.input["tstress"]=""
            task.input["tprnfor"]=""
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                task.input["K_POINTS"]="automatic\n %d %d %d 0 0 0" % (kx*2,ky*2,kz*2)
        elif name == "tdos":
            task.input["emin"]=self.emin 
            task.input["emax"]=self.emax 
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                task.input["K_POINTS"]="automatic\n %d %d %d 0 0 0" % (kx*2,ky*2,kz*2)
        elif name == "pdos":
            task.input["emin"]=self.emin 
            task.input["emax"]=self.emax 
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                task.input["K_POINTS"]="automatic\n %d %d %d 0 0 0" % (kx*2,ky*2,kz*2)
        elif name == "dos_band":
            pass
        elif name == "dos_band2":
            if self.spin == 1:
                task.input["spin_component"]=1
            pass
        elif name == "band":
            task.input["tstress"]=""
            task.input["tprnfor"]=""
            task.input["forc_conv_thr"]=""
            task.input["K_POINTS"]="crystal"
        elif name == "band_plot":
            pass
        elif name == "band_plot2":
            if self.spin == 1:
                task.input["spin_component"]=1
            pass

        # f1 = "job.in"
        f1 = task.option.get("_input_file", "job.in")
        self.input_calc(task.input,f1)

    def potentials(self,ele):
        for e in ele:
            pp = self.pp[e]
            shutil.copy2(self.ppdir+"/"+pp,".")

    def input_calc(self,inp,f):
        f = open(f,"w")
        #for k,v in inp.iteritems():
        for k,v in inp.items():
            if k.find("_namelist")>=0:
                if k != "_namelist" and k != "_namelist_1":
                    f.write("/\n")
                f.write("&%s\n" % (v))
                #if v == "cell":
                #    f.write("/\n")
            elif k=="prefix":
                f.write("!%s = %s\n" % (k,v))
            elif k=="nspin":
                f.write("%s = %s\n" % (k,v))
                # temporary
                if v == 2:
                    # initial_magnetizations for each atomic type
                    for i in range(len(self.ele)):
                        f.write("starting_magnetization(%d) = %s\n" % (i+1,1.0))
            elif k == "K_POINTS":
                pass
            elif (v is not None) and (len(str(v)) > 0): # key without value is set to None (Python2.7)
                f.write("%s = %s\n" % (k,v))
        f.write("/\n")

        # cell_parameters
        f.write("CELL_PARAMETERS angstrom\n")
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[0][0],self.cell[0][1],self.cell[0][2]))
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[1][0],self.cell[1][1],self.cell[1][2]))
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[2][0],self.cell[2][1],self.cell[2][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[0][0],self.cell[0][1],self.cell[0][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[1][0],self.cell[1][1],self.cell[1][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[2][0],self.cell[2][1],self.cell[2][2]))

        # atom
        f.write("ATOMIC_SPECIES\n")
        for e in self.ele:
            f.write("%s %s %s\n" % (e,self.elements[e][1],self.pp[e]))
        f.write("ATOMIC_POSITIONS crystal\n")
        for a in self.atom:
            #f.write("%s %10.5f %10.5f %10.5f\n" % (a[0],a[1],a[2],a[3]))
            f.write("%s %15.10f %15.10f %15.10f\n" % (a[0],a[1],a[2],a[3]))

        # kpoint
        #if "K_POINTS" in inp and inp["K_POINTS"]=="automatic":
        #    f.write("K_POINTS automatic\n")
        #    f.write("%d %d %d 0 0 0\n" % (self.nkp[0],self.nkp[1],self.nkp[2]))
        if "K_POINTS" in inp and "automatic" in inp["K_POINTS"]:
            f.write("K_POINTS %s\n" % inp["K_POINTS"])
        elif "K_POINTS" in inp and inp["K_POINTS"]=="crystal":
            f.write("K_POINTS crystal\n")
            f.write("%d\n" % (len(self.kpath)))
            for k in self.kpath:
                if len(k)>4:
                    #f.write("%10.5f %10.5f %10.5f %5.2f ! %s\n" % (k[0],k[1],k[2],k[3],k[4]))
                    f.write("%15.10f %15.10f %15.10f %5.2f ! %s\n" % (k[0],k[1],k[2],k[3],k[4]))
                else:
                    #f.write("%10.5f %10.5f %10.5f %5.2f\n" % (k[0],k[1],k[2],k[3]))
                    f.write("%15.10f %15.10f %15.10f %5.2f\n" % (k[0],k[1],k[2],k[3]))

        f.close()


    def nbands(self,ele,a):
        natom = 0
        za = 0
        nbands = 0
        for i in range(len(a)):
            natom += int(a[i])
        zv = []
        for e in ele:
            #print(e)
            pp = self.pp[e]
            PP = open(pp)
            for l in PP:
                if l.find("Z valence")>=0:
                    z = l.split()
                    zval = z[0]
                    zv.append(zval)
                elif l.find("z_valence")>=0:
                    z = l.split("=")
                    zval = z[1]
                    zval = zval.replace("\"","")
                    zv.append(zval)
        for i in range(len(a)):
            za += float(zv[i]) * float(a[i])
        nelect = za
        print("nelect:",nelect)
        nbands = int(nelect/2)+natom*2
        nbands = int((nelect)*0.6+0.5)+max(4,natom*2)
        if(nbands%2 == 1): nbands+=1
        print("nbands:",nbands)
        return nbands

    def structure(self,name, d):
        if name == "cif":
            pass
        else:
            # d = self.calc[name]["_calc_dir"]
            cell = []
            atom = []
            cell1 = None
            atom1 = None
            # job.in
            c, a = _get_structure_from_input("../"+d+"/job.in", self.natom)
            if c and a:
                cell.append(c)
                atom.append(a)
            # log
            f0 = open("../"+d+"/log")
            for l in f0:
                if l.find("CELL_PARAMETERS")>=0:
                    cell.append([])
                    for i in range(3):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        #print(i,l0)
                        cell[-1].append(l0)
                if l.find("ATOMIC_POSITIONS")>=0:
                    atom.append([])                
                    for i in range(self.natom):
                        #c = f0.next()
                        c = next(f0)
                        l0 = c.split()
                        #print(i,l0)
                        atom[-1].append(l0)
            f0.close()

            if len(cell) > 0:
                cell1 = cell[-1]
                cell1 = numpy.array(cell1,float)

            if len(atom) > 0:
                atom1 = atom[-1]
                for i,a in enumerate(atom1):
                    atom1[i][1:] = numpy.array(a[1:],float)


            qever = "qe62"
            if(os.path.exists("../"+d+"/pwscf.save/data-file.xml")):
                qever = "qe60"
            if(os.path.exists("../"+d+"/pwscf.xml")):
                qever = "qe62"

            # pwscf.xml | data-file.xml
            cell01 = []
            atom01 = []
            if(qever=="qe60"):
                doc = ElementTree.parse("../"+d+"/pwscf.save/data-file.xml")
                root = doc.getroot()
                for e in root.findall("CELL/DIRECT_LATTICE_VECTORS"):
                    for e1 in e.findall("a1"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                    for e1 in e.findall("a2"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                    for e1 in e.findall("a3"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                for e in root.findall("IONS"):
                    for e1 in e.getchildren():
                        if e1.tag.find("ATOM.")>=0:
                            l0 = e1.attrib.get("SPECIES").strip()
                            l1 = e1.attrib.get("tau").split()
                            atom01.append([l0,l1[0],l1[1],l1[2]])
            if(qever=="qe62"):
                doc = ElementTree.parse("../"+d+"/pwscf.xml")
                root = doc.getroot()
                for e in root.findall(".//output/atomic_structure/cell"):
                    for e1 in e.findall("a1"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                    for e1 in e.findall("a2"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                    for e1 in e.findall("a3"):
                        l0 = e1.text.split()
                        cell01.append(l0)
                for e in root.findall(".//output/atomic_structure/atomic_positions/atom"):
                    l0 = e.attrib.get("name").strip()
                    l1 = e.text.split()
                    atom01.append([l0,l1[0],l1[1],l1[2]])

            cell01 = numpy.array(cell01,float)
            transmat = numpy.linalg.inv(numpy.transpose(cell01))
            for i,a in enumerate(atom01):
                a0 = numpy.array(a[1:],float)
                atom01[i][1:] = numpy.dot(transmat,a0)
            cell01 = cell01 * unit_bohr_angstrom

            if cell1 is None:
                cell1 = cell01
            if atom1 is None:
                atom1 = atom01

            self.cell0 = self.cell
            if cell1 is not None:
                self.cell = cell1
            self.cell1 = self.cell

            self.atom0 = self.atom
            if atom1 is not None:
                self.atom = atom1
            self.atom1 = self.atom

    def mag(self, d):
        mag = 0.0
        # d = self.calc["check_spin"]["_calc_dir"]
        f = open("../"+d+"/log")
        for l in f:
            if l.find("absolute magnetization")>=0:
                l0 = l.split()
                mag = l0[3]
        mag = float(mag)
        #print("mag:",mag)
        return mag

    def efermi(self, d):
        efermi = 0.0
        # d = self.calc["scf"]["_calc_dir"]
        f = open("../"+d+"/log")
        for l in f:
            if l.find("the Fermi energy is")>=0:
                l0 = l.split()
                efermi = l0[4]
        efermi = float(efermi)
        print("efermi:",efermi)
        return efermi

    def results(self, task, f):
        name = task.name
        d = task.option["_calc_dir"]

        etotal = None
        efermi = None        
        mag = None
        cell = []
        atom = []
        cell0 = None
        cell1 = None
        atom0 = None
        atom1 = None

        ispin = self.spin
        nbands = self.nbands
        nelect = self.nelect
        nkpoints = None
        step = []

        # log
        #f0 = open("../"+d+"/log")
        f0 = open("log")
        for l in f0:
            if l.find("number of Kohn-Sham states")>=0:
                l0 = l.split("=")
                nbands = l0[1].strip()
            if l.find("number of electrons")>=0:
                l0 = l.split("=")
                nelect = l0[1].strip()
            if l.find("number of k points")>=0:
                l0 = l.split()
                nkpoints = l0[4]

            if l.find("!    total energy")>=0:
                l0 = l.split()
                etotal = l0[4]
            if l.find("the Fermi energy is")>=0:
                l0 = l.split()
                efermi = l0[4]
            if l.find("absolute magnetization")>=0:
                l0 = l.split()
                mag = l0[3]

            if l.find("CELL_PARAMETERS")>=0:
                cell.append([])
                for i in range(3):
                    #c = f0.next()
                    c = next(f0)
                    l0 = c.split()
                    #print(i,l0[0],l0[1],l0[2])
                    cell[-1].append([l0[0],l0[1],l0[2]])
            if l.find("ATOMIC_POSITIONS")>=0:
                atom.append([])                
                for i in range(self.natom):
                    #c = f0.next()
                    c = next(f0)
                    l0 = c.split()
                    #print(i,l0[0],l0[1],l0[2],l0[3])
                    atom[-1].append([l0[0],l0[1],l0[2],l0[3]])
            if l.find("Self-consistent Calculation")>=0:
                step.append([])
            if l.find("     total energy              =")>=0:
                l0 = l.split()
                step[-1].append(l0[3])
        f0.close()

        print("ispin:",ispin)
        print("nbands:",nbands)
        print("nelect:",nelect)
        print("nkpoints:",nkpoints)

        if etotal is not None:
            etotal = float(etotal)
            etotal = etotal * unit_hartree_ev * 0.5
        if len(step)>0:
            for i in range(len(step)):
                if len(step[i])>0:
                    e = numpy.array(step[i],float)
                    step[i] = e * unit_hartree_ev * 0.5                        

        print("etotal:",etotal)
        print("efermi:",efermi)
        print("mag:",mag)
        print("nstep:",len(step))

        cell0 = self.cell
        if len(cell) > 0:
            cell1 = cell[-1]
        atom0 = self.atom
        if len(atom) > 0:
            atom1 = atom[-1]

        #print("cell:")
        for i in range(len(cell)):
            for j in range(len(cell[i])):
                #print(i,j,cell[i][j])
                pass
        #print(cell0)
        #print(cell1)
        #print("atom:")
        for i in range(len(atom)):
            for j in range(len(atom[i])):
                #print(i,j,atom[i][j])
                pass
        #print(atom0)
        #print(atom1)

        qever = None
        if(os.path.exists("pwscf.save/data-file.xml")):
            qever = "qe60"
        if(os.path.exists("pwscf.xml")):
            qever = "qe62"
        #print(qever)

        # structure
        cell01 = []
        atom01 = []
        # pwscf.xml | data-file.xml
        if(qever=="qe60"):
            doc = ElementTree.parse("pwscf.save/data-file.xml")
            root = doc.getroot()
            for e0 in root.findall("CELL/DIRECT_LATTICE_VECTORS"):
                for e1 in e0.findall("a1"):
                    l0 = e1.text.split()
                    cell01.append(l0)
                for e1 in e0.findall("a2"):
                    l0 = e1.text.split()
                    cell01.append(l0)
                for e1 in e0.findall("a3"):
                    l0 = e1.text.split()
                    cell01.append(l0)
            for e in root.findall("IONS"):
                for e1 in e0.getchildren():
                    if e1.tag.find("ATOM.")>=0:
                        l0 = e1.attrib.get("SPECIES")
                        l1 = e1.attrib.get("tau").split()
                        atom01.append([l0,l1[0],l1[1],l1[2]])
        if(qever=="qe62"):
            doc = ElementTree.parse("pwscf.xml")
            root = doc.getroot()
            for e in root.findall(".//output/atomic_structure/cell"):
                for e1 in e.findall("a1"):
                    l0 = e1.text.split()
                    cell01.append(l0)
                for e1 in e.findall("a2"):
                    l0 = e1.text.split()
                    cell01.append(l0)
                for e1 in e.findall("a3"):
                    l0 = e1.text.split()
                    cell01.append(l0)
            for e in root.findall(".//output/atomic_structure/atomic_positions/atom"):
                l0 = e.attrib.get("name").strip()
                l1 = e.text.split()
                atom01.append([l0,l1[0],l1[1],l1[2]])

        print(cell01)
        print(atom01)
        cell01 = numpy.array(cell01,float)
        for i,a in enumerate(atom01):
            a0 = numpy.array(a[1:],float)
            atom01[i][1:] = numpy.dot(numpy.linalg.inv(cell01),a0)
        cell01 = cell01 * unit_bohr_angstrom

        if not cell1:
            cell1 = cell01
        if not atom1:
            atom1 = atom01
        #print(cell1)
        #print(atom1)

        # kpoint
        nkp = None
        f0 = open("job.in")
        for l in f0:
            if l.find("K_POINTS")>=0 and l.find("automatic")>=0:
                #l = f0.next()
                l = next(f0)
                l0 = l.split()
                nkp = [l0[0],l0[1],l0[2]]
        f0.close()

        # kpoint
        kp = None
        if name == "nscf" or name == "band":
            kp = []
            if(qever=="qe60"):
                for e0 in root.findall("BRILLOUIN_ZONE"):
                    for e1 in e0.getchildren():
                        if e1.tag.find("K-POINT.")>=0:
                            l = e1.attrib.get("XYZ")
                            l0 = l.split()
                            kp.append([float(l0[0]),float(l0[1]),float(l0[2])])
            if(qever=="qe62"):
                for e0 in root.findall(".//ks_energies/k_point"):
                    l0 = e0.text.split()
                    kp.append([float(l0[0]),float(l0[1]),float(l0[2])])

            for i in range(len(kp)):
                #print(kp[i])
                pass

        ke = None
        # eigenval
        if name == "nscf" or name == "band":
            ke = []
            ke.append([])
            if ispin == 2:
                ke.append([])

            if(qever=="qe60"):
                for i in range(int(nkpoints)):
                    d = "pwscf.save/K%05d/" % (i+1)
                    if ispin == 1:
                        f0 = d+"eigenval.xml"
                    elif ispin == 2:
                        f0 = d+"eigenval1.xml"
                    #print(f0)
                    ke[0].append([])
                    doc3 = ElementTree.parse(f0)
                    root3 = doc3.getroot()
                    for e0 in root3.findall("EIGENVALUES"):
                        l0 = e0.text.split()
                        #print(l0)
                        for e1 in l0:
                            e2 = float(e1)*unit_hartree_ev
                            ke[0][-1].append([e2,None])
                    for e0 in root3.findall("OCCUPATIONS"):
                        l0 = e0.text.split()
                        #print(l0)
                        for j,e1 in enumerate(l0):
                            ke[0][-1][j][1] = float(e1)

                    if ispin == 2:
                        f0 = d+"eigenval2.xml"
                        #print(f0)
                        ke[1].append([])
                        doc3 = ElementTree.parse(f0)
                        root3 = doc3.getroot()
                        for e0 in root3.findall("EIGENVALUES"):
                            l0 = e0.text.split()
                            #print(l0)
                            for e1 in l0:
                                e2 = float(e1)*unit_hartree_ev
                                ke[1][-1].append([e2,None])
                        for e0 in root3.findall("OCCUPATIONS"):
                            l0 = e0.text.split()
                            #print(l0)
                            for j,e1 in enumerate(l0):
                                ke[1][-1][j][1] = float(e1)

            if(qever=="qe62"):
                f0 = "pwscf.xml"
                #print(f0)
                doc3 = ElementTree.parse(f0)
                root3 = doc3.getroot()
                for e0 in root3.findall(".//ks_energies"):
                    ke[0].append([])
                    if ispin == 2:
                        ke[1].append([])
                    for e1 in e0.findall("eigenvalues"):
                        l0 = e1.text.split()
                        #print(l0)
                        if ispin == 1:
                            for e2 in l0:
                                e3 = float(e2)*unit_hartree_ev
                                ke[0][-1].append([e3,None])
                        if ispin == 2:                            
                            for j,e2 in enumerate(l0):
                                if j<len(l0)/2:
                                    e3 = float(e2)*unit_hartree_ev
                                    ke[0][-1].append([e3,None])
                                else:
                                    e3 = float(e2)*unit_hartree_ev
                                    ke[1][-1].append([e3,None])

                    for e1 in e0.findall("occupations"):
                        l0 = e1.text.split()
                        #print(l0)
                        if ispin == 1:
                            for j,e2 in enumerate(l0):
                                ke[0][-1][j][1] = float(e2)
                        if ispin == 2:
                            for j,e2 in enumerate(l0):
                                if j<len(l0)/2:
                                    ke[0][-1][j][1] = float(e2)
                                else:
                                    #j0 = j-len(l0)/2
                                    j0 = int(j-len(l0)/2)
                                    ke[1][-1][j0][1] = float(e2)                                    

            for i in range(len(ke)):
                for j in range(len(ke[i])):
                    for k in range(len(ke[i][j])):
                        #print(i,j,k,ke[i][j][k])
                        pass

        dos = None
        pdos = None
        if name == "tdos":
            dos = []
            f0 = task.input["fildos"]
            f0 = open(f0)
            dos.append([])
            if ispin == 2:
                dos.append([])
            for l in f0:
                if l.find("#")>=0:
                    continue
                else:
                    l0 = l.split()
                    if ispin == 1:
                        dos[0].append([l0[0],l0[1],l0[2]])
                    elif ispin == 2:
                        dos[0].append([l0[0],l0[1],l0[3]])
                        dos[1].append([l0[0],l0[2],l0[3]])
                        
        if name == "pdos":
            f0 = task.option["_log_file"]
            f0 = open(f0)
            p0 = []
            a0 = []
            w0 = []
            aw0 = []
            for l in f0:
                if l.find("state")>=0 and l.find("atom")>=0 and l.find("wfc")>=0:
                    p0.append(l)
                    l0 = l.split()
                    a0.append(int(l0[4]))
                    w0.append(int(l0[8]))
                    aw0.append([int(l0[4]),int(l0[8])])
            a0 = list(set(a0))
            w0 = list(set(w0))
            #print(aw0)
            #print(a0)
            #print(w0)
            p1 = []  # atom,wfc,[l,m]
            a1 = []
            for i in range(len(a0)):
                p1.append([])
                a1.append(None)
                for j in range(len(w0)):
                    p1[-1].append([])
                    p1[-1][-1] = []
                    for l in p0:
                        l0 = l.split()
                        if i==int(l0[4])-1 and j==int(l0[8])-1:
                            l1 = l0[9].split("=")
                            l2 = l0[11].rstrip(")")
                            p1[i][j].append([int(l1[1]),int(l2)])
                            a1[i] = l0[5].lstrip("(")
            #print(p1)
            #print(a1)

            pdos = []
            f0 = task.input["filpdos"]
            for i in range(len(p1)):
                pdos.append([])
                pdos[i].append([])
                if ispin==2:
                    pdos[i].append([])                    
                for j in range(len(p1[i])):
                    f1 = f0+".pdos_atm#"+str(i+1)+"("+a1[i]+")"
                    if len(p1[i][j])==0:
                        continue
                    if p1[i][j][0][0]==0:
                        f1 = f1+"_wfc#"+str(j+1)+"(s)"
                    elif p1[i][j][0][0]==1:
                        f1 = f1+"_wfc#"+str(j+1)+"(p)"
                    elif p1[i][j][0][0]==2:
                        f1 = f1+"_wfc#"+str(j+1)+"(d)"
                    if os.path.exists(f1):
                        print(f1)
                        f1 = open(f1)
                        for k,l in enumerate(f1):
                            if l.find("#")>=0 or l.strip()=="":
                                continue
                            l0 = l.split()

                            if len(pdos[i][0])<k:
                                pdos[i][0].append([])
                                if ispin==2:
                                    pdos[i][1].append([])

                            if len(pdos[i][0][k-1])==0:
                                pdos[i][0][k-1].append(float(l0[0]))
                                if ispin==2:
                                    pdos[i][1][k-1].append(float(l0[0]))

                            for p0 in range(len(p1[i][j])):
                                if ispin==1:
                                    pdos[i][0][k-1].append(float(l0[2+p0]))
                                if ispin==2:
                                    pdos[i][0][k-1].append(float(l0[3+p0*2]))
                                    pdos[i][1][k-1].append(float(l0[4+p0*2]))
                        f1.close()

            pdoscol0 = []
            pdoscol = []
            for i in range(len(p1)):
                pdoscol0.append([])
                pdoscol0[-1].append("energy")
                for j in range(len(p1[i])):
                    for k in range(len(p1[i][j])):
                        s0 = ""
                        if len(p1[i][j])==0:
                            continue
                        if p1[i][j][k][0]==0:
                            s0 = "s_"
                        elif p1[i][j][k][0]==1:
                            s0 = "p_"
                        elif p1[i][j][k][0]==2:
                            s0 = "d_"
                        s = s0 + "m="+ str(p1[i][j][k][1])
                        pdoscol0[-1].append(s)
            # s pz px py dz2 dzx dzy dx2-y2 dxy
            for i in range(len(p1)):
                pdoscol.append([])
                pdoscol[-1].append("energy")
                for j in range(len(p1[i])):
                    if len(p1[i][j])==0:
                        continue
                    if p1[i][j][0][0]==0:
                        pdoscol[-1].extend(["s"])
                    elif p1[i][j][0][0]==1:
                        pdoscol[-1].extend(["pz","px","py"])
                    elif p1[i][j][0][0]==2:
                        pdoscol[-1].extend(["dz2","dzx","dzy","dx2-y2","dxy"])
            #print(pdoscol0)
            #print(pdoscol)

        if name == "band_plot":
            f0 = task.input["filband"]
            kp = []
            ke = []
            ke.append([])
            f0 = open(f0)
            for l in f0:
                if l.find("&plot")>=0:
                    l0 = l.split(",")
                    l1 = l0[0].split()
                    nb = int(l1[2])
                    l1 = l0[1].split()
                    nk = int(l1[1])
                else:
                    l0 = l.split()
                    kp.append([float(l0[0]),float(l0[1]),float(l0[2])])
                    ke[0].append([])
                    ke0 = []
                    #for i in range(int((nb+10-1)/10)):
                    #for i in range(math.ceil(nb/10)):                        
                    nb0 = range(nb)
                    nb1 = [nb0[i:i+10] for i in range(0,nb,10)]
                    for i in range(len(nb1)):
                        #c = f0.next().rstrip()
                        c = next(f0).rstrip()
                        l0 = c.split()
                        if len(c) == len(nb1[i])*8 and len(l0) != len(nb1[i]):
                            # espresso 5.4.0 band %8.3f
                            l0 = [c[i:i+8] for i in range(0,len(c),8)]
                        for j in l0:
                            ke0.append(j)
                    for e in ke0:
                        ke[0][-1].append([float(e),0.0])

            for i in range(len(kp)):
                #print(kp[i])
                pass
            for i in range(len(ke)):
                for j in range(len(ke[i])):
                    for k in range(len(ke[i][j])):
                        #print(i,j,k,ke[i][j][k])
                        pass

        # update parameter
        doc2 = ElementTree.parse(f)
        root2 = doc2.getroot()
        #for e0 in root2.findall("calculation[@name="+name+"]/properties")):
        for e0 in root2.findall("calculation"):
            if e0.attrib.get("name") == name:
                # structure
                for e1 in e0.findall("structure"):
                    if e1.attrib.get("name") == "initial":
                        if cell0 is not None:
                            e2 = ElementTree.SubElement(e1,"lattice")
                            e2.set("type","primitive")
                            e3 = ElementTree.SubElement(e2,"a1")
                            e3.text = str(cell0[0][0])+" "+str(cell0[0][1])+" "+str(cell0[0][2])
                            e3 = ElementTree.SubElement(e2,"a2")
                            e3.text = str(cell0[1][0])+" "+str(cell0[1][1])+" "+str(cell0[1][2])
                            e3 = ElementTree.SubElement(e2,"a3")
                            e3.text = str(cell0[2][0])+" "+str(cell0[2][1])+" "+str(cell0[2][2])
                            #e2 = ElementTree.SubElement(e1,"atom")
                        if atom0 is not None:
                            e2 = ElementTree.SubElement(e1,"atom")
                            for i in range(len(atom0)):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name",self.atom[i][0])
                                e3.text = str(atom0[i][1])+" "+str(atom0[i][2])+" "+str(atom0[i][3])
                        
                    if e1.attrib.get("name") == "final":
                        if cell1 is not None:
                            e2 = ElementTree.SubElement(e1,"lattice")
                            e2.set("type","primitive")
                            e3 = ElementTree.SubElement(e2,"a1")
                            e3.text = str(cell1[0][0])+" "+str(cell1[0][1])+" "+str(cell1[0][2])
                            e3 = ElementTree.SubElement(e2,"a2")
                            e3.text = str(cell1[1][0])+" "+str(cell1[1][1])+" "+str(cell1[1][2])
                            e3 = ElementTree.SubElement(e2,"a3")
                            e3.text = str(cell1[2][0])+" "+str(cell1[2][1])+" "+str(cell1[2][2])
                            #e2 = ElementTree.SubElement(e1,"atom")
                        if atom1 is not None:
                            e2 = ElementTree.SubElement(e1,"atom")
                            for i in range(len(atom1)):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name",atom1[i][0])
                                e3.text = str(atom1[i][1])+" "+str(atom1[i][2])+" "+str(atom1[i][3])
                
                # kpoints
                for e1 in e0.findall("kpoints"):
                    if e1.attrib.get("type") == "kmesh":
                        if nkp is not None:
                            e1.text = str(nkp[0])+" "+str(nkp[1])+" "+str(nkp[2])
                    elif e1.attrib.get("type") == "kpath":
                        e1.clear()
                        e1.set("type","kpath")
                        for k in self.kpath:
                            e2 = ElementTree.SubElement(e1,"i")
                            if len(k)>4:
                                e2.set("name",str(k[4]))
                            #e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                            #s = "%10.5f %10.5f %10.5f" % (k[0],k[1],k[2])
                            s = "%15.10f %15.10f %15.10f" % (k[0],k[1],k[2])
                            e2.text = s.strip()

                # parameter
                for e1 in e0.findall("parameters"):
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","ispin")
                    e2.text = str(ispin)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nbands")
                    e2.text = str(nbands)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nelect")
                    e2.text = str(nelect)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nkpoints")
                    e2.text = str(nkpoints)
                    if kp is not None:
                        e2 = ElementTree.SubElement(e1,"kpoints")
                        for i in range(len(kp)):
                            e3 = ElementTree.SubElement(e2,"i")
                            e3.text = str(kp[i][0])+" "+str(kp[i][1])+" "+str(kp[i][2])

                # property
                for e1 in e0.findall("properties"):
                    if etotal is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal")
                        e2.set("unit","eV")
                        e2.text = str(etotal)
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal_per_atom")
                        e2.set("unit","eV/atom")
                        e2.text = str(float(etotal)/self.natom)
                    if efermi is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","efermi")
                        e2.set("unit","eV")
                        e2.text = efermi
                    if mag is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","magnetization")
                        e2.set("unit","uB")
                        e2.text = mag
                    if dos is not None:
                        e2 = ElementTree.SubElement(e1,"dos")
                        for i in range(len(dos)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(dos[i])):
                                e4 = ElementTree.SubElement(e3,"i")
                                e4.text = dos[i][j][0]+" "+dos[i][j][1]+" "+dos[i][j][2]
                    if pdos is not None:
                        e2 = ElementTree.SubElement(e1,"pdos")
                        #e3 = ElementTree.SubElement(e2,"i")
                        #e3.set("name","value")
                        #s = ""
                        #for i in range(len(pdoscol)):
                        #    s += str(pdoscol[i]) + " "
                        #e3.text = s.strip()
                        for i in range(len(pdos)):
                            e3 = ElementTree.SubElement(e2,"atom")
                            e3.set("name",self.atom[i][0])
                            e3.set("atom",str(i+1))

                            e4 = ElementTree.SubElement(e3,"i")
                            e4.set("name","value")
                            s = ""
                            for j in range(len(pdoscol[i])):
                                s += str(pdoscol[i][j]) + " "
                            e4.text = s.strip()

                            for j in range(len(pdos[i])):
                                e4 = ElementTree.SubElement(e3,"spin")
                                e4.set("spin",str(j+1))
                                for k in range(len(pdos[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    s = ""
                                    for l in range(len(pdos[i][j][k])):
                                        s += str(pdos[i][j][k][l]) + " "
                                    e5.text = s.strip()
                    if ke is not None:
                        e2 = ElementTree.SubElement(e1,"band")
                        for i in range(len(ke)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(ke[i])):
                                e4 = ElementTree.SubElement(e3,"kpoint")
                                kp = j+1
                                e4.set("kpoint",str(kp))
                                for k in range(len(ke[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    #s = "%10.5f %10.5f" % (ke[i][j][k][0],ke[i][j][k][1])
                                    #e5.text = s
                                    e5.text = str(ke[i][j][k][0])+" "+str(ke[i][j][k][1])

                # step
                for e1 in e0.findall("step"):
                    if step is not None:
                        for i in range(len(step)):
                            e2 = ElementTree.SubElement(e1,"sc")
                            e2.set("step",str(i+1))
                            for j in range(len(step[i])):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name","etotal")
                                e3.set("unit","eV")
                                e3.text = str(step[i][j])

        tree = ElementTree.tostring(root2,'utf-8')        
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    def calstat(self):
        stat = []

        f0 = "calculations.xml"

        if not os.path.exists(f0):
            return stat

        doc = ElementTree.parse(f0)
        root = doc.getroot()

        matid = None
        chemform = None
        crysys = None
        spg = None
        spgno = None
        cif = None
        calc = []
        for e in root.findall("material"):
            for e1 in e.getchildren():
                if e1.tag == "material_id":
                    matid = e1.text
                if e1.tag == "chemical_formula":
                    chemform = e1.text
                if e1.tag == "crystal_system":
                    crysys = e1.text
                if e1.tag == "spacegroup":
                    spg = e1.text
                if e1.tag == "spacegroup_no":
                    spgno = e1.text
                if e1.tag == "cif":
                    cif = e1.text
        for e in root.findall("calculation"):
            calc.append(e.attrib.get("name"))

        stat.append([matid,chemform,crysys,spg,spgno,cif])
        stat.append(calc)
        stat.append([])
        #stat.append([0]*len(calc))

        for name in calc:
            for e in root.findall("calculation"):
                if e.attrib.get("name")==name:
                    s=0
                    for e1 in e.findall("inputs/i"):
                        if e1.attrib.get("name")=="_calc_dir":
                            d = e1.text
                            f1 = d+"/log"
                            if os.path.exists(f1):
                                f = open(f1)
                                for l in f:
                                    if l.find("JOB DONE.")>=0:
                                        s = 0
                                        break
                                    else:
                                        s = 1
                            else:
                                s = -1

            stat[2].append(s)
            
        return stat
