import os
import collections
import ConfigParser
import argparse
import uuid
import string
from xml.etree import ElementTree
from xml.dom import minidom
import zipfile
import numpy
import spglib

import structure.crystal
import calculation.calc
import toast_config
import calprop

class Task():
    name=None    
    calc_option = {}
    property=None
    dir=None
    pp=None
    def __init__(self,name=None):
        self.name=name
        self.property=[]

class Data():
    scenario=None
    matrecordtmpl=None

    material_id=None 
    spacegroup=None
    spacegroup_no=None
    crystal_system=None
    cif=None
    tasks=[]

    calculation_code_name=None
    calculation_code_version=None
    auxiliary_code_name=None
    auxiliary_code_version=None    

    uuid = str(uuid.uuid4())

    files = collections.OrderedDict()

    atomwork=False

    def datasheet(self,f):
        print("[datasheet]")

        # cif
        cif0 = self.matid + ".cif"
        cif1 = self.matid + "-1-2.cif"            
        if os.path.isfile(cif0):
            cif = cif0
        elif os.path.isfile(cif1):
            cif = cif1
            self.atomwork=True
        else:
            cif = None
        self.cif = cif
        print("cif:",self.cif)

        # scenario
        cal = calculation.calc.Calc(self.program,self.matid,scenario=self.scenario)
        toast_cfg_file = os.path.join(toast_config.cfgdir,"toast.cfg")
        if not os.path.isfile(toast_cfg_file):
            sys.stderr.write("Error can not find {}".format(toast_cfg_file))
            exit(1)
        toast_cfg = ConfigParser.SafeConfigParser()
        toast_cfg.read(toast_cfg_file)
        program_options = {}
        if toast_cfg.has_section(self.program):
            program_options.update(toast_cfg.items(self.program))
        print("scenario:",self.scenario)

        # tasks
        tasks = []
        if self.scenario != None:
            if not os.path.exists(self.scenario):
                s = os.path.join(toast_config.cfgdir,self.scenario)
                if os.path.exists(s):
                    self.scenario = s
                elif self.scenario == "default":
                    self.scenario = os.path.join(toast_config.cfgdir,program_options["scenario"])
                else:
                    print("scenario not found")
                    exit(1)                    
            self.scenario = os.path.abspath(self.scenario)
            print("[scenario]\n",self.scenario)
        
            cal.read_config(self.scenario)

            for s in cal.task_dic.keys():
                t = Task()
                t.name = s
                t.dir = cal.task_dic[s].option["_calc_dir"]
                #print(t.name,t.dir)

                try:
                    ElementTree.parse(t.dir+"/vasprun.xml")
                    tasks.append(t)
                except:
                    print(t.dir+"/vasprun.xml","invalid...")
                
        else:
            dirs = []
            datadirs(".",dirs)
            #print(dirs)
            for d in dirs:
                t = Task()
                if str(d)==".":
                    t.name = os.path.basename(os.path.abspath("."))
                else:
                    t.name = d
                t.dir = d
                #print(t.name,t.dir)

                try:
                    ElementTree.parse(t.dir+"/vasprun.xml")
                    tasks.append(t)
                except:
                    print(t.dir+"/vasprun.xml","invalid...")

                print("["+t.name+"]",)
            print()

        # material
        # structure
        if cif is not None:
            cry = structure.crystal.Crystal()
            cry.read_cif(cif)
            cry.check_cif()
            cry.construct_structure()

            chemform = cry.chemical_formula_sum
            crysys = cry.space_group_crystal_system
            spgname = cry.symmetry_space_group_name
            spgnum = cry.symmetry_Int_Tables_number
            cell_a = cry.cell.length_a
            cell_b = cry.cell.length_b
            cell_c = cry.cell.length_c
            cell_alpha = cry.cell.angle_alpha
            cell_beta = cry.cell.angle_beta
            cell_gamma = cry.cell.angle_gamma
            primcell = cry.unit_prim
            convcell = cry.unit_conv

            ele = []
            for e in range(len(cry.velement_name)):
                ele.append(cry.velement_name[e])
            natom_ele = []
            for e in range(len(cry.velement_size)):
                natom_ele.append(cry.velement_size[e])

            atom_conv = []  # ele,pos,site,op
            for a in cry.vatom_conv:
                atom_conv.append([a.element,a.position[0],a.position[1],a.position[2],a.site_index,a.operation_index])
            atom_prim = []  # ele,pos,site,op
            for a in cry.vatom_prim:
                atom_prim.append([a.element,a.position[0],a.position[1],a.position[2],a.site_index,a.operation_index])

            atom = []
            for a in cry.vatom_prim:
                atom.append([a.element,a.position[0],a.position[1],a.position[2]])

        else:
            cell,ele,natom_ele,atom,num = structure_vasp(dirs[0]+"/POSCAR")

            s = [e+str(na) for e,na in zip(ele,natom_ele)]
            #chemform = string.join(s," ")
            chemform = " ".join(s)

            primcell = numpy.array(cell,float)
            primatom = []
            for a in atom:
                primatom.append([float(a[1]),float(a[2]),float(a[3])])
            primnum = num
            pcell = (primcell,primatom,primnum)
            print(pcell[0])
            print(pcell[1])
            scell = spglib.standardize_cell(pcell)
            print(scell[0])
            print(scell[1])
            convcell = scell[0]

            atom_prim = []
            for a,n in zip(pcell[1],pcell[2]):
                atom_prim.append([ele[n],a[0],a[1],a[2]])

            atom_conv = []
            for a,n in zip(scell[1],scell[2]):
                atom_conv.append([ele[n],a[0],a[1],a[2]])

            dataset = spglib.get_symmetry_dataset(scell,1.0e-5)
            symmetry = spglib.get_symmetry(scell)
            hall_number = dataset["hall_number"]

            spgname = dataset["international"]
            spgnum = dataset["number"]
            crysys = ""

        nele = len(ele)
        natom = len(atom)
        #print(nele, natom)

        # calculations
        root = ElementTree.Element("calculations")
        root.set("version","1.1")

        # program
        e0 = ElementTree.SubElement(root,"program")
        e1 = ElementTree.SubElement(e0,"name")
        e1 = ElementTree.SubElement(e0,"version")

        # material
        e0 = ElementTree.SubElement(root,"material")
        e1 = ElementTree.SubElement(e0,"material_id")
        e1.text = self.matid
        e1 = ElementTree.SubElement(e0,"chemical_formula")
        e1.text = chemform
        e1 = ElementTree.SubElement(e0,"crystal_system")
        e1.text = crysys
        e1 = ElementTree.SubElement(e0,"spacegroup")
        e1.text = spgname
        e1 = ElementTree.SubElement(e0,"spacegroup_no")
        e1.text = str(spgnum)
        e1 = ElementTree.SubElement(e0,"cif")
        e1.text = cif

        # structure
        e0 = ElementTree.SubElement(root,"structure")        
        e0.set("name","original")

        if cif is not None:
            e1 = ElementTree.SubElement(e0,"lattice")
            e1.set("type","conventional")
            e2 = ElementTree.SubElement(e1,"a")
            e2.text = str(cell_a)
            e2 = ElementTree.SubElement(e1,"b")
            e2.text = str(cell_b)
            e2 = ElementTree.SubElement(e1,"c")
            e2.text = str(cell_c)
            e2 = ElementTree.SubElement(e1,"alpha")
            e2.text = str(cell_alpha)
            e2 = ElementTree.SubElement(e1,"beta")
            e2.text = str(cell_beta)
            e2 = ElementTree.SubElement(e1,"gamma")
            e2.text = str(cell_gamma)

        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","conventional")
        e2 = ElementTree.SubElement(e1,"a1")
        e2.text = str(convcell[0][0])+" "+str(convcell[0][1])+" "+str(convcell[0][2])
        e2 = ElementTree.SubElement(e1,"a2")
        e2.text = str(convcell[1][0])+" "+str(convcell[1][1])+" "+str(convcell[1][2])
        e2 = ElementTree.SubElement(e1,"a3")
        e2.text = str(convcell[2][0])+" "+str(convcell[2][1])+" "+str(convcell[2][2])
        e1 = ElementTree.SubElement(e0,"atom")
        e1.set("type","conventional")
        for a in atom_conv:
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",a[0])
            if len(a) > 4:
                e2.set("operation",str(a[4]))
                e2.set("site",str(a[5]))
            e2.text = str(a[1])+" "+str(a[2])+" "+str(a[3])

        e1 = ElementTree.SubElement(e0,"lattice")
        e1.set("type","primitive")
        e2 = ElementTree.SubElement(e1,"a1")
        e2.text = str(primcell[0][0])+" "+str(primcell[0][1])+" "+str(primcell[0][2])
        e2 = ElementTree.SubElement(e1,"a2")
        e2.text = str(primcell[1][0])+" "+str(primcell[1][1])+" "+str(primcell[1][2])
        e2 = ElementTree.SubElement(e1,"a3")
        e2.text = str(primcell[2][0])+" "+str(primcell[2][1])+" "+str(primcell[2][2])

        e1 = ElementTree.SubElement(e0,"atom")
        e1.set("type","primitive")
        for a in atom_prim:
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name",a[0])
            if len(a) > 4:
                e2.set("operation",str(a[4]))
                e2.set("site",str(a[5]))
            e2.text = str(a[1])+" "+str(a[2])+" "+str(a[3])

        e0 = ElementTree.SubElement(root,"structure")        
        e0.set("name","relaxed")

        # calculation
        for t in tasks:
            if t.name == "fermisurf":
                e0 = root.find(".//calcution[@name='dos']/properties/i[@name='bandgap_type']")
                if e0==None or e0.text!="metal":
                    continue
            if t.name == "emass":
                e0 = root.find(".//calcution[@name='dos']/properties/i[@name='bandgap_type']")
                if e0==None or e0.text=="metal":
                    continue

            print("["+t.name+"]")
            e0 = ElementTree.SubElement(root,"calculation")
            e0.set("name",t.name)
            e0.set("dir",t.dir)
            e1 = ElementTree.SubElement(e0,"inputs")

            inputs = inputs_vasp(t.dir+"/INCAR")
            for i in inputs:
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name",i[0])
                e2.text = i[1]

            e1 = ElementTree.SubElement(e0,"structure")
            e1.set("name","initial")

            cell,ele,natom_ele,atom,num = structure_vasp(t.dir+"/POSCAR")

            #e2 = ElementTree.SubElement(e1,"lattice")
            #e2.set("type","primitve")
            #e3 = ElementTree.SubElement(e2,"a1")
            #e3.text = str(cell[0][0])+" "+str(cell[0][1])+" "+str(cell[0][2])
            #e3 = ElementTree.SubElement(e2,"a2")
            #e3.text = str(cell[1][0])+" "+str(cell[1][1])+" "+str(cell[1][2])
            #e3 = ElementTree.SubElement(e2,"a3")
            #e3.text = str(cell[2][0])+" "+str(cell[2][1])+" "+str(cell[2][2])

            #e2 = ElementTree.SubElement(e1,"atom")
            #for a in atom:
            #    e3 = ElementTree.SubElement(e2,"i")
            #    e3.set("name",a[0])
            #    e3.text = str(a[1])+" "+str(a[2])+" "+str(a[3])

            #e1 = ElementTree.SubElement(e0,"structure")
            #e1.set("name","final")

            cell,ele,natom_ele,atom,num = structure_vasp(t.dir+"/CONTCAR")

            primcell = numpy.array(cell,float)
            primatom = []
            for a in atom:
                primatom.append([float(a[1]),float(a[2]),float(a[3])])
            primnum = num
            pcell = (primcell,primatom,primnum)
            scell = spglib.standardize_cell(pcell)
            dataset = spglib.get_symmetry_dataset(scell,1.0e-5)
            symmetry = spglib.get_symmetry(scell)
            hall_number = dataset["hall_number"]
            sgtype = spglib.get_spacegroup_type(hall_number)
            #print("spg_dataset")
            #print("  number",dataset["number"])
            #print("  international",dataset["international"])
            #print("  hall_symbol",dataset["hall"])
            #print("  hall_number",dataset["hall_number"])
            #print("  choice",dataset["choice"])
            #print("  pointgroup",dataset["pointgroup"])
                        
            spgname = dataset["international"]
            spgnum = dataset["number"]
            print("structure:",spgnum,spgname)

            #e2 = ElementTree.SubElement(e1,"lattice")
            #e2.set("type","primitve")
            #e3 = ElementTree.SubElement(e2,"a1")
            #e3.text = str(cell[0][0])+" "+str(cell[0][1])+" "+str(cell[0][2])
            #e3 = ElementTree.SubElement(e2,"a2")
            #e3.text = str(cell[1][0])+" "+str(cell[1][1])+" "+str(cell[1][2])
            #e3 = ElementTree.SubElement(e2,"a3")
            #e3.text = str(cell[2][0])+" "+str(cell[2][1])+" "+str(cell[2][2])

            #e2 = ElementTree.SubElement(e1,"atom")
            #for a in atom:
            #    e3 = ElementTree.SubElement(e2,"i")
            #    e3.set("name",a[0])
            #    e3.text = str(a[1])+" "+str(a[2])+" "+str(a[3])

            e1 = ElementTree.SubElement(e0,"pseudopotentials")            

            if self.program == "vasp":
                pp = pseudopotentials_vasp(t.dir+"/POTCAR")
            else:
                pp = None

            for e,p in zip(ele,pp):
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name",e)
                p0 = p.split()
                e2.text = p0[1]
                e3 = ElementTree.SubElement(e2,"i")
                e3.set("name","title")
                e3.text = p

            # kpoints
            e1 = ElementTree.SubElement(e0,"kpoints")

            ktype,nkp,kp = kpoints_vasp(t.dir+"/KPOINTS")

            e1.set("type",ktype)
            if nkp==0:
                e1.text = str(kp[0][0])+" "+str(kp[0][1])+" "+str(kp[0][2])
            else:
                for k in kp:
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                    if len(k)==6:
                        e2.set("name",k[5])

            e1 = ElementTree.SubElement(e0,"parameters")
            e1 = ElementTree.SubElement(e0,"properties")
            e1 = ElementTree.SubElement(e0,"step")

            para = caldata_vasp(t.dir+"/vasprun.xml",t.name,root)
            
        # post
        calprop.cellparameter(root)
        calprop.bandgap(root)        

        #ElementTree.dump(root)
        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()
        
    def metadata(self,f):
        root = ElementTree.Element("rdf:RDF")

        # rdf
        root.set("xmlns:rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#")
        root.set("xmlns:dc","http://purl.org/dc/elements/1.1/")

        e0 = ElementTree.SubElement(root,"rdf:Description")

        e1 = ElementTree.SubElement(e0,"dc:title")
        e1 = ElementTree.SubElement(e0,"dc:subject")
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1 = ElementTree.SubElement(e0,"dc:type")
        e1 = ElementTree.SubElement(e0,"dc:relation")
        e1 = ElementTree.SubElement(e0,"dc:source")

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    def materialrecord(self,f):
        print("[materialrecord]")
        print("materialRecord.xml")

        print("cif:",self.cif)
        print("atomwork:",self.atomwork)
        print("regid:",self.regid)
        print("reguid:",self.reguid)

        root = ElementTree.Element("materialIdentification")

        # attribute
        root.set("xmlns","http://material-metadata.nims.go.jp/schema/record/input")
        root.set("xmlns:dc","http://purl.org/dc/elements/1.1/")
        root.set("version","1.0")

        e0 = ElementTree.SubElement(root,"metadata")

        #e1 = ElementTree.SubElement(e0,"dc:title")
        #e1 = ElementTree.SubElement(e0,"dc:subject")
        #e1 = ElementTree.SubElement(e0,"dc:description")
        #e1 = ElementTree.SubElement(e0,"dc:type")
        #e1.text = "calculation"
        #e1 = ElementTree.SubElement(e0,"dc:relation")
        #e1 = ElementTree.SubElement(e0,"dc:source")
        #e1 = ElementTree.SubElement(e0,"dc:publisher")
        #e1.text = "National Institute for Materials Science (NIMS)"
        #e1 = ElementTree.SubElement(e0,"dc:identifier")
        #e1 = ElementTree.SubElement(e0,"dc:creator")
        #e1 = ElementTree.SubElement(e0,"dc:contributor")
        #e1 = ElementTree.SubElement(e0,"dc:coverage")
        #e1 = ElementTree.SubElement(e0,"dc:date") # system
        #e1 = ElementTree.SubElement(e0,"dc:rights")
        #e1.text = "Copyright (C) National Institute for Materials Science (NIMS)"
        #e1 = ElementTree.SubElement(e0,"dc:format")
        #e1.text = "xml"
        #e1 = ElementTree.SubElement(e0,"dc:language")
        #e1.text = "en"

        e1 = ElementTree.SubElement(e0,"dc:title")
        e1.text = self.materialrecord_tmpl("dc:title")
        e1 = ElementTree.SubElement(e0,"dc:creator")
        e1.text = self.materialrecord_tmpl("dc:creator")
        e1 = ElementTree.SubElement(e0,"dc:subject")
        e1.text = self.materialrecord_tmpl("dc:subject")
        e1 = ElementTree.SubElement(e0,"dc:description")
        e1.text = self.materialrecord_tmpl("dc:description")
        e1 = ElementTree.SubElement(e0,"dc:publisher")
        e1.text = self.materialrecord_tmpl("dc:publisher")
        e1 = ElementTree.SubElement(e0,"dc:contributor")
        e1.text = self.materialrecord_tmpl("dc:contributor")
        e1 = ElementTree.SubElement(e0,"dc:date") # system
        e1 = ElementTree.SubElement(e0,"dc:type")
        e1.text = "calculation"
        e1 = ElementTree.SubElement(e0,"dc:format")
        e1.text = "xml"
        e1 = ElementTree.SubElement(e0,"dc:identifier")
        e1.text = self.materialrecord_tmpl("dc:identifer")
        e1 = ElementTree.SubElement(e0,"dc:source")
        e1.text = self.materialrecord_tmpl("dc:source")
        e1 = ElementTree.SubElement(e0,"dc:language")
        e1.text = "en"
        e1 = ElementTree.SubElement(e0,"dc:relation")
        e1.text = self.materialrecord_tmpl("dc:relation")
        e1 = ElementTree.SubElement(e0,"dc:coverage")
        e1.text = self.materialrecord_tmpl("dc:coverage")
        e1 = ElementTree.SubElement(e0,"dc:rights")
        e1.text = self.materialrecord_tmpl("dc:rights")

        # materials
        e0 = ElementTree.SubElement(root,"materials")
        e1 = ElementTree.SubElement(e0,"material")
        if self.atomwork:
            e1.set("registrationId","atomwork-adv:"+self.material_id)
            #e1.set("idref","atomwork-adv:"+self.material_id)
        else:
            e1.set("registrationId","")
            if self.regid is not None:
                e1.set("registrationId",self.regid)
            e1.set("registrationUserId","")
            if self.reguid is not None:
                e1.set("registrationUserId",self.reguid)
        e2 = ElementTree.SubElement(e1,"materialName")
        e2 = ElementTree.SubElement(e1,"materialType")
        e2.text = "atom"
        e2 = ElementTree.SubElement(e1,"relations")
        e3 = ElementTree.SubElement(e2,"relation")

        e2 = ElementTree.SubElement(e1,"chemicalSystem")
        e2.text = self.chemical_formula
        #e2 = ElementTree.SubElement(e1,"sortedChemicalSystem") # system
        #e2.text = self.chemical_formula

        e2 = ElementTree.SubElement(e1,"chemicalComposition")
        e2.set("ratio","atomic")
        e2.text = self.chemical_comp

        e2 = ElementTree.SubElement(e1,"standardSpecification")
        e3 = ElementTree.SubElement(e2,"standardType")
        e3 = ElementTree.SubElement(e2,"standardNumber")

        e2 = ElementTree.SubElement(e1,"substance")
        if self.atomwork:
            e2.set("idref",self.substance_id)

        e3 = ElementTree.SubElement(e2,"chemicalSystem")
        e3.text = self.chemical_formula
        #e3 = ElementTree.SubElement(e2,"sortedChemicalSystem") # system
        #e3.text = self.chemical_formula
        e3 = ElementTree.SubElement(e2,"chemicalFormula")
        e3.text = self.chemical_formula
        e3 = ElementTree.SubElement(e2,"state")
        e3.text = "crystalline"
        e3 = ElementTree.SubElement(e2,"structureType")

        e3 = ElementTree.SubElement(e2,"pearsonSymbol")
        e3.text = self.pearson_symbol

        e3 = ElementTree.SubElement(e2,"spaceGroup")
        e4 = ElementTree.SubElement(e3,"number")
        e4.text = self.spacegroup_no
        e4 = ElementTree.SubElement(e3,"name")

        # add metadata
        e0 = ElementTree.SubElement(root,"additionalMetadata")

        # calculation
        e1 = ElementTree.SubElement(e0,"calculationMetadata")
        e1.set("xmlns","http://material-metadata.nims.go.jp/schema/calculation")

        e2 = ElementTree.SubElement(e1,"calculationType")
        e2.text = "ab-initio_electronic_structure"

        e2 = ElementTree.SubElement(e1,"programs")

        e3 = ElementTree.SubElement(e2,"program")

        e4 = ElementTree.SubElement(e3,"code")
        e5 = ElementTree.SubElement(e4,"name")
        e5.text = self.calculation_code_name
        e5 = ElementTree.SubElement(e4,"version")
        e5.text = self.calculation_code_version
        e5 = ElementTree.SubElement(e4,"info")

        e4 = ElementTree.SubElement(e3,"auxiliary")
        e5 = ElementTree.SubElement(e4,"name")
        e5.text = self.auxiliary_code_name
        e5 = ElementTree.SubElement(e4,"version")
        e5.text = self.auxiliary_code_version
        e5 = ElementTree.SubElement(e4,"info")

        e2 = ElementTree.SubElement(e1,"localID")
        e2.text = self.uuid
        e2 = ElementTree.SubElement(e1,"info")

        e2 = ElementTree.SubElement(e1,"tasks")
        print("tasks:")
        for t in self.tasks:
            print(" ["+t.name+"]")
            e3 = ElementTree.SubElement(e2,"task")
            e3.set("name",t.name)
            e3.set("subdir",t.dir)

            # structure
            e4 = ElementTree.SubElement(e3,"structure")
            e5 = ElementTree.SubElement(e4,"chemicalFormula")
            e5.text = self.chemical_formula

            #e5 = ElementTree.SubElement(e4,"crystalSystem")
            #e5.text = self.crystal_system

            e5 = ElementTree.SubElement(e4,"pearsonSymbol")
            e5.text = self.pearson_symbol            
            e5 = ElementTree.SubElement(e4,"spaceGroupNumber")
            e5.text = self.spacegroup_no

            # classification
            e4 = ElementTree.SubElement(e3,"classification")

            # option
            # pp
            print("  pp:",t.pp)
            if t.pp is not None:
                e5 = ElementTree.SubElement(e4,"option")
                e5.set("name","pp")
                #print(t.name,t.pp)
                #e5.text = string.join(t.pp,", ")
                e5.text = ", ".join(t.pp)

            # calc option
            print("  option:",t.calc_option)
            for k,v in t.calc_option.items():
                e5 = ElementTree.SubElement(e4,"option")
                e5.set("name",k)
                e5.text = str(v)

            e5 = ElementTree.SubElement(e4,"keyword")
            e5 = ElementTree.SubElement(e4,"info")

            # property
            print("  property:",t.property)
            e4 = ElementTree.SubElement(e3,"properties")
            for p in t.property:
                e5 = ElementTree.SubElement(e4,"property")
                e5.text = p

        # manifest
        e0 = ElementTree.SubElement(root,"manifest")

        for f0 in self.files:
            e1 = ElementTree.SubElement(e0,"item")
            e1.set("path",f0)
            e1.set("searchable","false")
            e2 = ElementTree.SubElement(e1,"comment")
            #e2.text = ""

        tree = ElementTree.tostring(root,'utf-8')
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

    # temporary
    def read_calculations(self,f):
        print("[datasheet (load)]")

        doc = ElementTree.parse(f)
        root = doc.getroot()

        # program
        e0 = root.find(".//program/name")
        self.calculation_code_name = e0.text
        e0 = root.find(".//program/version")
        self.calculation_code_version = e0.text

        self.auxiliary_code_name=""
        self.auxiliary_code_version=""

        # material
        for e0 in root.findall(".//material/material_id"):
            self.material_id = e0.text
        for e0 in root.findall(".//material/chemical_formula"):
            self.chemical_formula = e0.text
        for e0 in root.findall(".//material/crystal_system"):
            self.crystal_system = e0.text
        for e0 in root.findall(".//material/spacegroup"):
            self.spacegroup = e0.text
        for e0 in root.findall(".//material/spacegroup_no"):
            self.spacegroup_no = e0.text

        e1 = root.findall(".//structure[@name='original']/atom[@type='conventional']/i")
        self.natom_conv = len(e1)
        e1 = root.findall(".//structure[@name='original']/atom[@type='primitive']/i")
        self.natom_prim = len(e1)
        #print(self.natom_conv,self.natom_prim)
        self.ele = []
        for e0 in e1:
            e = e0.attrib.get("name")
            if e not in self.ele:
                self.ele.append(e)
        self.natom_ele = []
        for e0 in self.ele:
            e1 = root.findall(".//structure[@name='original']/atom[@type='primitive']/i[@name='"+e0+"']")
            self.natom_ele.append(len(e1))
        #print(self.ele,self.natom_ele)

        self.substance_id = atomwork_matsub(self.material_id)
        self.pearson_symbol = structure.crystal.pearsonsymbol(int(self.spacegroup_no))+str(self.natom_conv)
        
        #s = [e+str(na) for e,na in zip(self.ele,self.natom_ele)]
        s = []
        if len(self.ele) == 1:
            s.append(self.ele[0])
        else:
            for e,na in zip(self.ele,self.natom_ele):
                if na==1:
                    s.append(e)
                else:
                    s.append(e+str(na))
        #self.chemical_comp = string.join(s," ")
        self.chemical_comp = " ".join(s)
        #print(self.chemical_comp)

        print( "tasks:")
        for e0 in root.findall(".//calculation"):
            #print(e0.attrib.get("name"),e0.attrib.get("dir"))
            t0 = Task()
            t0.name = e0.attrib.get("name")
            t0.dir = e0.attrib.get("dir")
            print(" ["+t0.name+"]",t0.dir)

            # pp
            pp = []
            for e1 in e0.findall("pseudopotentials/i"):
                p0 = e1.attrib.get("name")
                p1 = None
                for e2 in e1.findall("i[@name='title']"):
                    p1 = e2.text
                if p1 is not None:
                    pp.append(p1)
                else:
                    pp.append(p0)
            print(" pp:",pp)
            t0.pp = pp

            # calc option
            calop = {}
            # scf (ichage)
            e1 = e0.find("inputs/i[@name='ICHARG']")
            calop["scf"] = True
            if e1 is not None and int(e1.text)>=10:
                calop["scf"] = False               

            # opt (ibrion)
            e1 = e0.find("inputs/i[@name='IBRION']")
            if e1 is not None:
                if int(e1.text)==0:
                    calop["md"] = True
                elif int(e1.text)>0:
                    calop["opt"] = True            

            t0.calc_option = calop

            # property
            prop = []
            # energy, charge
            if calop["scf"] is not None and calop["scf"]==True:
                prop.append("total energy")
                prop.append("chage density")
            # dos
            e1 = e0.find("properties/dos")
            if e1 is not None:
                prop.append("density of state")
            # bandgap
            e1 = e0.find("properties/i[@name='bandgap_type']")
            if e1 is not None:
                prop.append("bandgap")
            # band
            e1 = e0.find("properties/band")
            e2 = e0.find("kpoints[@type='kpath']")
            #if e1 is not None:
            if e1 is not None and e2 is not None:
                prop.append("energy bands")
            
            t0.property = prop
                
            print("  option:",t0.calc_option)
            print("  property:",t0.property)

            self.tasks.append(t0)
            
    def materialrecord_tmpl(self,tag):
        s = ""
        if self.matrecordtmpl != None:
            doc0 = ElementTree.parse(self.matrecordtmpl)
            root0 = doc0.getroot()
            namespaces = {"ns":"http://material-metadata.nims.go.jp/schema/record/input","dc":"http://purl.org/dc/elements/1.1/"}

            t0 = root0.find(".//"+tag,namespaces)
            if t0 is not None and t0.text is not None and len(t0.text)>0:
                s = t0.text
        return s

    def datafiles(self,d):
        exceptfiles = ["POTCAR","WAVECAR","materialRecord.xml"]
        exceptfiles.append("metadata.xml")
        #exceptfiles.append("datasheet.xml")
        # temporary
        exceptfiles.append("tmp1.xml")
        exceptfiles.append("tmp2.xml")
        exceptfiles.append("tmp01.xml")
        exceptfiles.append("tmp02.xml")
        exceptfiles.append("tmp03.xml")
        
        files = os.listdir(d)
        for f in files:
            if os.path.isfile(f) and f not in exceptfiles:
                b, e = os.path.splitext(f)
                if e != ".zip":
                    self.files[f] = f

        for t in self.tasks:
            d0 = t.dir
            f0 = os.listdir(d0)
            for f in f0:
                if os.path.isfile(d0+"/"+f) and f not in exceptfiles:
                    self.files[d0+"/"+f] = d0+"/"+f

        #print(self.files)

def datadirs(d,dirs):
    f0 = os.listdir(d)
    if "vasprun.xml" in f0:
        try:
            ElementTree.parse(d+"/vasprun.xml")
            dirs.append(d)
        except:
            print(d+"/vasprun.xml","invalid...")
    for f1 in f0:
        if os.path.isdir(f1):
            datadirs(f1,dirs)

def atomwork_matsub(matid):
    cfgdir = str(os.path.dirname(__file__))+"/../../config"
    matsubfile = os.path.join(cfgdir, "atomwork.matsub.txt")

    matsub = {}
    f = open(matsubfile)
    for l in f:
        l0 = l.split()
        matsub[l0[0]] = l0[1]

    if matid in matsub:
        return matsub[matid]
    else:
        return None

def materialrecord_dataset(mf,zf):
    print("[materialrecord (archive dataset)]")
    print("dataset:",mf,zf)

    doc = ElementTree.parse(mf)
    root = doc.getroot()
    namespaces = {"ns":"http://material-metadata.nims.go.jp/schema/record/input"}

    z = zipfile.ZipFile(zf,"w",compression=zipfile.ZIP_DEFLATED)
    z.write(mf,arcname="materialRecord.xml")

    for e0 in root.findall(".//ns:manifest/ns:item",namespaces):
        f0 = e0.attrib.get("path")
        z.write(f0,arcname=f0)

    z.close()

def inputs_vasp(f):    
    inputs = []
    f0 = open(f)
    for l in f0:
        l = l.strip()
        l0 = l.split("=")
        inputs.append([l0[0].strip(),l0[1].strip()])
    f0.close()
    return inputs

def structure_vasp(f):    
    f0 = open(f)
    cell = []
    atom = []
    atomno = []
    for i,l in enumerate(f0):
        l = l.strip()
        l0 = l.split()
        #print(l0)
        if i==0:
            system = l0[0]
        if i==1:
            scale = float(l0[0])
        if i==2:
            cell.append(map(float,l0))
        if i==3:
            cell.append(map(float,l0))
        if i==4:
            cell.append(map(float,l0))
        if i==5:
            ele = l0
        if i==6:
            natom_ele = map(int,l0)
            natom=sum(natom_ele)
        if i==7:
            for j,e in enumerate(ele):
                na = natom_ele[j]
                for k in range(na):
                    l = f0.next()
                    l0 = l.split()
                    #l0 = map(float,l0)                    
                    atom.append([ele[j],float(l0[0]),float(l0[1]),float(l0[2])])
    f0.close()

    cell = numpy.array(cell,float)
    cell = scale * cell

    num = []
    for i in range(len(ele)):
        for j in range(natom_ele[i]):
            num.append(i)

    #print(cell)
    #print(ele)
    #print(natom_ele)
    #print(atom)
    #print(num)
    return cell,ele,natom_ele,atom,num

def pseudopotentials_vasp(f):
    f0 = open(f)
    pp = []
    for i,l in enumerate(f0):
        l = l.strip()
        #l0 = l.split()
        #print(l0)
        if l.find("TITEL")>=0:
            l0 = l.split("=")
            pp.append(l0[1].strip())
    f0.close()
    #print(pp)
    return pp

def kpoints_vasp(f):
    f0 = open(f)
    kp = []
    for i,l in enumerate(f0):
        l = l.strip()
        l0 = l.split()
        if i==0:
            comment = l0[0]
        if i==1:
            nkp = int(l0[0])
        if i==2:
            coord = l0[0]
            if nkp==0:
                nk=1
            else:
                nk=nkp
            for j in range(nk):
                l = f0.next()
                l0 = l.split()
                kp.append(l0)

    f0.close()

    if nkp==0:
        ktype = "kmesh"
    else:
        ktype = "kpath"
        
    #print(ktype,nkp)
    #print(kp)
    return ktype,nkp,kp

def caldata_vasp(f,name,root2):
    doc = ElementTree.parse(f)
    root = doc.getroot()

    e0 = root.find("./generator/i[@name='program']")
    program = e0.text.strip()
    e0 = root.find("./generator/i[@name='version']")
    version = e0.text.strip()

    #print("program:",program,version)

    etotal = None
    efermi = None
    for e0 in root.findall("./calculation/energy/i[@name='e_fr_energy']"):
        etotal = e0.text.strip()

    for e0 in root.findall("./calculation/dos/i[@name='efermi']"):
        efermi = e0.text.strip()

    #print("etotal:",etotal)
    #print("efermi:",efermi)

    cell0 = []
    cell1 = []
    atom0 = []
    atom1 = []

    for e0 in root.findall("structure[@name='initialpos']/crystal/varray[@name='basis']/v"):
        d = e0.text.split()
        #print(d[0],d[1],d[2])
        cell0.append([d[0],d[1],d[2]])
    for e0 in root.findall("structure[@name='initialpos']/varray[@name='positions']/v"):   
        d = e0.text.split()
        #print(d[0],d[1],d[2])
        atom0.append([d[0],d[1],d[2]])
    #e0 = root.findall("structure[@name='initialpos']/varray[@name='positions']/v")
    #natom = len(e0)
    e0 = root.find("atominfo/atoms")
    natom = int(e0.text)
    for e0 in root.findall("structure[@name='finalpos']/crystal/varray[@name='basis']/v"):
        d = e0.text.split()
        #print(d[0],d[1],d[2])
        cell1.append([d[0],d[1],d[2]])
    for e0 in root.findall("structure[@name='finalpos']/varray[@name='positions']/v"):
        d = e0.text.split()
        #print(d[0],d[1],d[2])
        atom1.append([d[0],d[1],d[2]])
    atom = []
    for e0 in root.findall("atominfo/array[@name='atoms']/set/rc"):
        e1 = e0.findall("c")
        atom.append([e1[0].text])

    #print(cell0)
    #print(atom0)
    #print(natom)
    #print(cell1)
    #print(atom1)
    #print(atom)

    #for i in range(len(cell0)):
    #    print(i,cell0[i][0],cell0[i][1],cell0[i][2])
    #for i in range(len(atom0)):
    #    print(i,atom0[i][0],atom0[i][1],atom0[i][2])
    #for i in range(len(cell1)):
    #    print(i,cell1[i][0],cell1[i][1],cell1[i][2])
    #for i in range(len(atom1)):
    #    print(i,atom1[i][0],atom1[i][1],atom1[i][2])
        
    nkp = None
    ispin = None
    nbands = None
    nelect = None
    nkpoints = None
    for e0 in root.findall("./kpoints/generation/v[@name='divisions']"):
        d = e0.text.split()
        #print(d)
        nkp = [d[0],d[1],d[2]]
    for e0 in root.findall("./parameters/separator/separator/i[@name='ISPIN']"):
        ispin = e0.text.strip()
    for e0 in root.findall("./parameters/separator/i[@name='NBANDS']"):
        nbands = e0.text.strip()
    for e0 in root.findall("./parameters/separator/i[@name='NELECT']"):
        nelect = e0.text.strip()
    e0 = root.findall("./kpoints/varray[@name='kpointlist']/v")
    nkpoints = len(e0)

    #if nkp is not None: 
    #    print("nkp:",nkp[0],nkp[1],nkp[2])
    #print("ispin:",ispin)
    #print("nbands:",nbands)
    #print("nelect:",nelect)
    #print("nkpoints:",nkpoints)

    #for e0 in root.findall("//modeling/kpoints/varray[@name='kpointlist']/v"):
    kp = []
    for e0 in root.findall("./kpoints/varray[@name='kpointlist']/v"):
        l = e0.text.split()
        kp.append([l[0],l[1],l[2]])

    #print(kp)
    for i in range(len(kp)):
        #print(i,kp[i][0],kp[i][1],kp[i][2])
        pass

    # opt, scf
    step = []
    for i,e0 in enumerate(root.findall("calculation")):
        step.append([])
        for e1 in e0.findall("energy/i[@name='e_fr_energy']"):
            pass
        for e1 in e0.findall("scstep/energy/i[@name='e_fr_energy']"):
            step[i].append(e1.text.strip())

    for i in range(len(step)):
        for j in range(len(step[i])):
            #print(i,j,opt[i][j])
            pass

    dos = None
    pdos = None
    ke = None
    if name == "dos":
        ndos = None
        for e0 in root.findall("./parameters/separator[@name='dos']/i[@name='NEDOS']"):
            ndos = e1.text
        #print("ndos:",ndos)

        #tdos
        dos = []
        for e0 in root.findall("./calculation/dos/total/array/set/set"):
            if e0.attrib.get("comment") == "spin 1":                    
                dos.append([])
                for e1 in e0.findall("r"):
                    l = e1.text.split()
                    #print(l[0],l[1],l[2])
                    dos[0].append([l[0],l[1],l[2]])
            if e0.attrib.get("comment") == "spin 2":
                dos.append([])
                for e1 in e0.findall("r"):
                    l = e1.text.split()
                    #print(l[0],l[1],l[2])
                    dos[1].append([l[0],l[1],l[2]])

        for i in range(len(dos)):
            for j in range(len(dos[i])):
                #print(i,j,dos[i][j][0],dos[i][j][1],dos[i][j][2])
                pass

        #pdos
        pdoscol = []
        for e0 in root.findall("./calculation/dos/partial/array/field"):
            pdoscol.append(e0.text.strip())
        pdos = []  # atom,spin
        for i,e0 in enumerate(root.findall("./calculation/dos/partial/array/set/set")):
            if "ion" in e0.attrib.get("comment"):
                pdos.append([])
                for e1 in e0.findall("set"):
                    if e1.attrib.get("comment") == "spin 1":
                        pdos[i].append([])
                        for e2 in e1.findall("r"):
                            #print(e2.text)
                            l0 = e2.text.split()
                            pdos[i][0].append(l0)                                
                    if e1.attrib.get("comment") == "spin 2":
                        pdos[i].append([])
                        for e2 in e1.findall("r"):
                            #print(e3.text)
                            l0 = e2.text.split()
                            pdos[i][1].append(l0)

        for i in range(len(pdos)):
            for j in range(len(pdos[i])):
                for k in range(len(pdos[i][j])):
                    #print(i,j,k,pdos[i][j][k])
                    pass
                        
    if name == "dos" or name == "band" or name == "fermisurf" or name == "emass":
        ke = []  # spin,kp,band,[ene,occ]
        for e0 in root.findall("./calculation/eigenvalues/array/set/set[@comment='spin 1']"):
            ke.append([])
            for i,e1 in enumerate(e0.findall("set")):
                if "kpoint" in e1.attrib.get("comment"):
                    ke[0].append([])
                    for e2 in e1.findall("r"):
                        #print(e2.text)
                        l = e2.text.split()
                        ke[0][i].append([l[0],l[1]])
        for e0 in root.findall("./calculation/eigenvalues/array/set/set[@comment='spin 2']"):
            ke.append([])
            for i,e1 in enumerate(e0.findall("set")):
                if "kpoint" in e1.attrib.get("comment"):
                    ke[1].append([])
                    for e2 in e1.findall("r"):
                        #print(e2.text)
                        l = e2.text.split()
                        ke[1][i].append([l[0],l[1]])

        for i in range(len(ke)):
            for j in range(len(ke[i])):
                for k in range(len(ke[i][j])):
                    #print(i,j,k,ke[i][j][k])
                    pass

    mag = None
    f0 = open(os.path.dirname(f)+"/OUTCAR")
    for l in f0:
        if l.find("number of electron")>=0 and l.find("magnetization")>=0:
            l0 = l.split()
            if len(l0)>5:
                mag = float(l0[5])
    f0.close()
    #print("mag:",mag)

    # update parameter
    #doc2 = ElementTree.parse(f)
    ##doc2 = ElementTree.parse("datasheet.xml")
    ##root2 = doc2.getroot()

    e0 = root2.find("program/name")
    e0.text = program
    e0 = root2.find("program/version")
    e0.text = version

    #for e in root2.findall("calculation[@name="+name+"]/properties")):
    for e0 in root2.findall("calculation[@name='"+name+"']"):
        #print("task:",name)
        # structure
        for e1 in e0.findall("structure"):
            if e1.attrib.get("name") == "initial":                    
                e2 = ElementTree.SubElement(e1,"lattice")
                e2.set("type","primitive")
                e3 = ElementTree.SubElement(e2,"a1")
                e3.text = cell0[0][0]+" "+cell0[0][1]+" "+cell0[0][2]
                e3 = ElementTree.SubElement(e2,"a2")
                e3.text = cell0[1][0]+" "+cell0[1][1]+" "+cell0[1][2]
                e3 = ElementTree.SubElement(e2,"a3")
                e3.text = cell0[2][0]+" "+cell0[2][1]+" "+cell0[2][2]
                e2 = ElementTree.SubElement(e1,"atom")
                for i in range(len(atom0)):
                    e3 = ElementTree.SubElement(e2,"i")
                    e3.set("name",atom[i][0])
                    e3.text = atom0[i][0]+" "+atom0[i][1]+" "+atom0[i][2]
                    
            if e1.attrib.get("name") == "final":
                e2 = ElementTree.SubElement(e1,"lattice")
                e2.set("type","primitive")
                e3 = ElementTree.SubElement(e2,"a1")
                e3.text = cell1[0][0]+" "+cell1[0][1]+" "+cell1[0][2]
                e3 = ElementTree.SubElement(e2,"a2")
                e3.text = cell1[1][0]+" "+cell1[1][1]+" "+cell1[1][2]
                e3 = ElementTree.SubElement(e2,"a3")
                e3.text = cell1[2][0]+" "+cell1[2][1]+" "+cell1[2][2]
                e2 = ElementTree.SubElement(e1,"atom")
                for i in range(len(atom1)):
                    e3 = ElementTree.SubElement(e2,"i")
                    e3.set("name",atom[i][0])
                    e3.text = atom1[i][0]+" "+atom1[i][1]+" "+atom1[i][2]

        # kpoints
        for e1 in e0.findall("kpoints"):
            if e1.attrib.get("type") == "kmesh":
                e1.text = str(nkp[0])+" "+str(nkp[1])+" "+str(nkp[2])
            elif e1.attrib.get("type") == "kpath":
                e1.clear()
                e1.set("type","kpath")
                #for k in self.kpath:
                for k in kp:
                    e2 = ElementTree.SubElement(e1,"i")
                    ##if len(k)>4:
                    ##    e2.set("name",str(k[4]))
                    e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                    #s = "%10.5f %10.5f %10.5f" % (k[0],k[1],k[2])
                    ##s = "%15.10f %15.10f %15.10f" % (k[0],k[1],k[2])
                    ##e2.text = s.strip()

        # parameter
        for e1 in e0.findall("parameters"):
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name","ispin")
            e2.text = str(ispin)
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name","nbands")
            e2.text = str(nbands)
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name","nelect")
            e2.text = str(nelect)
            e2 = ElementTree.SubElement(e1,"i")
            e2.set("name","nkpoints")
            e2.text = str(nkpoints)
            e2 = ElementTree.SubElement(e1,"kpoints")
            for i in range(len(kp)):
                e3 = ElementTree.SubElement(e2,"i")
                e3.text = kp[i][0]+" "+kp[i][1]+" "+kp[i][2]

        # property
        for e1 in e0.findall("properties"):
            if etotal is not None:
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name","etotal")
                e2.set("unit","eV")
                e2.text = etotal
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name","etotal_per_atom")
                e2.set("unit","eV/atom")
                e2.text = str(float(etotal)/natom)
            if efermi is not None:
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name","efermi")
                e2.set("unit","eV")
                e2.text = efermi
            if mag is not None:
                e2 = ElementTree.SubElement(e1,"i")
                e2.set("name","magnetization")
                e2.set("unit","uB")
                e2.text = str(mag)
            if dos is not None:
                e2 = ElementTree.SubElement(e1,"dos")
                for i in range(len(dos)):
                    e3 = ElementTree.SubElement(e2,"spin")
                    e3.set("spin",str(i+1))
                    for j in range(len(dos[i])):
                        e4 = ElementTree.SubElement(e3,"i")
                        e4.text = dos[i][j][0]+" "+dos[i][j][1]+" "+dos[i][j][2]
            if pdos is not None:
                e2 = ElementTree.SubElement(e1,"pdos")
                #e3 = ElementTree.SubElement(e2,"i")
                #e3.set("name","value")
                #s = ""
                #for i in range(len(pdoscol)):
                #    s += str(pdoscol[i]) + " "
                #e3.text = s.strip()
                for i in range(len(pdos)):
                    e3 = ElementTree.SubElement(e2,"atom")
                    e3.set("name",atom[i][0])
                    e3.set("atom",str(i+1))

                    e4 = ElementTree.SubElement(e3,"i")
                    e4.set("name","value")
                    s = ""
                    for j in range(len(pdoscol)):
                        s += str(pdoscol[j]) + " "
                    e4.text = s.strip()

                    for j in range(len(pdos[i])):
                        e4 = ElementTree.SubElement(e3,"spin")
                        e4.set("spin",str(j+1))
                        for k in range(len(pdos[i][j])):
                            e5 = ElementTree.SubElement(e4,"i")
                            s = ""
                            for l in range(len(pdos[i][j][k])):
                                s += pdos[i][j][k][l] + " "
                            e5.text = s.strip()
            if ke is not None:
                e2 = ElementTree.SubElement(e1,"band")
                for i in range(len(ke)):
                    e3 = ElementTree.SubElement(e2,"spin")
                    e3.set("spin",str(i+1))
                    for j in range(len(ke[i])):
                        e4 = ElementTree.SubElement(e3,"kpoint")
                        kp = j+1
                        e4.set("kpoint",str(kp))
                        for k in range(len(ke[i][j])):
                            e5 = ElementTree.SubElement(e4,"i")
                            #s = "%10.5f %10.5f" % (ke[i][j][k][0],ke[i][j][k][1])
                            #e5.text = s
                            e5.text = str(ke[i][j][k][0])+" "+str(ke[i][j][k][1])

        # step
        for e1 in e0.findall("step"):
            if step is not None:
                for i in range(len(step)):
                    e2 = ElementTree.SubElement(e1,"sc")
                    e2.set("step",str(i+1))
                    for j in range(len(step[i])):
                        e3 = ElementTree.SubElement(e2,"i")
                        e3.set("name","etotal")
                        e3.set("unit","eV")
                        e3.text = str(step[i][j])

    ##tree = ElementTree.tostring(root2,'utf-8')        
    ##f0 = open("datasheet.xml","w")
    ###minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
    ##minidom.parseString(tree).writexml(f0)
    ##f0.close()

    ##if name == "scf":
    ##    self.charge("CHGCAR","charge.cube")

def data():
    arg = argparse.ArgumentParser(description="ac-calc")
    arg.add_argument("-p","--program",action="store",dest="program",default="vasp")
    arg.add_argument("-m","--mat",action="store",dest="mat")
    arg.add_argument("-c","--cif",action="store",dest="cif")
    arg.add_argument("-o","--output",action="store",dest="output")

    arg.add_argument("-s","--scenario",action="store",dest="scenario")

    arg.add_argument("--regid",action="store",dest="regid")
    arg.add_argument("--reguid",action="store",dest="reguid")
    arg.add_argument("--template",action="store",dest="matrecordtmpl")

    arg.add_argument("--noarchive", dest="noarchive", action="store_true")

    args = arg.parse_args()
    
    print()
    print("Template Oriented Atomic Simulation Toolkit (TOAST)")
    print()
    print("[dataset creation]")
    print("program:",args.program)
    print("mat:",args.mat)
    print("cif:",args.cif)
    print("scenario:",args.scenario)
    print("output:",args.output)
    print()

    da = Data()
    da.program = args.program
    da.matid = args.mat
    da.scenario = args.scenario
    da.matrecordtmpl = args.matrecordtmpl
    da.regid = args.regid
    da.reguid = args.reguid

    if da.program == "vasp":
        pass
    else:
        print("program:",da.program,"not support (current version)...")
        exit(1)

    da.datasheet("datasheet.xml")

    da.read_calculations("datasheet.xml")
    #da.read_calculations("calculations.xml")
    da.datafiles(".")
    #da.metadata("metadata2.xml")

    da.materialrecord("materialRecord.xml")

    out = args.output
    if out is None:
        out = da.material_id+".zip"

    archive = True
    if args.output is None and args.noarchive is True:
        archive = False        

    if archive:
        materialrecord_dataset("materialRecord.xml",out)

    # temporary
    os.system("xmllint --format datasheet.xml > tmp01.xml")
    os.system("mv tmp01.xml datasheet.xml")    
    os.system("xmllint --format materialRecord.xml > tmp02.xml")
    os.system("mv tmp02.xml materialRecord.xml")
    #os.system("xmllint --format metadata2.xml > tmp03.xml")
    #os.system("mv tmp03.xml metadata2.xml")

if __name__ == "__main__":
    data()
