import os
import shutil
import math
import numpy
import collections
import sys
from xml.etree import ElementTree
from xml.dom import minidom

#from calculator import *
from calculation.calculator import *

class Vasp(Calculator):

    def __init__(self):
        super(Vasp,self).__init__()
        #print("vasp")
        self.scenario = "vasp.para.cfg"
        self.nkp_max = 48

    def calc(self):
        print("calculation (vasp)...",self.matid)

    def inputs(self, task):
        self.potcar("../POTCAR")

        [kx,ky,kz] = self.nkp
        self.kpoints(kx,ky,kz) 

        task.input["system"]=self.matid
        npar_conf = task.input.get("npar", "")
        ncore_conf = task.input.get("ncore", "")
        if (not npar_conf) and (not ncore_conf):
            task.input["npar"]=int(math.sqrt(float(self.nproc)))
        task.input["ispin"]=self.spin

        if not task.input.get("nbands", ""):
            lsorb = task.input.get("lsorbit", ".FALSE.")
            lnoncollinear = task.input.get("lnoncollinear", ".FALSE.")
            if (lsorb.upper() == ".TRUE.") or (lnoncollinear.upper() == ".TRUE."):
                task.input["nbands"]=self.nbands * 2
            else:
                task.input["nbands"]=self.nbands

        if task.name == "check_spin":
            pass
        elif task.name == "opt1":
            pass
        elif task.name == "opt2":
            pass
        elif task.name == "scf":
            pass
        elif task.name == "dos":
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                self.kpoints(kx*2,ky*2,kz*2)
            else:
                self.kpoints(kx,ky,kz)
            task.input["emin"]=self.emin
            task.input["emax"]=self.emax
        elif task.name == "band":
            self.kpoints_band()
        elif task.name == "fermisurf":
            if kx*2<=self.nkp_max and ky*2<=self.nkp_max and kz*2<=self.nkp_max:
                self.kpoints(kx*2,ky*2,kz*2)
            else:
                self.kpoints(kx,ky,kz)
        elif task.name == "emass":
            self.kpoints_band()

        self.incar(task.input)

    def potentials(self,ele):
        f = open("POTCAR","w")
        for e in ele:
            pp = self.pp[e]
            print("pp:",e,pp)
            f0 = open(self.ppdir+"/"+pp+"/POTCAR")
            for l in f0:
                f.write(l)
            f0.close()
        f.close()
            
    def incar(self,incar):
        f = open("INCAR","w")
        #for k,v in incar.iteritems():
        for k,v in incar.items():
            if (v is not None) and  (len(str(v))>0):     # key without value is set to None (Python2.7)
                f.write("%s = %s\n" % (k.upper(), v))    # Keywords in uppercase.
        f.close()

    def poscar(self):
        f = open("POSCAR","w")
        f.write("%s\n" % (self.matid))
        f.write("%10.5f\n" % (1.0))
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[0][0],self.cell[0][1],self.cell[0][2]))
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[1][0],self.cell[1][1],self.cell[1][2]))
        #f.write("%10.5f %10.5f %10.5f\n" % (self.cell[2][0],self.cell[2][1],self.cell[2][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[0][0],self.cell[0][1],self.cell[0][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[1][0],self.cell[1][1],self.cell[1][2]))
        f.write("%15.10f %15.10f %15.10f\n" % (self.cell[2][0],self.cell[2][1],self.cell[2][2]))
        for e in self.ele:
            f.write("%s " % e)
        f.write("\n")
        for a in self.natom_ele:
            f.write("%s " % a)
        f.write("\n")
        f.write("Direct\n")
        for a in self.atom:
            #f.write("%10.5f %10.5f %10.5f\n" % (a[1],a[2],a[3]))                    
            f.write("%15.10f %15.10f %15.10f\n" % (a[1],a[2],a[3]))                    

    def potcar(self,f):
        shutil.copy2(f,".")

    def kpoints(self,kx,ky,kz):
        f = open("KPOINTS","w")
        f.write("K-Points\n")
        f.write("0\n")
        f.write("Gamma\n")
        f.write("%d %d %d\n" % (kx,ky,kz))
        f.write("0. 0. 0.\n")
        f.close()

    def kpoints_band(self,coordinate="reciprocal"):
        f = open("KPOINTS","w")
        f.write("band kpath\n")
        f.write("%d\n" % len(self.kpath))
        f.write(coordinate+"\n")
        for k in self.kpath:
            if(len(k)==4):
                #f.write("%10.5f %10.5f %10.5f %5.1f\n" % (k[0],k[1],k[2],k[3]))
                f.write("%15.10f %15.10f %15.10f %5.1f\n" % (k[0],k[1],k[2],k[3]))
            elif(len(k)>4):
                #f.write("%10.5f %10.5f %10.5f %5.1f ! %s\n" % (k[0],k[1],k[2],k[3],k[4]))
                f.write("%15.10f %15.10f %15.10f %5.1f ! %s\n" % (k[0],k[1],k[2],k[3],k[4]))

    def nbands(self,ele,a):
        natom = 0
        za = 0
        for i in range(len(a)):
            natom += int(a[i])
        PP = open("POTCAR")
        zv = []
        for l in PP:
            if l.find("ZVAL")>=0:
                z = l.split()
                zval = z[5]
                zv.append(zval)
        for i in range(len(a)):
            #print(zv[i],a[i])
            za += float(zv[i]) * float(a[i])
        nelect = za
        print("nelect:",nelect)
        nbands = int(nelect/2)+natom*2
        nbands = int((nelect)*0.6+0.5)+max(4,natom*2)
        if(nbands%2 == 1): nbands+=1
        print("nbands:",nbands)
        return nbands

    def structure(self, name, d):
        if name == "cif":
            self.poscar()
        else:
            # d = self.calc[name]["_calc_dir"]
            print("copy ../"+d+"/CONTCAR POSCAR")
            shutil.copy2("../"+d+"/CONTCAR","POSCAR")

            f = open("POSCAR")
            cell = []
            atom = []
            for i,l in enumerate(f):
                l = l.strip()
                l0 = l.split()
                if i==1:
                    scale = l0[0]
                if i==2:
                    #cell.append(map(float,l0))
                    cell.append(list(map(float,l0)))
                if i==3:
                    #cell.append(map(float,l0))
                    cell.append(list(map(float,l0)))
                if i==4:
                    #cell.append(map(float,l0))
                    cell.append(list(map(float,l0)))
                if i==5:
                    ele = l0
                if i==6:
                    #natom_ele = map(int,l0)
                    natom_ele = list(map(int,l0))
                if i==7:
                    for j,e in enumerate(ele):
                        na = natom_ele[j]
                        for k in range(na):
                            #l = f.next()
                            l = next(f)
                            l0 = l.split()
                            #l0 = map(float,l0)
                            l0 = list(map(float,l0))
                            atom.append([l0[0],l0[1],l0[2]])
            f.close()
            self.cell0 = self.cell
            self.cell = cell
            self.cell1 = self.cell
            if len(self.atom) == len(atom):
                # update atom coordiates
                self.atom_old = self.atom
                for i in range(len(atom)):
                    self.atom[i][1:4] = atom[i][0:3]

    def mag(self, d):
        mag = 0.0
        # d = self.calc["check_spin"]["_calc_dir"]
        f = open("../"+d+"/OUTCAR")
        for l in f:
            if l.find("number of electron")>=0 and l.find("magnetization")>=0:
                l0 = l.split()
                mag = l0[5]
        mag = float(mag)
        #print("mag:",mag)
        return mag

    def efermi(self, d):
        efermi = 0.0
        # d = self.calc["scf"]["_calc_dir"]
        f = open("../"+d+"/OUTCAR")
        for l in f:
            if l.find("E-fermi")>=0:
                l0 = l.split()
                efermi = l0[2]
        efermi = float(efermi)
        print("efermi:",efermi)
        return efermi

    def results(self, task, f):
        name = task.name
        doc = ElementTree.parse("vasprun.xml")
        root = doc.getroot()

        etotal = None
        efermi = None
        for e0 in root.findall("./calculation/energy/i[@name='e_fr_energy']"):
            etotal = e0.text.strip()
        for e0 in root.findall("./calculation/dos/i[@name='efermi']"):
            efermi = e0.text.strip()

        print("etotal:",etotal)
        print("efermi:",efermi)

        cell0 = []
        cell1 = []
        atom0 = []
        atom1 = []
        #for e0 in root.findall(".//structure[@name='initialpos']/crystal/varray[@name='basis']/v"):
        for e0 in root.findall("structure"):
            if e0.attrib.get("name") == "initialpos":
                for e1 in e0.findall("crystal/varray"):
                    if e1.attrib.get("name") == "basis":
                        for e2 in e1.findall("v"):
                            d = e2.text.split()
                            #print(d[0],d[1],d[2])
                            cell0.append([d[0],d[1],d[2]])
                for e1 in e0.findall("varray"):
                    if e1.attrib.get("name") == "positions":
                        for e2 in e1.findall("v"):
                            d = e2.text.split()
                            #print(d[0],d[1],d[2])
                            atom0.append([d[0],d[1],d[2]])
            if e0.attrib.get("name") == "finalpos":
                for e1 in e0.findall("crystal/varray"):
                    if e1.attrib.get("name") == "basis":
                        for e2 in e1.findall("v"):
                            d = e2.text.split()
                            #print(d[0],d[1],d[2])
                            cell1.append([d[0],d[1],d[2]])
                for e1 in e0.findall("varray"):
                    if e1.attrib.get("name") == "positions":
                        for e2 in e1.findall("v"):
                            d = e2.text.split()
                            #print(d[0],d[1],d[2])
                            atom1.append([d[0],d[1],d[2]])

        #for i in range(len(cell0)):
        #    print(i,cell0[i][0],cell0[i][1],cell0[i][2])
        #for i in range(len(atom0)):
        #    print(i,atom0[i][0],atom0[i][1],atom0[i][2])
        #for i in range(len(cell1)):
        #    print(i,cell1[i][0],cell1[i][1],cell1[i][2])
        #for i in range(len(atom1)):
        #    print(i,atom1[i][0],atom1[i][1],atom1[i][2])
        
        nkp = None
        ispin = None
        nbands = None
        nelect = None
        nkpoints = None
        for e0 in root.findall("kpoints/generation/v"):
            if e0.attrib.get("name") == "divisions":
                d = e0.text.split()
                #print(d)
                nkp = [d[0],d[1],d[2]]
        #for e0 in root.findall("//modeling/parameters/separator/i[@name='ISPIN']"):
        for e0 in root.findall("./parameters/separator/separator/i"):
            if e0.attrib.get("name") == "ISPIN":
                ispin = e0.text.strip()
        #for e0 in root.findall("//modeling/parameters/separator/i[@name='NBANDS']"):
        for e0 in root.findall("./parameters/separator/i"):
            if e0.attrib.get("name") == "NBANDS":
                nbands = e0.text.strip()
        #for e0 in root.findall("//modeling/parameters/separator/i[@name='NELECT']"):
        for e0 in root.findall("./parameters/separator/i"):
            if e0.attrib.get("name") == "NELECT":
                nelect = e0.text.strip()
        #for e0 in root.findall(//modeling/kpoints/varray[@name='kpointlist']/v"):
        for e0 in root.findall("./kpoints/varray"):
            if e0.attrib.get("name") == "kpointlist":
                nkpoints = len(e0.findall("v"))

        if nkp is not None: 
            print("nkp:",nkp[0],nkp[1],nkp[2])
        print("ispin:",ispin)
        print("nbands:",nbands)
        print("nelect:",nelect)
        print("nkpoints:",nkpoints)

        #for e0 in root.findall("//modeling/kpoints/varray[@name='kpointlist']/v"):
        kp = []
        for e0 in root.findall("./kpoints/varray"):
            if e0.attrib.get("name") == "kpointlist":
                for i,e1 in enumerate(e0.findall("v")):                        
                    l = e1.text.split()
                    kp.append([l[0],l[1],l[2]])

        for i in range(len(kp)):
            #print(i,kp[i][0],kp[i][1],kp[i][2])
            pass

        # opt, scf
        step = []
        for i,e0 in enumerate(root.findall("calculation")):
            step.append([])
            for e1 in e0.findall("energy/i"):
                if e1.attrib.get("name") == "e_fr_energy":
                    pass
            for e1 in e0.findall("scstep/energy/i"):
                if e1.attrib.get("name") == "e_fr_energy":
                    step[i].append(e1.text.strip())

        for i in range(len(step)):
            for j in range(len(step[i])):
                #print(i,j,opt[i][j])
                pass

        dos = None
        pdos = None
        ke = None
        if name == "dos":
            ndos = None
            #//modeling/parameters/separator[@name='dos']/i[@name='NEDOS']
            for e0 in root.findall("./parameters/separator/"):
                if e0.attrib.get("name") == "dos":
                    for e1 in e0.findall("i"):
                        if e1.attrib.get("name") == "NEDOS":
                            ndos = e1.text
                            print("ndos:",ndos)

            #tdos
            dos = []
            #for e0 in root.findall("//modeling/calculation/dos/total/array/set/set[@comment='spin 1']/r/"):
            for e0 in root.findall("./calculation/dos/total/array/set/set"):
                #print(e0.attrib.get("comment"))
                if e0.attrib.get("comment") == "spin 1":                    
                    dos.append([])
                    for e1 in e0.findall("r"):
                        l = e1.text.split()
                        #print(l[0],l[1],l[2])
                        dos[0].append([l[0],l[1],l[2]])
                if e0.attrib.get("comment") == "spin 2":
                    dos.append([])
                    for e1 in e0.findall("r"):
                        l = e1.text.split()
                        #print(l[0],l[1],l[2])
                        dos[1].append([l[0],l[1],l[2]])

            for i in range(len(dos)):
                for j in range(len(dos[i])):
                    #print(i,j,dos[i][j][0],dos[i][j][1],dos[i][j][2])
                    pass

            #pdos
            natom = self.natom            
            pdoscol = []
            for e0 in root.findall("./calculation/dos/partial/array/field"):
                pdoscol.append(e0.text.strip())
            pdos = []  # atom,spin
            #for e in root.findall("//modeling/calculation/dos/partial/array/set/set[contains(@comment,'ion')]"):
            for i,e0 in enumerate(root.findall("./calculation/dos/partial/array/set/set")):
                if "ion" in e0.attrib.get("comment"):
                    pdos.append([])
                    for e1 in e0.findall("set"):
                        if e1.attrib.get("comment") == "spin 1":
                            pdos[i].append([])
                            for e2 in e1.findall("r"):
                                #print(e2.text)
                                l0 = e2.text.split()
                                pdos[i][0].append(l0)                                
                        if e1.attrib.get("comment") == "spin 2":
                            pdos[i].append([])
                            for e2 in e1.findall("r"):
                                #print(e3.text)
                                l0 = e2.text.split()
                                pdos[i][1].append(l0)

            for i in range(len(pdos)):
                for j in range(len(pdos[i])):
                    for k in range(len(pdos[i][j])):
                        #print(i,j,k,pdos[i][j][k])
                        pass
                        
        if name == "dos" or name == "band" or name == "fermisurf" or name == "emass":
            ke = []  # spin,kp,band,[ene,occ]
            #for e0 in root.findall("//modeling/calculation/eigenvalues/array/set/set[@comment='spin 1']/set"):
            for e0 in root.findall("./calculation/eigenvalues/array/set/set"):
                if e0.attrib.get("comment") == "spin 1":
                    ke.append([])
                    for i,e1 in enumerate(e0.findall("set")):
                        if "kpoint" in e1.attrib.get("comment"):
                            ke[0].append([])
                            for e2 in e1.findall("r"):
                                #print(e2.text)
                                l = e2.text.split()
                                ke[0][i].append([l[0],l[1]])
                if e0.attrib.get("comment") == "spin 2":
                    ke.append([])
                    for i,e1 in enumerate(e0.findall("set")):
                        if "kpoint" in e1.attrib.get("comment"):
                            ke[1].append([])
                            for e2 in e1.findall("r"):
                                #print(e2.text)
                                l = e2.text.split()
                                ke[1][i].append([l[0],l[1]])

            for i in range(len(ke)):
                for j in range(len(ke[i])):
                    for k in range(len(ke[i][j])):
                        #print(i,j,k,ke[i][j][k])
                        pass

        mag = None
        f0 = open("OUTCAR")
        for l in f0:
            if l.find("number of electron")>=0 and l.find("magnetization")>=0:
                l0 = l.split()
                if len(l0)>5:
                    mag = float(l0[5])
        f0.close()
        print("mag:",mag)

        # update parameter
        doc2 = ElementTree.parse(f)
        root2 = doc2.getroot()
        #for e in root2.findall("calculation[@name="+name+"]/properties")):
        for e0 in root2.findall("calculation"):
            if e0.attrib.get("name") == name:
                # structure
                for e1 in e0.findall("structure"):
                    if e1.attrib.get("name") == "initial":
                        e2 = ElementTree.SubElement(e1,"lattice")
                        e2.set("type","primitive")
                        e3 = ElementTree.SubElement(e2,"a1")
                        e3.text = cell0[0][0]+" "+cell0[0][1]+" "+cell0[0][2]
                        e3 = ElementTree.SubElement(e2,"a2")
                        e3.text = cell0[1][0]+" "+cell0[1][1]+" "+cell0[1][2]
                        e3 = ElementTree.SubElement(e2,"a3")
                        e3.text = cell0[2][0]+" "+cell0[2][1]+" "+cell0[2][2]
                        e2 = ElementTree.SubElement(e1,"atom")
                        for i in range(len(atom0)):
                            e3 = ElementTree.SubElement(e2,"i")
                            e3.set("name",self.atom[i][0])
                            e3.text = atom0[i][0]+" "+atom0[i][1]+" "+atom0[i][2]
                        
                    if e1.attrib.get("name") == "final":
                        e2 = ElementTree.SubElement(e1,"lattice")
                        e2.set("type","primitive")
                        e3 = ElementTree.SubElement(e2,"a1")
                        e3.text = cell1[0][0]+" "+cell1[0][1]+" "+cell1[0][2]
                        e3 = ElementTree.SubElement(e2,"a2")
                        e3.text = cell1[1][0]+" "+cell1[1][1]+" "+cell1[1][2]
                        e3 = ElementTree.SubElement(e2,"a3")
                        e3.text = cell1[2][0]+" "+cell1[2][1]+" "+cell1[2][2]
                        e2 = ElementTree.SubElement(e1,"atom")
                        for i in range(len(atom1)):
                            e3 = ElementTree.SubElement(e2,"i")
                            e3.set("name",self.atom[i][0])
                            e3.text = atom1[i][0]+" "+atom1[i][1]+" "+atom1[i][2]

                # kpoints
                for e1 in e0.findall("kpoints"):
                    if e1.attrib.get("type") == "kmesh":
                        e1.text = str(nkp[0])+" "+str(nkp[1])+" "+str(nkp[2])
                    elif e1.attrib.get("type") == "kpath":
                        e1.clear()
                        e1.set("type","kpath")
                        for k in self.kpath:
                            e2 = ElementTree.SubElement(e1,"i")
                            if len(k)>4:
                                e2.set("name",str(k[4]))
                            #e2.text = str(k[0])+" "+str(k[1])+" "+str(k[2])
                            #s = "%10.5f %10.5f %10.5f" % (k[0],k[1],k[2])
                            s = "%15.10f %15.10f %15.10f" % (k[0],k[1],k[2])
                            e2.text = s.strip()

                # parameter
                for e1 in e0.findall("parameters"):
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","ispin")
                    e2.text = str(ispin)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nbands")
                    e2.text = str(nbands)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nelect")
                    e2.text = str(nelect)
                    e2 = ElementTree.SubElement(e1,"i")
                    e2.set("name","nkpoints")
                    e2.text = str(nkpoints)
                    e2 = ElementTree.SubElement(e1,"kpoints")
                    for i in range(len(kp)):
                        e3 = ElementTree.SubElement(e2,"i")
                        e3.text = kp[i][0]+" "+kp[i][1]+" "+kp[i][2]

                # property
                for e1 in e0.findall("properties"):
                    if etotal is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal")
                        e2.set("unit","eV")
                        e2.text = etotal
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","etotal_per_atom")
                        e2.set("unit","eV/atom")
                        e2.text = str(float(etotal)/self.natom)
                    if efermi is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","efermi")
                        e2.set("unit","eV")
                        e2.text = efermi
                    if mag is not None:
                        e2 = ElementTree.SubElement(e1,"i")
                        e2.set("name","magnetization")
                        e2.set("unit","uB")
                        e2.text = str(mag)
                    if dos is not None:
                        e2 = ElementTree.SubElement(e1,"dos")
                        for i in range(len(dos)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(dos[i])):
                                e4 = ElementTree.SubElement(e3,"i")
                                e4.text = dos[i][j][0]+" "+dos[i][j][1]+" "+dos[i][j][2]
                    if pdos is not None:
                        e2 = ElementTree.SubElement(e1,"pdos")
                        #e3 = ElementTree.SubElement(e2,"i")
                        #e3.set("name","value")
                        #s = ""
                        #for i in range(len(pdoscol)):
                        #    s += str(pdoscol[i]) + " "
                        #e3.text = s.strip()
                        for i in range(len(pdos)):
                            e3 = ElementTree.SubElement(e2,"atom")
                            e3.set("name",self.atom[i][0])
                            e3.set("atom",str(i+1))

                            e4 = ElementTree.SubElement(e3,"i")
                            e4.set("name","value")
                            s = ""
                            for j in range(len(pdoscol)):
                                s += str(pdoscol[j]) + " "
                            e4.text = s.strip()

                            for j in range(len(pdos[i])):
                                e4 = ElementTree.SubElement(e3,"spin")
                                e4.set("spin",str(j+1))
                                for k in range(len(pdos[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    s = ""
                                    for l in range(len(pdos[i][j][k])):
                                        s += pdos[i][j][k][l] + " "
                                    e5.text = s.strip()
                    if ke is not None:
                        e2 = ElementTree.SubElement(e1,"band")
                        for i in range(len(ke)):
                            e3 = ElementTree.SubElement(e2,"spin")
                            e3.set("spin",str(i+1))
                            for j in range(len(ke[i])):
                                e4 = ElementTree.SubElement(e3,"kpoint")
                                kp = j+1
                                e4.set("kpoint",str(kp))
                                for k in range(len(ke[i][j])):
                                    e5 = ElementTree.SubElement(e4,"i")
                                    #s = "%10.5f %10.5f" % (ke[i][j][k][0],ke[i][j][k][1])
                                    #e5.text = s
                                    e5.text = str(ke[i][j][k][0])+" "+str(ke[i][j][k][1])

                # step
                for e1 in e0.findall("step"):
                    if step is not None:
                        for i in range(len(step)):
                            e2 = ElementTree.SubElement(e1,"sc")
                            e2.set("step",str(i+1))
                            for j in range(len(step[i])):
                                e3 = ElementTree.SubElement(e2,"i")
                                e3.set("name","etotal")
                                e3.set("unit","eV")
                                e3.text = str(step[i][j])

        tree = ElementTree.tostring(root2,'utf-8')        
        f0 = open(f,"w")
        #minidom.parseString(tree).writexml(f0,indent='',addindent='  ',newl='\n',encoding="utf-8")
        minidom.parseString(tree).writexml(f0)
        f0.close()

        if name == "scf":
            self.charge("CHGCAR","charge.cube")

    def charge(self,f1,f2):
        dataline = False
        f = open(f1)
        c1 = []
        c2 = []
        cell = []
        atom = []
        atomno = []
        ngrid = None
        for i,l in enumerate(f):
            l = l.strip()
            l0 = l.split()
            # poscar
            if i==0:
                system = l0[0]
            if i==1:
                scale = l0[0]
            if i==2:
                #cell.append(map(float,l0))
                cell.append(list(map(float,l0)))
            if i==3:
                #cell.append(map(float,l0))
                cell.append(list(map(float,l0)))
            if i==4:
                #cell.append(map(float,l0))
                cell.append(list(map(float,l0)))
            if i==5:
                ele = l0
            if i==6:
                #natom_ele = map(int,l0)
                natom_ele = list(map(int,l0))
                natom=sum(natom_ele)
            if i==7:
                for j,e in enumerate(ele):
                    na = natom_ele[j]
                    for k in range(na):
                        #l = f.next()
                        l = next(f)
                        l0 = l.split()
                        #l0 = map(float,l0)
                        l0 = list(map(float,l0))
                        atom.append([l0[0],l0[1],l0[2]])
                        atomno.append(int(self.elements[e][0]))
            # data
            if i>0 and l=="":
                dataline = True
            if dataline:
                if len(l0)==3:
                    if not ngrid:
                        #ngrid = map(int,l0)
                        ngrid = list(map(int,l0))
                        ndata = ngrid[0]*ngrid[1]*ngrid[2]
                        nl = int(ndata/5)
                        if int(ndata%5)>0:
                            nl = nl+1
                        for j in range(nl):
                            #l = f.next()
                            l = next(f)
                            l0 = l.split()
                            c1.extend(l0)
                    else:
                        for j in range(nl):
                            #l = f.next()
                            l = next(f)
                            l0 = l.split()
                            c2.extend(l0)
            if "augmentation occupancies" in l:
                nocc = int(l0[-1])
                nl2 = int(nocc/5)
                if int(nocc%5)>0:
                    nl2 = nl2+1
                for j in range(nl2):
                    #f.next()                
                    next(f)                

        f.close()

        # bohr/hartree
        cell = numpy.dot(cell,1.889725992)
        for i in range(len(atom)):
            atom[i] = numpy.dot(cell,atom[i])
        a = cell[0]
        b = cell[1]
        c = cell[2]
        d = numpy.cross(b,c)
        vol = numpy.dot(a,d)/(1.889725992**3)

        #print(system)
        #print(scale)
        #print(cell)
        #print(vol)
        #print(ele)
        #print(natom_ele)
        #print(atom)
        #print(ngrid,ndata)
        #print(len(c1))
        #print(len(c2))
        #print(atomno)

        f = open(f2,"w")
        f.write(system+"\n")
        f.write("charge density (total)\n")
        f.write("%5d %12.6f %12.6f %12.6f\n" % (natom,0,0,0))
        for i in range(3):
            f.write("%5d %12.6f %12.6f %12.6f\n" % (ngrid[i],cell[i][0]/ngrid[i],cell[i][1]/ngrid[i],cell[i][2]/ngrid[i]))
        for i,a in enumerate(atom):
            f.write("%5d %12.6f %12.6f %12.6f %12.6f\n" % (atomno[i],atomno[i],a[0],a[1],a[2]))
        i0=0
        for i in range(ngrid[0]):
            for j in range(ngrid[1]):
                for k in range(ngrid[2]):
                    i1 = k*ngrid[0]*ngrid[1] + j*ngrid[0] + i
                    c0 = float(c1[i1])*0.036749309/vol
                    f.write("%13.5E" % c0)
                    if i0%6==5:
                        f.write("\n")
                    i0=i0+1
        f.close()

    def calstat(self):
        stat = []

        f0 = "calculations.xml"

        if not os.path.exists(f0):
            return stat

        doc = ElementTree.parse(f0)
        root = doc.getroot()

        matid = None
        chemform = None
        crysys = None
        spg = None
        spgno = None
        cif = None
        calc = []
        for e in root.findall("material"):
            for e1 in e.getchildren():
                if e1.tag == "material_id":
                    matid = e1.text
                if e1.tag == "chemical_formula":
                    chemform = e1.text
                if e1.tag == "crystal_system":
                    crysys = e1.text
                if e1.tag == "spacegroup":
                    spg = e1.text
                if e1.tag == "spacegroup_no":
                    spgno = e1.text
                if e1.tag == "cif":
                    cif = e1.text
        for e in root.findall("calculation"):
            calc.append(e.attrib.get("name"))

        stat.append([matid,chemform,crysys,spg,spgno,cif])
        stat.append(calc)
        stat.append([])
        #stat.append([0]*len(calc))

        for name in calc:
            for e in root.findall("calculation"):
                if e.attrib.get("name")==name:
                    s=0
                    for e1 in e.findall("inputs/i"):
                        if e1.attrib.get("name")=="_calc_dir":
                            d = e1.text
                            f1 = d+"/vasprun.xml"
                            f2 = d+"/OUTCAR"
                            if os.path.exists(f1):
                                try:
                                    doc2 = ElementTree.parse(f1)
                                    s = 0
                                except:
                                    s = 1
                            elif os.path.exists(f2):
                                f = open(f2)
                                for l in f:
                                    if l.find("General timing and accounting informations for this job:")>=0:
                                        s = 0
                                        break
                                    else:
                                        s = 1
                            else:
                                s = -1

            stat[2].append(s)
            
        return stat


