
import os
import sys

class Cif:
    data = None
    chemical_formula_sum = None
    space_group_crystal_system = None
    symmetry_space_group_name_HM = None
    symmetry_Int_Tables_number = 0
    cell_length_a = 0.0
    cell_length_b = 0.0
    cell_length_c = 0.0
    cell_angle_alpha = 0.0
    cell_angle_beta = 0.0
    cell_angle_gamma = 0.0
    cell_volume = 0.0
    cell_formula_units_Z = 0
    publ_section_title = None
    journal_coden_ASTM = None
    journal_name_full = None
    journal_year = None
    journal_volume = None
    journal_page_first = None
    journal_page_last = None
    journal_language = None

    class Atom:
        atom_site_label = None
        atom_site_fract_x = 0.0
        atom_site_fract_y = 0.0
        atom_site_fract_y = 0.0
        atom_site_occupancy = 0.0
        atom_site_symmetry_multiplicity = 0
        atom_site_Wyckoff_symbol = None
        atom_site_U_iso_or_equiv = None
        atom_site_type_symbol = None

    atom_site = None

    def __init__(self):
        pass

    def read(self,file):
        print("cif read:",file)
        f = open(file)
        lines = f.readlines()
        f.close()

        loop = []
        symmop = []
        atomsite = []
        publ = []

        index = 0
        while index < len(lines):
            l = lines[index]
            l = l.strip()

            if l.find("data_")>=0:
                l0 = l.strip()
                self.data = l0.lstrip("data_")
            if l.find("_chemical_formula_sum")>=0:
                l0 = l.split("_chemical_formula_sum")
                self.chemical_formula_sum = l0[1].strip()
            if l.find("_space_group_crystal_system")>=0:
                l0 = l.split()
                self.space_group_crystal_system = l0[1]
            if l.find("_symmetry_space_group_name_H-M")>=0:
                l0 = l.split("_symmetry_space_group_name_H-M")
                self.symmetry_space_group_name_HM = l0[1].strip()
            if l.find("_symmetry_Int_Tables_number")>=0:
                l0 = l.split()
                self.symmetry_Int_Tables_number = int(l0[1])

            if l.find("_cell_length_a")>=0:
                l0 = l.split()
                self.cell_length_a = float(l0[1])
            if l.find("_cell_length_b")>=0:
                l0 = l.split()
                self.cell_length_b = float(l0[1])
            if l.find("_cell_length_c")>=0:
                l0 = l.split()
                self.cell_length_c = float(l0[1])
            if l.find("_cell_angle_alpha")>=0:
                l0 = l.split()
                self.cell_angle_alpha = float(l0[1])
            if l.find("_cell_angle_beta")>=0:
                l0 = l.split()
                self.cell_angle_beta = float(l0[1])
            if l.find("_cell_angle_gamma")>=0:
                l0 = l.split()
                self.cell_angle_gamma = float(l0[1])
            if l.find("_cell_volume")>=0:
                l0 = l.split()
                self.cell_volume = float(l0[1])
            if l.find("_cell_formula_units_Z")>=0:
                l0 = l.split()
                self.cell_formula_units_Z = int(l0[1])

            if l.find("_publ_section_title")>=0:
                l0 = l.split("_publ_section_title")
                self.publ_section_title = l0[1].strip()
            if l.find("_journal_coden_ASTM")>=0:
                l0 = l.split()
                self.journal_coden_ASTM = l0[1]
            if l.find("_journal_name_full")>=0:
                l0 = l.split("_journal_name_full")
                self.journal_name_full = l0[1].strip()
            if l.find("_journal_year")>=0:
                l0 = l.split()
                self.journal_year = l0[1]
            if l.find("_journal_volume")>=0:
                l0 = l.split()
                self.journal_volume = l0[1]
            if l.find("_journal_page_first")>=0:
                l0 = l.split()
                self.journal_page_first = l0[1]
            if l.find("_journal_page_last")>=0:
                l0 = l.split()
                self.journal_page_last = l0[1]
            if l.find("_journal_language")>=0:
                l0 = l.split()
                self.journal_language = l0[1]

            if l.find("loop_")>=0:
                loop.append([])

                index += 1

                symmflag = False
                atomflag = False
                publflag = False

                while index < len(lines):
                    l = lines[index]
                    l = l.strip()

                    if l.strip().startswith("_"):
                        loop[-1].append(l.strip())
                    else:
                        break
                    index += 1

                while index < len(lines):
                    l = lines[index]
                    l = l.strip()

                    for l0 in loop[-1]:
                        if "_symmetry_equiv_pos_as_xyz" in l0 or "_space_group_symop_operation_xyz" in l0:
                            symmflag = True
                        if "_atom_site_label" in l0 and "_geom_bond" not in l0:
                            atomflag = True
                        if "_publ_author" in l0:
                            publflag = True
                            
                    if len(l)>0 and l[0]=="#":
                        index += 1
                        continue
                    if len(l)==0 or l=="loop_" or l[0]=="_":
                        index -= 1
                        break

                    #print(l.strip())
                    l0 = l.split()
                    if symmflag:
                        symmop.append(l0)
                    elif atomflag:
                        atomsite.append(l0)
                    elif publflag:
                        publ.append([l.strip()])
                    else:
                        pass

                    index += 1

            index += 1

        #print("loop")
        #for l in loop:
        #    print(l)

        self.atom = []
        self.symmop = []
        self.publ = []

        for l in loop:
            if "symmetry_equiv" in l[0]:
                for e in symmop:
                    self.symmop.append(e[l.index("_symmetry_equiv_pos_as_xyz")])
            if "atom_site" in l[0] and "atom_site_aniso" not in l[0] and "geom_bond" not in l[0]:
                for e in atomsite:
                    a = Cif.Atom()
                    a.atom_site_label = e[l.index("_atom_site_label")]
                    a.atom_site_fract_x = float(e[l.index("_atom_site_fract_x")])
                    a.atom_site_fract_y = float(e[l.index("_atom_site_fract_y")])
                    a.atom_site_fract_z = float(e[l.index("_atom_site_fract_z")])
                    if "_atom_site_occupancy" in l:
                        a.atom_site_occupancy = float(e[l.index("_atom_site_occupancy")])
                    if "_atom_site_symmetry_multiplicity" in l:
                        a._atom_site_symmetry_multiplicity = int(e[l.index("_atom_site_symmetry_multiplicity")])
                    if "_atom_site_Wyckoff_symbol" in l:
                        a.atom_site_Wyckoff_symbol = e[l.index("_atom_site_Wyckoff_symbol")]
                    if "_atom_site_U_iso_or_equiv" in l:
                        a.atom_site_U_iso_or_equiv = e[l.index("_atom_site_U_iso_or_equiv")]
                    a.atom_site_type_symbol = e[l.index("_atom_site_type_symbol")]
                    self.atom.append(a)
            if "publ_author" in l[0]:
                for e in publ:
                    self.publ.append(e[l.index("_publ_author_name")])

    def write(self,file):
        #f = open(file)
        f = sys.stdout

        f.write("data_%s\n" % self.data)
        f.write("\n")

        f.write("_chemical_formula_sum %s\n" % self.chemical_formula_sum)
        f.write("_space_group_crystal_system %s\n" % self.space_group_crystal_system)
        f.write("_symmetry_space_group_name_H-M %s\n" % self.symmetry_space_group_name_HM)
        f.write("_symmetry_Int_Tables_number %d\n" % self.symmetry_Int_Tables_number)
        f.write("\n")

        f.write("_cell_length_a %f\n" % self.cell_length_a)
        f.write("_cell_length_b %f\n" % self.cell_length_b)
        f.write("_cell_length_c %f\n" % self.cell_length_c)
        f.write("_cell_angle_alpha %f\n" % self.cell_angle_alpha)
        f.write("_cell_angle_beta %f\n" % self.cell_angle_beta)
        f.write("_cell_angle_gamma %f\n" % self.cell_angle_gamma)
        f.write("_cell_volume %f\n" % self.cell_volume)
        f.write("_cell_formula_units_Z %d\n" % self.cell_formula_units_Z)
        f.write("\n")

        f.write("loop_\n")
        f.write("_symmetry_equiv_pos_site_id\n")
        f.write("_symmetry_equiv_pos_as_xyz\n")
        for i,o in enumerate(self.symmop):
            f.write("%d %s\n" % (i+1,o))
        f.write("\n")

        f.write("loop_\n")
        f.write("_atom_site_label\n")
        f.write("_atom_site_fract_x\n")
        f.write("_atom_site_fract_y\n")
        f.write("_atom_site_fract_z\n")
        f.write("_atom_site_occupancy\n")
        f.write("_atom_site_symmetry_multiplicity\n")
        f.write("_atom_site_Wyckoff_symbol\n")
        f.write("_atom_site_U_iso_or_equiv\n")
        f.write("_atom_site_type_symbol\n")
        for a in self.atom:
            f.write("%s %f %f %f %f %d %s %s %s\n" \
                        % (a.atom_site_label,a.atom_site_fract_x,a.atom_site_fract_y,a.atom_site_fract_z, \
                               a.atom_site_occupancy,a.atom_site_symmetry_multiplicity, \
                               a.atom_site_Wyckoff_symbol,a.atom_site_U_iso_or_equiv, \
                               a.atom_site_type_symbol))
        f.write("\n")

        f.write("_publ_section_title %s\n" % self.publ_section_title)
        f.write("_journal_coden_ASTM %s\n" % self.journal_coden_ASTM)
        f.write("_journal_name_full %s\n" % self.journal_name_full)
        f.write("_journal_year %s\n" % self.journal_year)
        f.write("_journal_volume %s\n" % self.journal_volume)
        f.write("_journal_page_first %s\n" % self.journal_page_first)
        f.write("_journal_page_last %s\n" % self.journal_page_last)
        f.write("_journal_language %s\n" % self.journal_language)
        f.write("\n")

        f.write("loop_\n")
        f.write("_publ_author_name\n")
        for a in self.publ:
            f.write("%s\n" % a)
        f.write("\n")

        #f.close()

if __name__ == "__main__":
    cif = Cif()
    cif.read(sys.argv[1])
    cif.write("tmp.cif")
