
import numpy
import spglib

def standardize_cell(cry,tol=1.0e-5):

    primcell = numpy.array(cry.unit_prim,float)
    primatom = []  # pos
    for a in cry.vatom_prim:
        primatom.append([a.position[0],a.position[1],a.position[2]])
    primnum = []
    for i in range(len(cry.velement_size)):
        for j in range(cry.velement_size[i]):
            primnum.append(i)
    pcell = (primcell,primatom,primnum)
    print_cell(pcell)

    scell = spglib.standardize_cell(pcell)
    dataset = spglib.get_symmetry_dataset(scell, tol)
    symmetry = spglib.get_symmetry(scell)
    print_cell(scell)
    return scell,dataset,symmetry
    
def csa_check(cry,tol=1.0e-5):
    print("[csa check]")
    spgnum_cif = cry.symmetry_Int_Tables_number

    #tol = 1.0e-5    

    print( "[conventional cell (csa)]")

    convcell = numpy.array(cry.unit_conv,float)
    convatom = []  # pos
    for a in cry.vatom_conv:
        convatom.append([a.position[0],a.position[1],a.position[2]])
    convnum = []
    for i in range(len(cry.velement_size_conv)):
        for j in range(cry.velement_size_conv[i]):
            convnum.append(i)

    ccell = (convcell,convatom,convnum)
    print_cell(ccell)

    dataset = spglib.get_symmetry_dataset(ccell, tol)
    spgnum_ccell = dataset["number"]
    hallnum_ccell = dataset["hall_number"]
    print_dataset(dataset)

    #symmetry = spglib.get_symmetry(ccell)
    #print(symmetry)

    # temporary
    cry.symmetry_space_group_name_hall = dataset["hall"]

    print("[primitive cell (csa)]")

    primcell = numpy.array(cry.unit_prim,float)
    primatom = []  # pos
    for a in cry.vatom_prim:
        primatom.append([a.position[0],a.position[1],a.position[2]])
    primnum = []
    for i in range(len(cry.velement_size)):
        for j in range(cry.velement_size[i]):
            primnum.append(i)

    pcell = (primcell,primatom,primnum)
    print_cell(pcell)

    dataset = spglib.get_symmetry_dataset(pcell, tol)    
    spgnum_pcell = dataset["number"]
    hallnum_pcell = dataset["hall_number"]
    print_dataset(dataset)
    
    print("[conventional cell (csa pcell std)]")

    scell = spglib.standardize_cell(pcell)
    print_cell(scell)
    dataset = spglib.get_symmetry_dataset(scell, tol)    
    spgnum_ccell_pcell_std = dataset["number"]
    hallnum_ccell_pcell_std = dataset["hall_number"]
    print_dataset(dataset)

    #
    print("[standardize conventional cell]")
    scell = spglib.standardize_cell(ccell)
    print_cell(scell)
    dataset = spglib.get_symmetry_dataset(scell, tol)
    print_dataset(dataset)
    print("[standardize primitive cell]")
    primscell,primsatom,primsnum = spglib.find_primitive(scell, tol)
    pscell = (primscell,primsatom,primsnum)
    print_cell(pscell)

    print("# check crystal structure...")
    print("%-25s %10s %10s" % ("","spg num","hall num"))
    print("%-25s %10d %10s" % ("cell orginal cif",spgnum_cif,"-"))
    print("%-25s %10d %10d" % ("ccell csa",spgnum_ccell,hallnum_ccell))
    print("%-25s %10d %10d" % ("pcell csa",spgnum_pcell,hallnum_pcell))
    print("%-25s %10d %10d" % ("ccell csa pcell+std",spgnum_ccell_pcell_std,hallnum_ccell_pcell_std))

    b = True
    if spgnum_cif != spgnum_ccell:
        b = False
        print("wraning: conventional cell... NG")
    if spgnum_cif != spgnum_pcell:
        b = False
        print("wraning: primitive cell... NG")
    if b == True:
        print("csa... OK")

    return b,hallnum_ccell,hallnum_pcell

def print_cell(cell):
    lattice, positions, numbers = cell
    print("lattice vectors")
    for vec in lattice:
        #print("{:15.9f} {:15.9f} {:15.9f}".format(vec[0], vec[1], vec[2]))
        print("%15.9f %15.9f %15.9f" % (vec[0], vec[1], vec[2]))
    print("atomic postions and type")
    for coor, atype in zip(positions, numbers):
        #print("{:4d} {:15.9f} {:15.9f} {:15.9f}".format(atype, coor[0], coor[1], coor[2]))
        print("%4d %15.9f %15.9f %15.9f" % (atype, coor[0], coor[1], coor[2]))    

def print_dataset(dataset):
    print("spg_dataset")
    print("  number",dataset["number"])
    print("  international",dataset["international"])
    print("  hall_symbol",dataset["hall"])
    print("  hall_number",dataset["hall_number"])
    print("  choice",dataset["choice"])
    print("  pointgroup",dataset["pointgroup"])
    #print(dataset["wyckoffs"])
    #print(dataset["equivalent_atoms"])
    print("  trans_matrix\n",dataset["transformation_matrix"])
    print("  original_shift\n",dataset["origin_shift"])
    #print(dataset["rotations"])
    #print(dataset["translations"])
    #print(dataset["std_lattice"])
    #print(dataset["std_positions"])
    #print(dataset["std_types"])
    hall_number = dataset["hall_number"]
    sgtype = spglib.get_spacegroup_type(hall_number)
    print_spgtype(sgtype)

def print_spgtype(sgtype):
    print("spg_spgtype")
    print("  number",sgtype["number"])
    print("  international_short",sgtype["international_short"])
    print("  international_full",sgtype["international_full"])
    print("  international",sgtype["international"])
    print("  hall_sysmbol",sgtype["hall_symbol"])
    print("  choice",sgtype["choice"])
    print("  schoenflies",sgtype["schoenflies"])
    print("  pointgroup",sgtype["pointgroup_schoenflies"])
    print("  pointgroup_international",sgtype["pointgroup_international"])
    print("  arithmetic_crystal_class_number",sgtype["arithmetic_crystal_class_number"])
    print("  arithmetic_crystal_class_symbol",sgtype["arithmetic_crystal_class_symbol"])
