import numpy

def celllength(cell):
    ''' calculate lattice constants from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return a,b,c
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la = numpy.sqrt(numpy.dot(a,a))
    lb = numpy.sqrt(numpy.dot(b,b))
    lc = numpy.sqrt(numpy.dot(c,c))
    #print("cell_length:",la,lb,lc)
    return la,lb,lc

def cellangle(cell):
    ''' calculate lattice angles (in degree) from primitive vectors
        in
           cell[0], cell[1], cell[2]: primitive vectors
        return alpha,beta,gamma
    '''
    a = cell[0]
    b = cell[1]
    c = cell[2]
    la,lb,lc = celllength(cell)
    #aa = math.acos(numpy.dot(b,c)/(lb*lc)) * 180.0 / math.pi
    #ab = math.acos(numpy.dot(c,a)/(lc*la)) * 180.0 / math.pi
    #ac = math.acos(numpy.dot(a,b)/(la*lb)) * 180.0 / math.pi
    aa = math.degrees(math.acos(numpy.dot(b,c)/(lb*lc)))
    ab = math.degrees(math.acos(numpy.dot(c,a)/(lc*la)))
    ac = math.degrees(math.acos(numpy.dot(a,b)/(la*lb)))
    #print("cell_angle:",aa,ab,ac)
    return aa,ab,ac

def cellrecip(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    rcell = numpy.zeros((3,3))
    v = cellvolume(cell)
    d = numpy.cross(b,c)
    rcell[0] = d / v
    d = numpy.cross(c,a)
    rcell[1] = d / v
    d = numpy.cross(a,b)
    rcell[2] = d / v
    #print("recip_cell:")
    #for i in xrange(3):
    #    print(rcell[i][0],rcell[i][1],rcell[i][2])
    return rcell

def cellvolume(cell):
    a = cell[0]
    b = cell[1]
    c = cell[2]
    d = numpy.cross(b,c)
    v = numpy.dot(a,d)
    #print("volume:",v)
    return v
